#include <stdio.h>
#include <stdlib.h>

#include "rob.h"


extern char *optarg;                    /* getopt(3) control vars */



#define ARGUMENTS "r:s"                 /* command-arguments */

#define FRAME_NAME "frame.pgm"          /* filename for frame-saving */


/* grab a b/w-Frame and save it(only possible in conjuction
 * with the Simator-library
 */
void grab_frame(int robot)
{
    static unsigned char frame[CAMERA_DEF_WIDTH][CAMERA_DEF_HEIGHT];

    if (rob_camera_frame((unsigned char *)frame, robot, 0,
			 CAMERA_DEF_WIDTH, CAMERA_DEF_HEIGHT, 0)==RobOk) {

	printf("*** sucessfuly grabed a frame.\n");
	printf("*** save frame to file '%s'\n", FRAME_NAME);
	rob_save_frame(FRAME_NAME, (unsigned char*)frame, 
			   CAMERA_DEF_WIDTH, CAMERA_DEF_HEIGHT, 0);
    }
    else {
	printf("*** error. no line to the camera.\n");
    }
    fflush(stdout);
}    


/* drive to robot 1 meter forward and 1 meter back
 */
void drive(int robot)
{
    char reply[ROBOMSGLENGTH+1];
	  
    printf("*** driving 1m forward an back with breaks of 5 secs\n");
    fflush(stdout);
    sleep(5);
    rob_command(robot, "MOTV ON", reply);
    sleep(5);
    rob_command(robot, "MOTV LA v=1000 d=1000", reply);
    sleep(5);
    grab_frame(robot);
    rob_command(robot, "MOTV LA v=-1000 d=1000", reply);
    sleep(5);
    rob_command(robot, "MOTV OFF", reply);
    sleep(5);
    printf("*** done\n");
    fflush(stdout);
}


int main(int argc, char* argv[])
{
    int c;
    int robot;
    char *robname = "porthos";
    int mode = RobViaServer;
    
    /* DRI & Simulator -conform argument-parsing
     */
    while ((c = getopt(argc, argv, ARGUMENTS)) != -1)
	switch (c) {
	case 'r':
	    robname = optarg;
	    break;
	case 's':
	    mode = RobViaSimulator;
	    break;
	case 'h':
	case '?':
	    printf("usage: %s [-s] [-r <robotname>]\n",
		   argv[0]);
	    printf("description: drives a robot one metre forward and back\n");
	    exit(0);
	    break;
	default:
	    break;
	}
    
    printf("*** Example-Robot-Application ***\n");
    printf("*** using %s-mode\n", 
	   (mode == RobViaSimulator) ? "Simulation" : "Server");
    
    fflush(stdout);
    
    rob_start(mode);
    
    robot = rob_init(robname);
    if (robot > 0) {
	printf("*** connecting to robot '%s'\n", robname);
	fflush(stdout);
	
	drive(robot);
	
	printf("*** close connection to robot '%s'\n", robname);
	rob_exit(robot);
    }
    else
	printf("*** ERROR: no connection to robot '%s'\n", robname);
    
    rob_halt();
    
    fflush(stdout);

    exit(0);
}
