/**************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: clipping.h
 *  TYPE		: module definition
 *  Time-stamp: <95/02/13 17:04:55 stolz>
 **************************************************************************
 *  MODULE              : clipping
 *
 *  FUNCTION
 *      This module provide clipping-functions.
 *
 *
 *  NOTE
 *
 *  HISTORY
 *  15-May-94   Creation date.
 *
 **************************************************************************/
#ifndef CLIPPING_H
#define CLIPPING_H

#ifndef VECTOR_H
#include "vector.h"
#endif

int clip_points(vector3 *points, vector3 start, vector3 line,
		      int xyz_select, double min, double max);
/****************************************************************************
 * FUNCTION
 *	Clips the line (start->+line) in the x or y or z-coordinates at
 *	the values min and max. The clipped points are added to the
 *      vector-array points. Only clipped point which are not identicall
 *	to the start or end of the line are returned!
 *
 * PREREQUISITES
 *	min < max
 *      0 <= xyz_select <= 2  (x=0, y=1, z=2)
 *
 *	start is the starting point of the line.
 *	the vector line is the index of the line, so the end-point
 *	is start+line!
 *
 * PARAMETERS
 *	points		: array to store the computed clipping points
 *      start, line     : Line from point start to point start+line
 *	xyz_select	: select index 0..2 of the vector3.
 *	min, max	: clipping-borders in the above index.
 *
 *
 * RESULTS
 *	The clipped points are stored in the points-array. The order
 *	of saving is from lines-start to line-end.
 *      The number of clipped points is returned.
 *	->   0 <= return value <= 2
 *
 */

#endif
