/********************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: com_scanner.c
 *  TYPE		: module implementation
 *  Time-stamp: <95/02/13 17:04:53 stolz>
 ********************************************************************************
 *  MODULE              : com_scanner
 *  IMPORT		: basic
 *
 *  NOTE
 *      The module specification and definition is given in com_scanner.h!
 *
 *  HISTORY
 *  24-Mar-94   Creation date
 *  10-Jun-94   Implement get_bitfield() for us-commands.
 *
 ********************************************************************************/

#include <math.h>
#include <ctype.h>
#include <stdlib.h>

#include "com_scanner.h"
#include "basic.h"


int get_long(char **string, long *lp) /* returns TRUE if an error occurs */
/**************************************************************************
 */
{
  char *cp = *string;
  long l = 0;
  long minus = 0;
  int dr = 0;   /* digits read */

  while (isspace(*cp)) cp++;

  while (*cp == '-') {
    cp++;
    minus = 1 - minus;
  }

  while (isspace(*cp)) cp++;

  if ((*cp) == '$' || ((*cp=='0') && ((cp[1]=='x')||(cp[1]=='X')))) {
    if (*cp == '$') cp++; else cp+=2;

    /* Hexadezimalzahl einlesen */
    while (isxdigit(*cp)) {
      l = (l<<4) + (isdigit(*cp) ? (*cp-'0') : (10+toupper(*cp)-'A'));
      cp++;
      dr++;
    }
  }
  else {
    /* Dezimalzahl einlesen */
    while (isdigit(*cp)) {
      l = l * 10 + (*(cp++) - '0');
      dr++;
    }
  }

  *string = cp;
  *lp = (minus ? -l : l);
  return (dr==0);
}


long get_word(char **string, int len)
/**************************************************************************
 */
{
    char *cp = *string;
    long word = 0;
    int i = 0;

    if (*cp=='\n') return word;

    while (isspace(*cp))
	cp++;

    while ((isalpha(*cp)) && i < len) {
	word = (word<<8) | toupper(*cp);
	cp++;
	i++;
    }
    while (isalpha(*cp))
	cp++;

    if ((*cp == '=' || *cp == '+' || *cp == '-') && (i < len)) {
        word = (word<<8) | *cp;
        cp++;
        i++;
    }

    *string = cp;
    return word;
}


char *get_value(long *d, char **args, long *opt,
		long valopt, long valpos, long valneg,
		long max_value, long min_value, long default_value,
		char *errormsg, char *invalidmsg)
/**************************************************************************
 */
{
    long l;

    if (*opt == 0) *opt = get_word(args ,4);

    if ((valopt!=WORD_EMPTY) &&  (valopt == *opt)) {
	if (get_long(args, &l)) return invalidmsg;
	*d = l;
	if (l<0) l = -l;
	if (not_in_range(min_value, l, max_value))
	    return errormsg;
	*opt = get_word(args, 4);
    }
    else if ((valpos!=WORD_EMPTY) && (valpos == *opt)) {
	*d = max_value;
	*opt = get_word(args, 4);
    }
    else if ((valneg!=WORD_EMPTY) && (valneg == *opt)) {
	*d = -max_value;
	*opt = get_word(args, 4);
    }
    else {
	*d = default_value;
    }
    
    return 0;
}



int get_comma(char **string)
/**************************************************************************
 */
{
    while (isspace(**string))
	*string = (*string)+1;

    if (**string == ',') {
	*string = (*string)+1;
	return FALSE;
    }
    else return TRUE;
}



int get_bitfield(unsigned long *field, char **string)
/**************************************************************************
 */
{
   int len = 0;
   unsigned long mask = 1;

   *field = 0;
   do {
	if (**string == '0') {}
	else if (**string == '1')  *field |= mask;
	else return len;

	mask <<=1;

	*string = (*string)+1;
        len++;
   } while (len<32);

   return len;
}


int get_bitfield2(unsigned long *field, char **string)
/**************************************************************************
 */
{
   int len = 0;
   unsigned long mask = 1;

   *field = 0;
   do {
	if (**string == '0') {}
	else if (**string == '1')  *field |= mask;
	else if (**string == '2')  *field |= mask<<1;
	else if (**string == '*')  *field |= mask | (mask<<1);
	else return len;

	mask <<= 2;

	*string = (*string)+1;
        len++;
   } while (len<32);

   return len;
}
