/********************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: com_scanner.h
 *  TYPE		: module definition
 *  Time-stamp: <95/02/13 16:52:44 stolz>
 ********************************************************************************
 *  MODULE              : com_scanner
 *
 *  FUNCTION
 *
 *	This module provides functions to handle the scanning
 *	of the Robuter-input.
 *
 *  NOTE
 *
 *  HISTORY
 *  20-Apr-94   Build this module out of utils.c.
 *  16-Jun-94   Add some us-specific error-code and functions.
 *
 ********************************************************************************/
#ifndef COM_SCANNER_H
#define COM_SCANNER_H

/* a empty string
 */
#define WORD_EMPTY	0x0	/* "" */


/* the Command-strings, returned by get_word(string,4).
 */
#define COM_MOTN	0x4D4F544E	/* 'MOTN' */
#define COM_MOTV	0x4D4F5456	/* 'MOTV' */
#define COM_MOTP	0x4D4F5450	/* 'MOTP' */
#define COM_ODOM	0x4F444F4D	/* 'ODOM' */
#define COM_ODOS	0x4F444F53	/* 'ODOS' */
#define COM_ODOF	0x4F444F46	/* 'ODOF' */
#define COM_READ	0x52454144	/* 'READ' */
#define COM_RNOD	0x524E4F44	/* 'RNOD' */
#define COM_SNEW	0x534E4557	/* 'SNEW' */
#define COM_SNOD	0x534E4F44	/* 'SNOD' */
#define COM_SSET	0x53534554	/* 'SSET' */
#define COM_BUMP	0x42554D50	/* 'BUMP' */
#define COM_RESET	0x52455345	/* 'RESE' */
#define COM_LCOM	0x4C434F4D	/* 'LCOM' */
#define COM_PROMPT	0x50525054	/* 'PRPT' */

#define COM_HALT	0x48414C54	/* 'HALT' */
#define COM_RESUME	0x52455355 	/* 'RESU' */


/* the option-strings, use get_word(string,2)
 */
#define OPT_ON		0x4f4e		/* 'ON' */
#define OPT_OFF		0x4f46		/* 'OF' */
#define OPT_FORWARD	0x464f	/* FO */
#define OPT_BACKWARD	0x4241	/* BA */
#define OPT_LEFT	0x4c45	/* LE */
#define OPT_RIGHT	0x5249	/* RI */
#define OPT_KILL	0x4b49	/* KI */
#define OPT_WAIT	0x5741	/* WA */
#define OPT_STATUS	0x5354	/* ST */
#define OPT_ADD		0x4144	/* AD */
#define OPT_LAUNCH	0x4c41	/* LA */
#define OPT_CHANGE	0x4348	/* CH */
#define OPT_RESET	0x5253	/* RS */
#define OPT_COMPACTST	0x534c	/* SL */
#define OPT_MOTIONQUEUE	0x4d53	/* MS */
#define OPT_QUEUESIZE	0x4e53	/* NS */
#define OPT_MOTIONPOS	0x4d50	/* MP */
#define OPT_LINVELLIMIT	0x564c	/* VL */
#define OPT_ANGVELLIMIT	0x574c	/* WL */
#define OPT_GETTOKEN	0x4754	/* GT */
#define OPT_DELTOKEN	0x4454	/* DT */
#define OPT_CLOTHID	0x434c	/* CL */
#define OPT_JOYSTICK	0x4a4b	/* JK */
#define OPT_S		0x53	/* S */
#define OPT_INFINITE    0x494e  /* IN */

/* the value-strings, use get_word(string,2)
 */
#define VAL_ANGLE	0x413d	/* A= */
#define VAL_DISTANCE	0x443d	/* D= */
#define VAL_VELOCITY	0x563d	/* V= */
#define VAL_ANGULAR_VEL	0x573d	/* W= */
#define VAL_TIME	0x543d	/* T= */
#define VAL_LIN_ACC	0x4c3d	/* L= */
#define VAL_ANG_ACC	0x4b3d	/* K= */
#define VAL_COORDINATES	0x433d	/* C= */
#define VAL_CONFIG	0x433d	/* C= */
#define VAL_R		0x523d	/* R= */
#define VAL_S		0x533d	/* S= */
#define VAL_N		0x4e3d	/* N= */
#define VAL_D		0x443d	/* D= */

/* special value strings, use get_word(string,2)
 */
#define VAL_POS_LIN_VEL 0x562b  /* V+ */
#define VAL_NEG_LIN_VEL 0x562d  /* V- */
#define VAL_POS_ANG_VEL 0x572b  /* W+ */
#define VAL_NEG_ANG_VEL 0x572d  /* W- */
#define VAL_POS_LIN_ACC	0x4c2b	/* L+ */
#define VAL_POS_ANG_ACC	0x4b2b	/* K+ */

/* errors-messages
 */
#define ERR_ILLEGAL_COMMAND	"*ILL"
#define ERR_INVALID_PARAMETER	"PARM"
#define ERR_SERVER_INACTIVE	"MOFF"
#define ERR_SERVER_ALLOCATED	"SERV"
#define ERR_ROBOT_IN_MOTION     "MOVE"
#define ERR_ILLEGAL_DISTANCE	"DIST"
#define ERR_ILLEGAL_ANGLE	"ANGL"
#define ERR_ILLEGAL_VELOCITY	"VELY"
#define ERR_OBSTACLE		"OBST"

/* MOTV specials */
#define ERR_LIN_VEL_RANGE       "LVEL"
#define ERR_ANG_VEL_RANGE       "AVEL"
#define ERR_ILLEGAL_LIN_ACC     "LACC"
#define ERR_ILLEGAL_ANG_ACC     "AACC"
#define ERR_ILLEGAL_TIME        "TIME"
#define ERR_QUEUE_FULL          "FUST"
#define ERR_QUEUE_EMPTY         "EMST"
#define ERR_RADIUS_RANGE        "RADI"
#define ERR_JOYSTICK            "JKON"
#define ERR_ILLEGAL_TOKEN       "TOKE"
#define ERR_NO_TOKEN            "NOTO"
#define ERR_NAME_EXISTS         "NAEX"

/* error of odometric commands
 */
#define ERR_ODOM_INACTIVE	"OOFF"

/* errors of us-commands
 */
#define ERR_INVALID_DEVICE_NR	"DEVI"
#define ERR_AUTO_MODE_ACTIVE	"RNOD"
#define ERR_REQ_MODE_ACTIVE     "READ"
#define ERR_NO_US_MODE_ACTIVE	"VOID"
#define ERR_IN_CONFIGURATION	"CONF"
#define ERR_INVALID_DESCRIPTOR	"DNUM"
#define ERR_INVALID_SENSOR_NR	"SNUM"
#define ERR_IN_SCAN_DIR		"SDIR"
#define ERR_NO_READ_REQUEST	"VOID"
#define ERR_READ_NOT_FINISHED	"READ"
#define ERR_REQ_MODE_INACTIVE   "NREQ"
#define ERR_ILLEGAL_NODE        "NODE"

#define ERR_HALT	        "HALT"


/* transformation of meter to millimeter, rad to grad and
 * back.
 */
#define DTOI_X1000(x) ((long)((x)*1000.0+0.5))
#define DTOI_RAD_TO_GRAD(x) ((long)((x)*180.0/PI+0.5))
#define ITOD_GRAD_TO_RAD(x) ((x)*PI/180.0)
#define ITOD_DIV1000(x) ((x)/1000.0)




long get_word(char **string, int len);
/**************************************************************************
 * NAME
 *	get_word(...)	read a single word from a string
 *
 * FUNCTION
 *	A single word consisting of a-z,A-Z is without prefixing
 *	white spaces readen from the string. The word in uppercase
 *	letters is packed to a single long-value by shifting into a
 *	zero-value from right to left. Therefor the maximum
 *	length is 4.
 *
 * PARAMETERS
 *      char **string		0-terminated string.
 *	int len                 significant len, should>0 and <5.
 *
 * RESULTS
 *	returns the packed word or 0=WORD_EMPTY if the string is empty.
 *      The Parameter *string point behind the scanned word.
 *
 * EXAMPLE
 *	char text[] = " hi hello world!";
 *	char *string = text[0];
 *      long x;
 *
 *	x = get_word(&string, 4);
 *	-- should return 0x4849="HI"
 *	-- string = &text[3]
 *
 *	x = get_word(&string, 3);
 *	-- should return 0x48454C="HEL"
 *	-- string = &text[9]
 *
 *	x = get_word(&string, 4);
 *	-- should return 0x574F524C="WORL"
 *	-- string = &text[15]
 *
 * SEE ALSO
 *      get_long(), get_value(), get_comma(),
 *	get_bitfield(), get_bitfield2()
 *
 **************************************************************************
 */


int get_long(char **string, long *lp);
/**************************************************************************
 * NAME
 *	get_long(...)	read a decimal or hex-number
 *
 * FUNCTION
 *	Read a signed decimal or hex-number(prefixed with $ or 0x or 0X)
 *	from the string and stores the value in *lp. Prefixed white spaces
 *	are skipped!
 *
 * PARAMETERS
 *      char **string		0-terminated string.
 *	long *lp                place to store the value
 *
 * RESULTS
 *	returns TRUE the string contains no number on the first place.
 *      else FALSE(0) for an successful read.
 *      The Parameter *string points then behind the scanned word.
 *	In *lp, the value can be found.
 *
 * EXAMPLE
 *	char text[] = " - 0x1Ca";
 *	char *string = &text[0];
 *      long x;
 *
 *	get_long(&string, 4);
 *	-- *lp = -548
 *	-- string = &text[8]
 *
 * NOTE
 *	The +-sign is not allowded!
 *
 * SEE ALSO
 *      get_word(), get_value(), get_comma(),
 *	get_bitfield(), get_bitfield2()
 *
 **************************************************************************
 */


char *get_value(long *d, char **args, long *opt,
		long valopt, long valpos, long valneg,
		long max_value, long min_value, long default_value,
		char *errormsg, char *invalidmsg);
/**************************************************************************
 * NAME
 *	get_value(...)	reads a value
 *
 * FUNCTION
 *	Scan the args-string for an option-value.
 *
 *	This can be a <valopt>-string followed by an long-value, or an
 *	option-string <valpos> or <valneg> for the positiv or negativ
 *	<default_value> without a following long-value-string.
 *
 *	Therefor this routine first scans a word if <opt>=0, or use the
 *	word in <*opt>. If this word is not a option-string of <valopt>
 *	<valpos> or <valneg> the error-message <invalidmsg> is returned.
 *
 *      If the scanned long-value is not in the range
 *	[<min_value>, <max_value>] the error <errormsg> is returned.
 *	A successful value-scan returns 0.
 *
 * PARAMETERS
 *	long *d			place to store scanned value.
 *	char **args		0-terminated string
 *	long *opt               optional option-word, use 0 to disable
 *	long valopt		Normal option-string for following value.
 *				use 0 to disable
 *	long valpos		+ option-string, use 0 to disable
 *	long valneg		- option-string, use 0 to disable
 *	long max_value		maximum and minimum valid values
 *	long min_value
 *	long default_value	default value for +/- option-string
 *	char *errormsg		error-message for an illegal value
 *	char *invalidmsg        error-message for an invalid option-string
 *
 * RESULTS
 *	returns a error-string or 0 for success.
 *      For an successful scan *d contains the scanned value.
 *	**args point behind the scanned string.
 *
 * EXAMPLE
 *	char text[] = " a=123 a+";
 *	char *string = &text[0];
 *      long value, word;
 *	char *err;
 *
 *	err = get_value(&value, string, 0, "A=", "A+", 0, 1000, -10, 10,
 *			"out of range", "missing option");
 *	-- result: value = 123, string = &text[6], err = 0
 *
 *      word = get_word(string, 4);
 *	err = get_value(&value, string, word, "A=", "A+", 0, 1000, -10, 10,
 *			"out of range", "missing option");
 *	-- result: value = 1000, string = &text[8], err = 0
 *
 *	err = get_value(&value, string, 0, "A=", "A+", 0, 1000, -10, 10,
 *			"out of range", "missing option");
 *	-- result: value = -, string = &text[8], err = "missing option"
 *
 * NOTE
 *	The + sign is not allowded for an value. The value can be in
 *	hex or decimal. White spaces can prefix the option-string and
 *	the scanned value.
 *
 * SEE ALSO
 *      get_word(), get_long(), get_comma(),
 *	get_bitfield(), get_bitfield2()
 *
 **************************************************************************
 */


int get_comma(char **string);  /* returns TRUE is an error occurs */
/**************************************************************************
 * NAME
 *	get_comma(...)		scans a ,
 *
 * FUNCTION
 *	Scans a comma, perheps prefixed by white space, from the string.
 *
 * PARAMETERS
 *      char **string		0-terminated string.
 *
 * RESULTS
 *	returns TRUE(1) the string contains no comma, else FALSE(0).
 *      The Parameter *string points then behind the scanned comma.
 *
 * EXAMPLE
 *	char text[] = " ,123";
 *      int i;
 *
 *	i = get_comma(&string);
 *	-- i = 0
 *	-- string = &text[2]
 *
 * SEE ALSO
 *      get_word(), get_value(), get_long(),
 *	get_bitfield(), get_bitfield2()
 *
 **************************************************************************
 */

int get_bitfield(unsigned long *field, char **string); /* returns len of field */
int get_bitfield2(unsigned long *field, char **string); /* returns len of field */
/**************************************************************************
 * NAME
 *	get_bitfield(...)	scans a array of 0|1-numbers
 *	get_bitfield2(...)	scans a array of 0|1|2|*-numbers
 *
 * FUNCTION
 *	get_bitfield scans a array of 0|1 numbers and stores the value
 *	as array of one bit/number.
 *
 *	get_bitfield2 scans a array of 0|1|2|*-numbers and stores the
 *	value with two bits per number.
 *
 *	The value is stored in reverse order with most significant number
 *	as last significant bit.
 *
 * PARAMETERS
 *      char **string		0-terminated string.
 *	long *lp                place to store the value
 *
 * RESULTS
 *	return the len of the scanned number-array.
 *      The Parameter *string points then behind the scanned word.
 *	*field contains the scanned array in reverse order.
 *
 * EXAMPLE
 *	char text[] = "1100 102*";
 *      unsigned long a, b;
 *      int len;
 *
 *	get_bitfield(&field,  &string);
 *	-- result: field=%0011, *string=text[4]
 *
 *	get_bitfield2(&field,  &string);
 *	-- result: field=%11100001, *string=text[9]
 *
 *
 * NOTE
 *	The *-Number is only allowded in the get_bitfield2() routine.
 *
 * SEE ALSO
 *      get_long(), get_word(), get_value(), get_comma()
 *
 **************************************************************************
 */


#endif
