/********************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: command.h
 *  TYPE		: module definition
 *  Time-stamp: <95/02/13 16:53:01 stolz>
 ********************************************************************************
 *  MODULE              : command
 *
 *  FUNCTION
 *      This module provides the interface to the command-interpreter
 *	of an Mobile Robot.
 *
 *  NOTE
 *
 *  HISTORY
 *  24-Mar-94   Creation date.
 *
 ********************************************************************************/
#ifndef COMMAND_H
#define COMMAND_H

#ifndef MOB_PRIVATE_H
#include "mob_private"
#endif


void mob_do_command(mob m, double t, char *com);
/**************************************************************************
 * NAME
 *	mob_do_command(...)	emulate Robuter-command
 *
 * FUNCTION
 *	Interpets the command-string and execute the command.
 *	The command execution is done in zero-time! The time-parameter
 *	t is necessary for internal computations.
 *
 * PARAMETERS
 *      m	Mobile Robuter
 *	t	Invokation time of the command.
 *	com	Command-String
 *
 * NOTE
 *	Outputs or Errors are passed via the mob_output()-routine.
 *
 * EXAMPLE
 *	mob_do_command(mymob, 0.0, "MOTV LA V+");
 *
 * SEE ALSO
 *      mob.c, mob.h, mob_output().
 *
 **************************************************************************
 */

#endif
