/**************************************************************************
 *  PROJECT   : DA mobile robot simulation
 *  AUTHOR(S) : Horst Stolz (HUS)
 *  FILE      : conmob.c
 *  TYPE      : module implementation
 *  Time-stamp: <94/11/04 14:54:54 stolz>
 **************************************************************************
 *  MODULE              : conmob
 *
 *  FUNCTION
 *      This main module is for testing the function of a mob.
 *      The stdin goes as command-string to the mob, the output
 * 	goes to stdout.
 *	Use ! to exit and %<v_low> <v_high> <w_low> <w_high>
 *	to set the velocity disturbance.
 *
 *  HISTORY
 *  23-Mar-94   Creation date
 *
 **************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "mob.h"
#include "rev.h"
#include "scheduler.h"



static timepoint mobinfo;
static double dt_mobinfo;

static void mob_info(scheduler *s, timepoint *tp, double t)
{
    mob m = tp->workspace;
    mob_state real, intern;

    mob_get_motion_state(m, &real, &intern);

    printf("t:%2.3f a:%f u:%2.3f v:%1.3f "
	   "w:%2.3f x:%2.3f y:%2.3f angle:%1.3f\n",
	   t, real.a, real.u,
	   real.v, real.w, real.x, real.y, real.angle);

    scheduler_point_add(s, tp, t + dt_mobinfo);
}


static void com_in(char *com, void *uservar)
{
    char *cp;
    mob m = (mob)uservar;
    static mob_disturbance vd;

    printf("ROBOT>");
    cp = gets(com); /* MAX_COMMAND_LENGTH bachten@!!! */

    if (cp == 0 || *cp == '!') exit(0);

    if (*cp == '%') {
	sscanf(&com[1], "%lf%lf%lf%lf", &(vd.v_low), &(vd.v_high),
	       &(vd.w_low), &(vd.w_high));
	printf("v_rand [%f-%f], w_rand [%f-%f]\n",
	       vd.v_low, vd.v_high, vd.w_low, vd.w_high);
	mob_set_vel_disturbance(m, vd);
    }
}

static void com_out(void *uservar, char *out)
{
    if (out) printf("OUTPUT>%s\n", out);
}

static void mobs()
{
    scheduler s;
    mob m;

    scheduler_new(&s, 0.0);
    
    m = mob_new(&s);
    mob_set_io(m, com_in, com_out, m);

    dt_mobinfo = 0.1;
    mobinfo.workspace = m;
    mobinfo.task = 0;
    mobinfo.function = mob_info;
    mobinfo.free = 0;
    scheduler_point_add(&s, &mobinfo, 0.0);


    scheduler_sync(&s, 1e6);
}


int main(int argc, char **argv)
{
    printf("CONMOBS\n"
	   "use ! to exit and %4x<float> to set the velocity-disturbance\n");
    printf("%s %s\n", project_id_string, project_version_string);

    mobs();

    return 0;
}


