/********************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: error.h
 *  TYPE		: module definition
 *  Time-stamp: <95/02/13 16:54:05 stolz>
 ********************************************************************************
 *  MODULE              : error
 *
 *  FUNCTION
 *	Provide routines for error-handling.
 *
 *	warning("err-text") prints a text on stderr with file-name and
 *	line-number.
 *
 *	error("err-text") print a text on stderr with file-name and
 *	line-number and exit with the EXIT_FAILURE-code (20).
 *
 *  EXAMPLE
 *	if (i<10)
 *	   warning("index out of range");
 *
 *	if (i==0)
 *	   error("can't alloc memory");
 *
 *  NOTE
 *
 *  HISTORY
 *  19-Apr-94   Creation date.
 *
 ********************************************************************************/
#ifndef ERROR_H
#define ERROR_H

#define error(x) error_in(__FILE__,__LINE__,x)
#define warning(x) warning_in(__FILE__,__LINE__,x)

void	error_in(char *file, int linenr, char *error_text);
void	warning_in(char *file, int linenr, char *warning_text);

#endif
