/**************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: xdraw.c
 *  TYPE		: module implementation
 *  Time-stamp: <95/02/13 16:54:25 stolz>
 **************************************************************************
 *  MODULE              : xdraw
 *
 *  IMPLEMENTATION
 *
 *  NOTE
 *      The module specification and definition is given in xdraw.h!
 *
 *  HISTORY
 *  19-May-94   Creation date
 *  06-Jun-94   Add draw2d_space(), draw_get_pick_parameters().
 *
 **************************************************************************/

/* 
        /usr/bin/cc -xansi        -nostdinc -I/usr/include -O  -MDupdate Makedepend -woff 515,608,658,799 -o 02.1.HelloCone_c 02.1.HelloCone_c.c     -nostdlib -L/lib -L/usr/lib  -lGLU -lGL -lX11 -lm -lInventorXt_c -lInventor_c -lInventorXt  
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <X11/Intrinsic.h>
#include <Inventor_c/SbColor.h>
#include <Inventor_c/SoOutput.h>
#include <Inventor_c/nodes/SoPerspectiveCamera.h>
#include <Inventor_c/nodes/SoMaterial.h>
#include <Inventor_c/nodes/SoDirectionalLight.h>
#include <Inventor_c/nodes/SoCone.h>
#include <Inventor_c/nodes/SoSeparator.h>
#include <Inventor_c/nodes/SoMaterial.h>
#include <Inventor_c/nodes/SoCube.h>
#include <Inventor_c/nodes/SoSphere.h>
#include <Inventor_c/nodes/SoTransform.h>
#include <Inventor_c/nodes/SoCoordinate3.h>
#include <Inventor_c/nodes/SoFaceSet.h>
#include <Inventor_c/nodes/SoShapeHints.h>
#include <Inventor_c/Xt/SoXt.h>
#include <Inventor_c/Xt/SoXtRenderArea.h>
#include <Inventor_c/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor_c/actions/SoWriteAction.h>


#include "iv_draw.h"
#include "mob_world_private.h"
#include "triangle.h"
#include "triangle_projection.h"
#include "error.h"
#include "basic.h"



static Widget       mainWindow;
static SoXtRenderArea *renderArea;
static SoSep  *root;
static SoNode *node;
static int viewer;

void iv_draw_obj(void *vp)
{
    mw_object obj = (mw_object)vp;
    SoSep *group;
    SoTransform  *transform = SoXfCreate();
    double len;
    SbVec3f axis;

    if (obj==0) return;

    group = SoSepCreate();


    SoSV3fSetX_Y_Z(&(transform->translation),
		   (float)(obj->position[0]),
		   (float)(obj->position[1]),
		   (float)(obj->position[2]));
/*
   What't the hell with this????

    SoSRotSetQ_U_A_T(&(transform->rotation),
			 -(float)(obj->orientation[0]),
			 (float)(obj->orientation[1]),
			 (float)(obj->orientation[2]),
			 (float)(obj->orientation[3]));
*/

    len = sqrt(obj->orientation[1] * obj->orientation[1] + obj->orientation[2] * obj->orientation[2] + obj->orientation[3] * obj->orientation[3]);
    if (len<1e-9) len=1.0;

    SbV3fSetX_Y_Z(axis, (float)(obj->orientation[1]/len),
		   (float)(obj->orientation[2]/len),
		   (float)(obj->orientation[3]/len));

    SoSRotSetAxisAngle(&(transform->rotation), &axis, (float)(-2.0 * acos(obj->orientation[0])));

    SoSepAddChild(group, (SoNode *) transform);

    if (obj->type != MWO_GROUP) {
	SoMtl        *material = SoMtlCreate();

	SoMColSetR_G_B( &(material->diffuseColor), obj->r, obj->g, obj->b );
	SoSepAddChild(group, (SoNode *) material );
    }

    switch (obj->type) {
    case MWO_GROUP:
	{
	    SoNode  *old_node = node;
	    SoSep *sep = SoSepCreate();
	    
	    SoSepAddChild(group, (SoNode *) sep);

	    node = (SoNode *)sep;
	    list_for_all(obj->obj.group.members, iv_draw_obj); 
	    node = old_node;
	}
	break;

    case MWO_CUBOID:
	{
	    SoCube *cube = SoCubeCreate();

	    SoSFloatSet(&(cube->width), (float)(obj->obj.cuboid.edges_length[0]));
	    SoSFloatSet(&(cube->height), (float)(obj->obj.cuboid.edges_length[1]));
	    SoSFloatSet(&(cube->depth), (float)(obj->obj.cuboid.edges_length[2]));

	    SoSepAddChild(group, (SoNode *)cube);
	}
	break;

    case MWO_RECTANGLE:
	{
	    SoShapeHints *shape_hints = SoShapeHintsCreate();
	    SoCoord3 *coord = SoCoord3Create();
	    SoFaceSet *faceset = SoFSetCreate();
	    SbVec3f poly[4];
	    long numvertices = 4;
	    float x_size = 0.5 * (float)(obj->obj.rectangle.x_size);
	    float y_size = 0.5 * (float)(obj->obj.rectangle.y_size);

/* old Inventor
	    SoSBitMaskSet(&(shape_hints->hints),
			  (SO_SHAPE_HINTS_SURFACE | SO_SHAPE_HINTS_ORDERED | 
			   SO_SHAPE_HINTS_CONVEX));
*/
	    SoSEnumSet(&(shape_hints->faceType), SO_SHAPE_HINTS_CONVEX);
	    SoSEnumSet(&(shape_hints->vertexOrdering), SO_SHAPE_HINTS_CLOCKWISE);

	    SoSepAddChild(group, (SoNode *)shape_hints);

	    SbV3fSetX_Y_Z(poly[0], -x_size, -y_size, 0.0);
	    SbV3fSetX_Y_Z(poly[1], -x_size,  y_size, 0.0);
	    SbV3fSetX_Y_Z(poly[2],  x_size,  y_size, 0.0);
	    SbV3fSetX_Y_Z(poly[3],  x_size, -y_size, 0.0);

	    SoMV3fSetN(&(coord->point), 0, 4, poly);
	    SoSepAddChild(group, (SoNode *)coord);

	    SoMLongSetN(&(faceset->numVertices), 0, 1, &numvertices);
	    SoSepAddChild(group, (SoNode *)faceset);
	}
	break;

    case MWO_TRIANGLE:
	{
	    SoShapeHints *shape_hints = SoShapeHintsCreate();
	    SoCoord3 *coord = SoCoord3Create();
	    SoFaceSet *faceset = SoFSetCreate();
	    SbVec3f poly[3];
	    long numvertices = 3;

/* old Inventor
	    SoSBitMaskSet(&(shape_hints->hints),
			  (SO_SHAPE_HINTS_SURFACE | SO_SHAPE_HINTS_ORDERED |
			   SO_SHAPE_HINTS_CONVEX));
*/
	    SoSepAddChild(group, (SoNode *)shape_hints);


	    SbV3fSetX_Y_Z(poly[0], (float)(obj->obj.triangle.edges[0][0]),
			  (float)(obj->obj.triangle.edges[0][1]),
			  (float)(obj->obj.triangle.edges[0][2]));
	    SbV3fSetX_Y_Z(poly[1], (float)(obj->obj.triangle.edges[1][0]),
			  (float)(obj->obj.triangle.edges[1][1]),
			  (float)(obj->obj.triangle.edges[1][2]));
	    SbV3fSetX_Y_Z(poly[2], (float)(obj->obj.triangle.edges[2][0]),
			  (float)(obj->obj.triangle.edges[2][1]),
			  (float)(obj->obj.triangle.edges[2][2]));

	    SoMV3fSetN(&(coord->point), 0, 3, poly);
	    SoSepAddChild(group, (SoNode *)coord);

	    SoMLongSetN(&(faceset->numVertices), 0, 1, &numvertices);
	    SoSepAddChild(group, (SoNode *)faceset);
	}
	break;

    default:
	error("iv_draw: iv_draw_obj() - unknown obstacle type");
	break;
    }

    SoSepAddChild(node, (SoNode *)group);
}



void iv_draw(mob_world mw)
{
    node = (SoNode *)root;

    list_for_all(mw->objects, iv_draw_obj);
}



void iv_draw_point(void *vp)
{
    double *point = (double *)vp;

#if 0
    return ;
#endif

    if (point) {
	SoSep *group = SoSepCreate();
	SoSphere *sphere = SoSphCreate();
	SoTransform  *transform = SoXfCreate();


        SoSV3fSetX_Y_Z(&(transform->translation),
		       (float)(point[0]),
		       (float)(point[1]),
		       (float)(point[2]));
	SoSepAddChild(group, (SoNode *) transform);


	SoSFloatSet(&(sphere->radius), 0.01);
/*
	SoSepAddChild(node, (SoNode *)sphere); 
*/
	SoSepAddChild(group, (SoNode *)sphere);
	SoSepAddChild(node, (SoNode *)group); 
    }
}


void iv_draw_simple_triangle(void *vp)
{
    vector3 *edges = (vector3 *)vp;
    SoSep *group;
    SoCoord3 *coord;
    SoFaceSet *faceset;
    SbVec3f poly[3];
    long numvertices = 3;

    if (vp==0) return;

    group = SoSepCreate();
    coord = SoCoord3Create();
    faceset = SoFSetCreate();

    SbV3fSetX_Y_Z(poly[0], (float)(edges[0][0]),
			  (float)(edges[0][1]),
			  (float)(edges[0][2]));
    SbV3fSetX_Y_Z(poly[1], (float)(edges[1][0]),
			  (float)(edges[1][1]),
			  (float)(edges[1][2]));
    SbV3fSetX_Y_Z(poly[2], (float)(edges[2][0]),
			  (float)(edges[2][1]),
			  (float)(edges[2][2]));

    SoMV3fSetN(&(coord->point), 0, 3, poly);
    SoSepAddChild(group, (SoNode *)coord);

    SoMLongSetN(&(faceset->numVertices), 0, 1, &numvertices);
    SoSepAddChild(group, (SoNode *)faceset);

    SoSepAddChild(node, (SoNode *)group);
}


void iv_draw_t3(void *vp)
{
    triangle3 t = (triangle3)vp;

    if (t) {
        iv_draw_simple_triangle(triangle3_abs_edges(t));
    }
}

void iv_draw_t3p(void *vp)
{
    t3p tp = (t3p)vp;
    vector3 *focus;

    if (tp) {
        focus = t3p_focus(tp);
        iv_draw_point(*focus);
        iv_draw_simple_triangle(triangle3_abs_edges(t3p_triangle3(tp)));
    }
}

void iv_draw_tp(void *vp)
{
    triangle_projection tp = (triangle_projection)vp;
    vector3 *focus;

    if (tp) {
        focus = tp_focus(tp);
        iv_draw_point(*focus);
        iv_draw_simple_triangle(tp_edges(tp));
    }
}


void iv_save_iv(char *name)
{
    SoWriteAction *wa;
    FILE *fh;

    fh = fopen(name, "w");
    if (fh) {
	wa = SoWriteActCreate();

	SoOutSetFilePtr(SoWriteActGetOut(wa), fh);
	SoWriteActApply(wa, (SoNode *)root);

	SoWriteActDelete(wa);
    }
    else {
	error("iv_draw: iv_save_iv() - can't open file for writing");
    }
}


void iv_draw_triangle(void *vp)
{
    triangle t = (triangle)vp;

    if (t) iv_draw_simple_triangle(triangle_edges(t));
}


void iv_color(float r, float g, float b)
{
    SoMtl        *material = SoMtlCreate();

    SoMColSetR_G_B( &(material->diffuseColor), r, g, b );
    SoSepAddChild(node, (SoNode *) material );
}

void iv_init(int argc, char **argv, int viewer_type)
{
    viewer = viewer_type;

    /* Initialize Xt and Inventor 
     */
    mainWindow = SoXtInit( argv[0], "Inventor" );
    if ( mainWindow == NULL ) exit( 1 );

    root = SoSepCreate();
    node = (SoNode *)root;

    SoSepRef( root );
}



void iv_save_ra(char *filename)
{
    short rows, cols;
    unsigned long *rgba_buf;
    SbColor  sbcolor, *sbcol;
    int x, y;
    FILE *fh;

    if (viewer>0)
	error("iv_draw: iv_save_ra() - not in RenderArea-Mode (Viewer on!)");

    fh = fopen(filename, "wb");
    if (fh==0) return;
    
    SoXtRARender(renderArea);
/*    
    rgba_buf = SoXtRACopyScreen(renderArea, &rows, &cols);

    
    fprintf(fh, "P6\n%d %d\n255\n", cols, rows);
    for (y=rows-1; y>=0; y--)
	for (x=0; x<cols; x++) {
	    sbcol = SbColSetPacked(&sbcolor, rgba_buf[x+y*cols]);
	    fprintf(fh, "%c%c%c", 
		    (int)(255.0* sbcol->rgb[0]),
		    (int)(255.0* sbcol->rgb[1]),
		    (int)(255.0* sbcol->rgb[2]));
	}
    
    free(rgba_buf);
    */
    fclose(fh);
}



static SbBool appEventHandler( void *userData, XAnyEvent *anyevent )
{
    SoXtRenderArea  *renderArea = (SoXtRenderArea *) userData;
    XButtonEvent  *be;
    XMotionEvent  *me;
    SbVec3f       vec;
    SbTime        t;
    SbBool        handled = TRUE;
    
    switch (anyevent->type) {
       
    case ButtonPress:
	be = (XButtonEvent *) anyevent;
	if (be->button == Button1) {
	    printf("save RA as 'iv_draw_ra.ppm'\n");
	    iv_save_ra("iv_draw_ra.ppm");
	} else if ( be->button == Button2 ) {
	    printf("save root as 'iv_draw.iv'\n");
	    iv_save_iv("iv_draw.iv");
	} else if ( be->button == Button3 ) {
	}
      break;

    default:
	handled = FALSE;
	break;
    }
    
    return handled;
}


void iv_main_loop()
{
    if (viewer>0) {
	
	SoXtExamVwr  *viewer;

	/* Setup viewer 
	 */
	viewer = SoXtExamVwrCreate(NULL, NULL, TRUE, 
				   SO_XT_FULL_VWR_BUILD_ALL, SO_XT_VWR_BROWSER );
	SoXtExamVwrSetScene( viewer, (SoNode *) root );
	SoXtExamVwrShow( viewer );

    }
    else {
	SbVec2s      size;
	SoCam        *camera;

	/* Build the render area in the applications main window 
	 */
	renderArea = SoXtRACreateStd(NULL, NULL);
	SbV2sSetX_Y(size, 512, 512);
	SoXtRASetSize(renderArea, &size);
/*	SoXtRABuild(renderArea, mainWindow, NULL); */
	/*    renderArea = SoXtRACreate(FALSE, FALSE, TRUE); */
	
	camera = (SoCamera *) SoPerspCamCreate();
	SoSepAddChild(root, (SoNode *) camera );
	
	/* Make the scene graph visible
	 */
/*old:	SoCamViewAll( camera, (SoNode *) root, 1.0 ); */
	SoCamViewAll( camera, (SoNode *) root, NULL, 1.0 );
	SoXtRASetScene(renderArea, (SoNode *) root);
	
	SoXtRASetEvCB(renderArea, appEventHandler, renderArea);

	SoXtRAShow(renderArea);
    }

    SoXtShow(mainWindow);

    SoXtMainLoop();
}

