/**************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: space_test.c
 *  TYPE		: module implementation
 *  Time-stamp: <95/02/13 16:54:39 stolz>
 **************************************************************************
 *  MODULE              : space_test
 *
 *  IMPLEMENTATION
 *
 *  NOTE
 *
 *  HISTORY
 *  25-Aug-94   Creation date.
 *  28-Aug-94   Add projection-test-data.
 *
 **************************************************************************/

#include <stdio.h>

#include "iv_draw.h"
#include "space.h" 



/****************************************************************************
 * test datas for the triangle-projection.
 */

vector3 tp_triangle_focus = { 0.7, -0.5, 0.5 };
vector3 tp_triangle[3] = {
	{ 0.6, 0.3, 0.9 },
	{ 0.9, 0.2, 0.5 },
	{ 0.6, 0.3, 0.1 }
};

#define TP_TEST_TRIANGLES 4

vector3 tp_triangles[TP_TEST_TRIANGLES][3] = {
    {
	{ 0.8, 0.6, 0.4 },
	{ 0.9, 0.3, 0.4 },
	{ 0.9, 0.3, 0.2 }
     },
    {
	{ 0.6, 0.6, 0.6 },
	{ 0.7, 0.6, 0.5 },
	{ 0.6, 0.6, 0.4 }
     },
    {
	{ 0.6, 0.4, 0.7 },
	{ 0.7, 0.4, 0.5 },
	{ 0.6, 0.4, 0.3 }
     },
    {
	{ 0.8, 1.2, 1.1 },
	{ 1.1, 1.1, 0.6 },
	{ 0.8, 1.2, 0.6 }
     }
};

/* correct results!
 */

double tp_acc = 0.05;

#define TP_POLY_SIZE	12
vector3 tp_rest_poly[TP_POLY_SIZE] = {
	{0.6, 0.3, 0.9},
	{0.7, 2.33, 7.5},
	{0.7, 2.33, 0.69},
	{0.83, 0.23, 0.6},
	{0.9, 0.2, 0.5},
	{0.42, 0.22, 0.4},
	{0.8, 0.23, 0.4},
	{0.8, 0.23, 0.37},
	{0.6, 0.3, 0.1},
	{0.6, 0.3, 0.3},
	{0.7, 0.26, 0.5},
	{0.6, 0.3, 0.7}
};





int space_test()
{
    int failed = FALSE;
    int errors = 0;
    char index;

    triangle t[6];
    space s;
    list l;
    int i;
    double dist = 0.0;
    triangle picked_t = 0;
    triangle_projection tp;

    index = 'a';
/*
   printf("SPACE-TEST from %s %s\n"
          "  %c) generate an empty space via space_new(..)\n",
          __DATE__, __TIME__, index++);
 */
   printf("SPACE-TEST\n"
          "  %c) generate an empty space via space_new(..)\n", index++);


    s = space_new();

    printf("The empty space looks like:\n");

    space_print(s);

    printf("\n\n"
	   "  %c) add four test triangles to the empty space", index++);

    iv_color(1.0, 1.0, 1.0);

    failed = FALSE;
    for (i=0; i<TP_TEST_TRIANGLES; i++) {
        t[i] = triangle_new(s, tp_triangles[i]);
        failed = failed || (t[i] == 0);
	if (failed) errors++;
	if (t[i]) iv_draw_triangle(t[i]); 
    }
    printf(" : %s\n", (failed ? "FAILED" : "PASSED"));


    printf("\nThe space after adding the triangles is:\n");

    space_print(s);

    iv_color(1.0, 0.0, 0.0);

    tp = tp_new(tp_triangle, tp_triangle_focus);
    if (tp) iv_draw_tp(tp); 
    if (failed = (tp == 0)) errors++;
    printf("  %c) Generate a triangle_projection : %s\n", index++,
	   (failed ? "FAILED" : "PASSED"));

    if (!failed) {
        list tp_cut = list_new();
        list tp_rest = list_new();

    	printf("  %c) Project the triangle in the space.\n", index++);

        project_triangle(tp_cut, tp_rest, s, tp);

	printf("  Result - Rest-faces of the triangle:\n");
/* 	list_for_all(tp_rest, tp_print); */
	iv_color(0.0, 1.0, 0.0);
	list_for_all(tp_rest, iv_draw_tp); 

	printf("  Result - Projection on other triangles:\n");
/*	list_for_all(tp_cut, tp_print); */
	iv_color(0.0, 0.0, 1.0);
	list_for_all(tp_cut, iv_draw_tp); 

	list_for_all(tp_cut, tp_free);
	list_for_all(tp_rest, tp_free);
	list_free(tp_cut);
	list_free(tp_rest);
    }
    if (tp) tp_free(tp);


    printf("  %c) clear space\n", index++);
    space_free(s);

    printf("Total amount of errors: %d. Test %s!\n\n",
			errors, errors ? "FAILED" : "PASSED");


    return errors;
}



int main(int argc, char **argv)
{
    int i;

    iv_init(argc, argv, 1);

    i = space_test();

    iv_main_loop();

    return i;
}
