/**************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: ivd.h
 *  TYPE		: module definition
 *  Time-stamp: <95/02/14 16:05:59 stolz>
 **************************************************************************
 *  MODULE              : ivd
 *
 *  FUNCTION
 *	Functions to display a mw_object as 3D with an iv-display-server
 *	are provided.
 *
 *
 *  NOTE
 *	look into ivdp_obj.h for the communication protocoll.
 *
 *  HISTORY
 *  07-Sep-94   Creation date.
 *  31-Jan-95   Change ivd_new* to support a different render-display
 *  13-Jan-95   ivd_new starts only the ivd-server with an given servername
 *
 **************************************************************************/
#ifndef IVD_H
#define IVD_H

#ifndef MW_OBJECT_H
#include "mw_object.h"
#endif


/* name of the iv-display-server-process
 */
#define IVD_SERVER "ivd_server"


/* open the display-server via process name or tid-number
 */
void ivd_new(char *ivd_server, char *ivd_server_host, char *ivd_display);
void ivd_new2(int server_tid, char *ivd_display);
void ivd_free();


/* lock and unlock the server-display
 */
void ivd_lock();
void ivd_unlock();

/* create or change the exquivalent object on the display-server.
 */
void ivd_object_new(mw_object obj);
void ivd_object_free(mw_object obj);
void ivd_object_change(mw_object obj);
void ivd_object_changes(mw_object obj, unsigned int changes);


/* request a camera-frame.
 * the image is rendered on the display-server and
 * automaticaly send to the tid-process.
 */
void ivd_req_camera_frame(mw_object camera, int tid,
			  int width, int height, int color);

/* set the current display-view of the server to the 
 * given camera.
 */
void ivd_camera_view(mw_object camera);

/* send explicit a zero-from to
 * the tid-process.
 */
void ivd_send_zero_frame(int tid);


/* hang a iv-file into the root of the display.
 */
void ivd_insert_file(char *name);


/* save the scene on the ivd-server as iv-file.
 */
void ivd_save_file(char *name);


/* save image shown in the control window as
 * pbm+-file.
 */
void ivd_save_image(char *name);


#endif
