/**************************************************************************
 *  PROJECT   : DA mobile robot simulation
 *  AUTHOR(S) : Horst Stolz (HUS)
 *  FILE      : ivd_tip.c
 *  TYPE      : module implementation
 *  Time-stamp: <94/12/06 18:41:18 stolz>
 **************************************************************************
 *  MODULE    : ivd_tip
 *
 *  FUNCTION
 *	Test-Program to send Messages to an ivd-server.
 *
 *  HISTORY
 *  23-Mar-94   Creation date
 *
 **************************************************************************/

#include "ivd.h"
#include "ivdp_obj.h"
#include "error.h"

#ifndef NO_PVM
#include "pvm3.h"
#endif



static int ivd_server_tid = -1;
static int ivd_tid = -1;
static unsigned int obj_changes = OC_ALL;


#if 0
void ivd_object_new(mw_object obj)
{
#ifndef NO_PVM
    int id;

#if 0
    printf("ivd_object_new()\n");
#endif

    if (ivd_server_tid<0) return;

    /* request unique id from server process
     */
    pvm_initsend(PvmDataDefault);
    pvm_send(ivd_server_tid, IVDP_REQ_OBJ_ID);

    /* receive id from server.
     */

    pvm_recv(ivd_server_tid, IVDP_OBJ_ID);
    pvm_upkint(&id, 1, 1);

#if 0
    printf("obj-id = %d\n", id);
#endif
    mw_object_set_id(obj, id);

    ivd_object_change(obj);
#endif
}


void ivd_object_free(mw_object obj)
{
#ifndef NO_PVM
    int id;

#if 0
    printf("ivd_object_free()\n");
#endif

    if (ivd_server_tid<0) return;

    id = mw_object_id(obj);

    /* send free-packet to server
     */
    pvm_initsend(PvmDataDefault);
    pvm_pkint(&id, 1, 1);
    pvm_send(ivd_server_tid, IVDP_OBJ_FREE);
#endif
}

void ivd_object_change(mw_object obj)
{
#ifndef NO_PVM
    int id, parent, type;
    mw_object head;
    vector3    v;
    vector3    v3[3];
    quaternion q;
    float f[9];
    char *app3d;

#if 0
    printf("ivd_object_change()\n");
#endif

    if (ivd_server_tid<0) return;

    /* send object definition to server
     */
    pvm_initsend(PvmDataDefault);

    id = mw_object_id(obj);
    pvm_pkint(&id, 1, 1);

    pvm_pkuint(&obj_changes, 1, 1);

    head = mw_object_head(obj);
    if (head) parent = mw_object_id(head);
    else parent = -1;
    pvm_pkint(&parent, 1, 1);

    type = mw_object_type(obj);
    pvm_pkint(&type, 1, 1);

    mw_object_position(&v, &q, obj);
    f[0] = (float)v[0];
    f[1] = (float)v[1];
    f[2] = (float)v[2];
    f[3] = (float)q[0];
    f[4] = (float)q[1];
    f[5] = (float)q[2];
    f[6] = (float)q[3];
    pvm_pkfloat(&f[0], 7, 1);

    app3d = (char *)mw_object_get_3d(obj);
    if (app3d)
	pvm_pkstr(app3d);
    else
	pvm_pkstr("");
	
	
    if (type != MWO_GROUP) {

	mw_object_get_color(&f[0], &f[1], &f[2], obj);
	pvm_pkfloat(&f[0], 3, 1);

	switch(type) {
        case MWO_CUBOID:
	    mwo_cuboid_get(&v, obj);
            f[0] = (float)v[0];
            f[1] = (float)v[1];
            f[2] = (float)v[2];
	    pvm_pkfloat(&f[0], 3, 1);
	    break;

	case MWO_RECTANGLE:
	    mwo_rectangle_get(&v[0], &v[1], obj);
            f[0] = (float)v[0];
            f[1] = (float)v[1];
	    pvm_pkfloat(&f[0], 2, 1);
	    break;

	case MWO_TRIANGLE:
	    mwo_triangle_get(v3, obj);
            f[0] = (float)v3[0][0];
            f[1] = (float)v3[0][1];
            f[2] = (float)v3[0][2];
            f[3] = (float)v3[1][0];
            f[4] = (float)v3[1][1];
            f[5] = (float)v3[1][2];
            f[6] = (float)v3[2][0];
            f[7] = (float)v3[2][1];
            f[8] = (float)v3[2][2];
	    pvm_pkfloat(&f[0], 9, 1);
	    break;

	default:
            error("ivd: ivd_object_change() - unknown object type");
	}
    }

    pvm_send(ivd_server_tid, IVDP_OBJ_DEF);
#endif
    obj_changes = OC_ALL;
}

void ivd_object_changes(mw_object obj, unsigned int changes)
{
    obj_changes = changes;
    ivd_object_change(obj);
}
#endif

void ivd_new(char *ivd_server, char *ivd_server_host,
	     int width,
	     int height)
{
    if (width<=0 || height<=0)
	error("ivd: ivd_new() - display width or height <=0");
#ifndef NO_PVM
    ivd_tid = pvm_mytid();
    if (ivd_tid<0) {
	warning("ivd() - ivd_new() PVM not started! ");
	ivd_tid = -1;
    }

    /* start remote ivd-server
     */
    if (pvm_spawn(ivd_server ? ivd_server : IVD_SERVER,
	      0, 0, ivd_server_host, 1,  &ivd_server_tid) != 1) {

	ivd_server_tid = -1;
	warning("ivd: ivd_new() - can't start remote ivd-server");
	return;
    }

    pvm_initsend(PvmDataDefault);
    pvm_pkint(&width, 1, 1);
    pvm_pkint(&height, 1, 1);
    pvm_send(ivd_server_tid, IVDP_OPEN_SERVER);
#endif
}


void ivd_new2(int server_tid, int width, int height)
{
    if (width<=0 || height<=0)
	error("ivd: ivd_new2() - display width or height <=0");

#ifndef NO_PVM
    ivd_tid = pvm_mytid();
    if (ivd_tid<0) {
	warning("ivd() - ivd_new() PVM not started! ");
	ivd_tid = -1;
    }

    ivd_server_tid = server_tid;

    pvm_initsend(PvmDataDefault);
    pvm_pkint(&width, 1, 1);
    pvm_pkint(&height, 1, 1);
    pvm_send(ivd_server_tid, IVDP_OPEN_SERVER);
#endif
}


void ivd_free()
{
    if (ivd_server_tid >=0) {
#ifndef NO_PVM
	pvm_initsend(PvmDataDefault);
	pvm_send(ivd_server_tid, IVDP_CLOSE_SERVER);
#endif
#if 0	
	pvm_kill(ivd_server_tid);
#endif	

    }
    ivd_tid = -1;
    ivd_server_tid = -1;
}

#if 0
void ivd_req_camera_frame(mw_object camera, int tid, 
			  int width, int height, int color)
{
    int xyscale[2] = {1, 1};
    vector3 v;
    vector3 cam_pos = {0.0, 0.0, 0.0};
    vector3 cam_dir = {0.0, 0.0, 1.0};
    float   f[6];
    float   zoom = 1.0;

#ifndef NO_PVM
    if (ivd_server_tid < 0) return;

    pvm_initsend(PvmDataDefault);
    pvm_pkint(&xyscale[0], 2, 1);

    pvm_pkfloat(&zoom, 6, 1);

    mw_object_get_abs_v3(&v, camera, cam_pos);
    f[0] = (float)v[0];
    f[1] = (float)v[1];
    f[2] = (float)v[2];

    mw_object_get_abs_v3(&v, camera, cam_dir);
    f[3] = (float)v[0];
    f[4] = (float)v[1];
    f[5] = (float)v[2];

    pvm_pkfloat(&f[0], 6, 1);

    pvm_send(ivd_server_tid, IVDP_OBJ_DEF);

#endif
}


void ivd_transmit_frame(int tid)
{
#ifndef NO_PVM
    if (ivd_server_tid <0) return;

    pvm_initsend(PvmDataDefault);
    pvm_pkint(&tid, 1, 1);
    pvm_send(ivd_server_tid, IVDP_FRAME_REQ);
#endif
}
#endif


void ivd_insert_file(char *name)
{
#ifndef NO_PVM
    if (ivd_server_tid <0) return;

    pvm_initsend(PvmDataDefault);
    if (name) pvm_pkstr(name); else pvm_pkstr("");
    pvm_send(ivd_server_tid, IVDP_INSERT_IV);
#endif
}

void ivd_save_file(char *name)
{
#ifndef NO_PVM
    if (ivd_server_tid <0) return;

    pvm_initsend(PvmDataDefault);
    if (name) pvm_pkstr(name); else pvm_pkstr("");
    pvm_send(ivd_server_tid, IVDP_SAVE_IV);
#endif
}

void ivd_save_image(char *name)
{
#ifndef NO_PVM
    if (ivd_server_tid <0) return;

    pvm_initsend(PvmDataDefault);
    if (name) pvm_pkstr(name); else pvm_pkstr("");
    pvm_send(ivd_server_tid, IVDP_SAVE_IMAGE);
#endif
}

void ivd_send_zero_frame(int tid)
{
#ifndef NO_PVM
    long frame_size = 0;
    double dt = 0.0;
    
    printf("idd_send_zero_frame() - to tid %d\n", tid);

    pvm_initsend(PvmDataDefault);
    pvm_pkdouble(&dt, 1, 1);
    pvm_pklong(&frame_size, 1, 1);
    pvm_send(tid, IVDP_FRAME);
#endif
}

void ivd_req_camera_frame(mw_object camera, int tid, 
			  int width, int height, int color)
{
    vector3 v;
    quaternion q;
    vector3 cam_pos = {0.0, 0.0, 0.0};
    quaternion cam_dir = {1.0, 0.0, 0.0, 0.0};
    float   f[10];

#ifndef NO_PVM
    if (ivd_server_tid < 0) {
	ivd_send_zero_frame(tid);
	return;
    }

#ifdef DEBUG_DETAIL
    printf("ivd_req_camera_frame() - camera=%p, tid=%d, width=%d, height=%d, color=%d\n",
	   camera, tid, width, height, color);
#endif


    pvm_initsend(PvmDataDefault);

    pvm_pkint(&tid, 1, 1);
    pvm_pkint(&width, 1, 1);
    pvm_pkint(&height, 1, 1);
    pvm_pkint(&color, 1, 1);

    /* position */
    f[0] = 0.0;
    f[1] = 0.0;
    f[2] = 0.0;

    /* orientation */
    f[3] = 1.0;
    f[4] = 0.0;
    f[5] = 0.0;
    f[6] = 0.0;

    /* angle, focus, aspect */
    f[7]  = 30.0;
    f[8]  = 1.5;
    f[9]  = 1.0;

    pvm_pkfloat(&f[0], 10, 1);

    pvm_send(ivd_server_tid, IVDP_FRAME_REQ);

#endif
}


void main(int argc, char **argv)
{
    int i;

    printf("*** IVD-TIP (Inventor-Display-Server-Tip-Console)\n");
    printf("*** possible options -load <fname> -save <fname> -image <fname> -close\n");

    if (i>1) {
	ivd_new2(atoi(argv[1]), 512, 512);
    }
    else {
	printf("*** missing ivd_server-tid!\n");
	exit(10);
    }

    for (i=2; i<argc; i++) {
	if (strcmp(argv[i], "-load") == 0) {
	    if (++i <argc) ivd_insert_file(argv[i]);
	    else printf("*** missing filename, ignore option\n");
	}
	else if (strcmp(argv[i], "-save") == 0) {
	    if (++i <argc) ivd_save_file(argv[i]);
	    else printf("*** missing filename, ignore option\n");
	}
	else if (strcmp(argv[i], "-image") == 0) {
	    if (++i <argc) ivd_save_image(argv[i]);
	    else printf("*** missing filename, ignore option\n");
	}
	else if (strcmp(argv[i], "-close") == 0) {
	    ivd_free();
	}
	else if (strcmp(argv[i], "-req") == 0) {
	    ivd_req_camera_frame(0, -1, 128, 128, 0);
	}
	else printf("*** unknown option '%s'\n", argv[i]);
    }
    exit(0);
}





