/**************************************************************************
 *  PROJECT   : DA mobile robot simulation
 *  AUTHOR(S) : Horst Stolz (HUS)
 *  FILE      : ivdp_obj.h
 *  TYPE      : module definition
 *  Time-stamp: <95/02/14 21:40:22 stolz>
 **************************************************************************
 *  MODULE    : -
 *
 *  FUNCTION
 *	Defines the iv-display-packet-object, which is transmitted
 *      from the simulator to the display-server.
 *
 *	Message protocoll between the module ivd and ivd_server:
 *
 *	ivd_server	  	  ivd
 *          |<-----OPEN_SERVER-----| open server
 *          | (int width, height)  |
 *          |                      |
 *          |<----REQ_OBJ_ID-------| create a object
 *          |--------OBJ_ID------->|
 *          |        (id)          |
 *          |                      |
 *          |     <--OBJ_DEF       | sets/change the object
 *          |    (id, ivdp_obj)    |
 *          |                      |
 *	    |         ...          |
 *          |                      |
 *          |<-----OBJ_FREE--------| delete object
 *          |        (id)          |
 *          |                      |
 *          |<----FRAME_REQ--------| request camera-frame
 *	    |  (tid,cam-pars)      |
 *          |-------FRAME--------->| send back to tid-process.(not nec. ivd)
 *          |(dt,frame-size,fra|me)|
 *          |                      |
 *          |                      |
 *	    |<--IVDP_INSERT_IV-----| insert a iv-file to the root-node
 *          |    (filename)        |
 *          |                      |
 *   	    |<--IVDP_SAVE_IMAGE----| save the image of the window to a file
 *          |    (filename)        |
 *          |                      |
 *  	    |<--IVDP_SAVE_IV-------| save the iv-objects of the ivd_server
 *          |    (filename)        |
 *          |                      |
 *   exit   |<---CLOSE_SERVER------| close the server
 *
 *  NOTE
 *
 *  HISTORY
 *  07-Mar-94   Creation date.
 *
 **************************************************************************/
#ifndef IVDP_OBJ_H
#define IVDP_OBJ_H


/* Message-tag-types between
 */
enum ivdp_types {IVDP_OPEN_SERVER,
		     IVDP_FRAME=0,
		     IVDP_CLOSE_SERVER,
		     IVDP_REQ_OBJ_ID,
		     IVDP_OBJ_ID,
		     IVDP_OBJ_DEF,
		     IVDP_OBJ_FREE,
		     IVDP_FRAME_REQ,
		     IVDP_INSERT_IV,
		     IVDP_SAVE_IMAGE,
		     IVDP_SAVE_IV,
		     IVDP_LOCK,
		     IVDP_UNLOCK
	     };


enum obj_changes {
    OC_UNCHANGED   = 0,
    OC_POSITION    = 1,
    OC_ORIENTATION = 2,
    OC_COLOR       = 4,
    OC_SHAPE       = 8,
    OC_PARENT      = 16,
    OC_VISUAL      = 32,
    OC_ALL         = 63
};


typedef struct {		/* ivdp: inventor display packet */
   /* object definitions
    */
   int id;
   unsigned int changes;
   int parent;
   int type;


   float position[3];
   float orientation[4]; 		/* quaternion */

   char  iv_file[100];

   /* for triangle/rectangle/cuboid:
    */
   float rgb[3];

   float rectangle_xy_size[2];
   float cuboid_edge_length[3];
   float triangle_edges[3][3];

} ivdp_obj;
#endif
