/********************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: mob_limits.h
 *  TYPE		: module definition
 *  Time-stamp: <95/02/27 19:40:29 stolz>
 ********************************************************************************
 *  MODULE              : - (only macro defs)
 *
 *  FUNCTION
 *
 *      The macros in this module are default, min and max values for
 *	the robuter motion-commands.
 *
 *
 *  NOTE
 *
 *  HISTORY
 *  27-Mar-94   Creation date.
 *  27-Mar-95   Change linear and angular max and min values.
 *
 ********************************************************************************/
#ifndef MOB_LIMITS_H
#define MOB_LIMITS_H


#define MAX_DISTANCE		0.001	/* mm */
#define MAX_ANGLE		1.0	/* rad */
#define MAX_ANGULAR_VEL		2.0	/* mrad/s */
#define MAX_LINEAR_VEL		2.0	/* mm/s */
#define MAX_ANGULAR_ACC		0.9	/* mrad/s^2 */
#define MAX_LINEAR_ACC		0.9	/* mm/s^2 */

#define LMAX_DISTANCE           ((1<<30)+1) /* mm */
#define LMAX_ANGLE		((1<<30)+1) /* rad */
#define LMAX_TIME		((1<<30)+1) /* ?sec */
#define LMAX_ANGULAR_VEL	2000	/* mrad/s */
#define LMAX_LINEAR_VEL		2000	/* mm/s */
#define LMAX_ANGULAR_ACC	900	/* mrad/s^2 */
#define LMAX_LINEAR_ACC		900	/* mm/s^2 */

#define MIN_DISTANCE		0.001	/* mm */
#define MIN_ANGLE		1.0	/* rad */
#define MIN_ANGULAR_VEL		0.1	/* rad/s */
#define MIN_LINEAR_VEL		0.05	/* m/s */
#define MIN_ANGULAR_ACC		0.0	/* rad/s^2 */
#define MIN_LINEAR_ACC		0.0	/* mm/s^2 */

#define LMIN_DISTANCE		1	/* mm */
#define LMIN_ANGLE		1	/* rad */
#define LMIN_ANGULAR_VEL	0	/* mrad/s */
#define LMIN_LINEAR_VEL		0	/* mm/s */
#define LMIN_ANGULAR_ACC	1	/* mrad/s^2 */
#define LMIN_LINEAR_ACC		1	/* mm/s^2 */

#define DEFAULT_ANGULAR_VEL	0.5	/* rad/s */
#define DEFAULT_LINEAR_VEL	0.5	/* m/s */
#define DEFAULT_ANGULAR_ACC	0.5	/* rad/s^s */
#define DEFAULT_LINEAR_ACC	0.5	/* m/s^s */

#define LDEFAULT_ANGULAR_VEL	500	/* mrad/s */
#define LDEFAULT_LINEAR_VEL	500	/* mm/s */
#define LDEFAULT_ANGULAR_ACC	500	/* mrad/s^s */
#define LDEFAULT_LINEAR_ACC	500	/* mm/s^s */


#define SMALLEST_RADIUS         0.4     /* of motv */

#endif
