/**************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: mob_world.h
 *  TYPE		: module definition
 *  Time-stamp: <95/02/13 16:56:35 stolz>
 **************************************************************************
 *  MODULE              : mob_world
 *
 *  FUNCTION
 *
 *  NOTE
 *
 *  HISTORY
 *  21-Apr-94   Creation date.
 *  06-Jun-94   Add some mob-specific functions
 *  13-Jun-94   Write code for add/remove objects from space.
 *  16-Jun-94   Add mob_world_is_empty() and mob_soft_reset()
 *  19-Jun-94   Split mob_world into mob_world+mw_object+mw_mob.
 *  02-Jul-94   Add mob_world_set/get().
 *  14-Sep-94   Add support for the us-computation with triangle-projs.
 *
 **************************************************************************/
#ifndef MOB_WORLD_H
#define MOB_WORLD_H

#ifndef BASIC_H
#include "basic.h"
#endif

#ifndef VECTOR_H
#include "vector.h"
#endif

#ifndef LIST_H
#include "list.h"
#endif


/**************************************************************************
 * abstract datatypes
 */
typedef struct adt_mob_world*	mob_world;

/**************************************************************************
 * enumeration for mob_world_set/get().
 */
enum mob_world_select {
    MWS_AIR_SPEED, MWS_US_AIR_DAMPING, MWS_COLLISION_STEP,
    MWS_MAX_REFLEXION_COUNT
};

enum mob_world_bselect {
    MWB_MOB_PROTOCOL, MWB_US_SENSOR_PROTOCOL, MWB_REAL_US
};

/**************************************************************************
 */

/* creation and freeing of a mob_world.
 */
mob_world	mob_world_new();
void		mob_world_free(mob_world);
void		mob_world_clear();


/* simulating
 */
double		mob_world_get_time(mob_world mw);
void		mob_world_simulate_to(mob_world, double time);
void            mob_world_reset(mob_world mw);

/* load/save
 */
extern char *mob_world_io_error_string;

mob_world	mob_world_load(char *filename);
boolean		mob_world_save(mob_world, char *filename);
boolean		mob_world_insert_objects(mob_world, char *filename);
/* returns TRUE for an error -> look at the error_string! */

/* setting world-parameters, see also the enumeration!
 */
void		mob_world_set(mob_world, int select, double);
double		mob_world_get(mob_world, int select);

void		mob_world_setb(mob_world, int select, boolean);
boolean		mob_world_getb(mob_world, int select);

/* world contents information
 */
boolean		mob_world_is_empty(mob_world);
list		mob_world_objects(mob_world);
list		mob_world_mobs(mob_world);

#endif
