/**************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: mob_world_private.h
 *  TYPE		: module definition
 *  Time-stamp: <95/02/13 16:56:46 stolz>
 **************************************************************************
 *  MODULE              : mob_world
 *
 *  FUNCTION
 *	private definitions for mob_world.c
 *
 *  NOTE
 *
 *  HISTORY
 *  20-May-94   Creation date.
 *  05-Sep-94   Add id/user_data to the mw_object.
 *  14-Sep-94   Change the camera-definition a little bit.
 *
 **************************************************************************/
#ifndef MOB_WORLD_PRIVATE_H
#define MOB_WORLD_PRIVATE_H

#ifndef MOB_WORLD_H
#include "mob_world.h"
#endif

#ifndef MW_OBJECT_H
#include "mw_object.h"
#endif

#ifndef MWO_MOB_H
#include "mwo_mob.h"
#endif

#ifndef MW_FILE_H
#include "mw_file.h"
#endif

#ifndef MOB_H
#include "mob.h"
#endif

#ifndef VECTOR_H
#include "vector.h"
#endif

#ifndef BASIC_H
#include "basic.h"
#endif

#ifndef LIST_H
#include "list.h"
#endif

#ifndef QUEUE_H
#include "queue.h"
#endif

#ifndef SPACE_H
#include "space.h"
#endif

#ifndef SCHEDULER_H
#include "scheduler.h"
#endif

#ifndef TEXT_H
#include "text.h"
#endif


#ifndef APP2D_H
#include "app2d.h"
#endif

#ifndef US_SENSOR_H
#include "us_sensor.h"
#endif


typedef struct {
    mw_object	mob;
    double	t;
    mwo_mob_state state;
} mob_check_point; 


typedef struct adt_mob_world	adt_mob_world;
struct adt_mob_world {
    scheduler	scheduler_struct;		/* for convenience */
    scheduler*	scheduler;

    timepoint	tp_collision_test;
    double	dt_collision_test;

    space	space;

    list	mobs;
    list	objects;
    list	actuators;

    list        us_sections;
    double	us_air_damping;
    double	air_speed;
    int		max_reflexion_count;

    queue	us_sensor_results;
    queue	mob_check_points;

    boolean	mob_check_point_protocol;
    boolean	us_sensor_protocol;
    boolean	real_us;

    mw_object	*remote_control;
    int		min_remote_nr;		/* min <= nr < max */
    int		max_remote_nr;
};


typedef struct {
    list	members;
}  obj_group;


typedef struct {
    vector3	edges_length;
}  obj_cuboid;


typedef struct {
    double	x_size, y_size;
} obj_rectangle;


typedef struct {
    vector3	edges[3];
}  obj_triangle;


#define MAX_NR_SENSORS        32
#define CAMERA_OFFSET         24



typedef struct {
    double	a, b, c, d, r1, r2;

    textbuffer  io_buffer;
    queue       input_queue;
    queue       output_queue;

    mob		mob;

    textbuffer	reply;
} mw_mob;


typedef struct {
    int         sensor_nr;
    double	angle;
    double	radius;
    double      threshold;
    vector3	focus;	/* relative */
/*    mw_object	controller; */
} mw_sensor;


typedef struct {
    int		camera_nr;
    double	angle;
    double	focus_distance;
    double	aspect_ratio;			/* width/height */
} mw_camera;


typedef struct {
    int         nr;
} mw_actuator;


typedef struct adt_mw_object adt_mw_object;
struct adt_mw_object {
    mob_world   world;                  /* corresponding world of this object */
    mw_object   group_head;
    char        *name;


    /* geometric attributes
     */
    int         type;
    union {
	obj_triangle	triangle;
	obj_rectangle	rectangle;
	obj_cuboid	cuboid;
	obj_group	group;
    } obj;
    vector3     position;
    quaternion  orientation;

    /* triangle-represaentation
     */
    list	triangles;

    /* ultrasonic parameters for the triangles
     */
    double	us_reflexion_damping;

    /* appearance in 2D for the Editor
     */
    boolean	highlighted;
    boolean	marked;
    app_2d	*appearance_2d;

    void	*appearance_3d;

    int function;                       /* functional attributes */
    union {
	mw_mob 		mob;
        mw_sensor       sensor;
        mw_camera       camera;
    } func;


    /* controll code for the position by an actuator
     */
    mw_actuator   actuator;

    int		attribute;
    mw_object   *sensors;

    float	r, g, b;

    hom_matrix	htm;
    vector3	*abs_edge;

    int		remote_nr;
    int		remote_tid;
    double	remote_time;
    int		remote_nr_ios;

    int         id;
    void        *user_data;
};




typedef struct io_marker io_marker;
struct io_marker {
    double	time;
    textmarker  start, end;
};

/**************************************************************************
 * private function-definition from mwo_mob.c
 */
void mwo_mob_input(char *com, void *uservar);
void mwo_mob_output(void *uservar, char *out);
void mwo_mob_open_output(void *uservar);
void mwo_mob_close_output(void *uservar);
void mwo_mob_us_sensor_scan(mob m, int sensor_nr, void *uservar);


void mob_world_store_us_result(mob_world, us_sensor_check_point *usscp);

int		mob_world_remote_nr();
void		mob_world_set_remote(mob_world mw, int nr, mw_object obj);
mw_object	mob_world_get_remote(mob_world mw, int nr);


#endif
