#include "basic.h"
#include "vector.h"
#include "error.h"
#include "mwo.h"
#include "mwo_mob.h"


mw_object mwo_create_robuter(mob_world mw)
{
    return mob_create(mw);
}



mw_object mwo_create_chair(mob_world mw)
{
    mw_object obj, box;
    int i;
    static vector3 box_size[6] = {
	{+0.350, +0.400, +0.050},
	{+0.050, +0.400, +0.200},
	{+0.050, +0.050, +0.400},
	{+0.050, +0.050, +0.400},
	{+0.050, +0.050, +0.600},
	{+0.050, +0.050, +0.600}
    };
    static vector3 box_pos[6] = {
	{-0.025,  0.000, +0.425},
	{+0.175,  0.000, +0.700},
	{-0.175, +0.175, +0.200},
	{-0.175, -0.175, +0.200},
	{+0.175, +0.175, +0.300},
	{+0.175, -0.175, +0.300}
    };

    obj = mw_object_new(mw, MWO_GROUP, MWF_OBSTACLE);
    for (i=0; i<6; i++) {
	box = mw_object_new2(mw, MWO_CUBOID, MWF_OBSTACLE,
			     box_pos[i], q_zero, box_size[i]);
	mw_object_add_to_group(obj, box);
    }
    return obj;
}




mw_object mwo_create_table(mob_world mw)
{
    mw_object obj, box;
    int i;
    static vector3 box_size[5] = {
	{+1.000, +1.000, +0.100},
	{+0.050, +0.050, +0.550},
	{+0.050, +0.050, +0.550},
	{+0.050, +0.050, +0.550},
	{+0.050, +0.050, +0.550}
    };
    static vector3 box_pos[5] = {
	{+0.000,  0.000, +0.600},
	{-0.300, -0.300, +0.275},
	{-0.300, +0.300, +0.275},
	{+0.300, -0.300, +0.275},
	{+0.300, +0.300, +0.275}
    };

    obj = mw_object_new(mw, MWO_GROUP, MWF_OBSTACLE);
    for (i=0; i<5; i++) {
	box = mw_object_new2(mw, MWO_CUBOID, MWF_OBSTACLE,
			     box_pos[i], q_zero, box_size[i]);
	mw_object_add_to_group(obj, box);
    }
    return obj;
}

mw_object mwo_create_human(mob_world mw)
{
    mw_object obj, box;
    int i;
    static vector3 box_size[7] = {
	{+0.200, +0.200, +0.250},
	{+0.400, +0.250, +0.700},
	{+0.100, +0.100, +0.800},
	{+0.100, +0.100, +0.800},
	{+0.150, +0.150, +0.700},
	{+0.150, +0.150, +0.700},
	{+0.200, +0.200, +1.000}
    };
    static vector3 box_pos[7] = {
	{+0.000,  0.000, +1.500},
	{+0.000,  0.000, +1.050},
	{-0.250,  0.000, +1.000},
	{+0.250,  0.000, +1.000},
	{-0.125,  0.000, +0.350},
	{+0.125,  0.000, +0.350},
	{+0.000, -0.200, +1.600}
    };

    obj = mw_object_new(mw, MWO_GROUP, MWF_OBSTACLE);
    for (i=0; i<6; i++) {
	box = mw_object_new2(mw, MWO_CUBOID, MWF_OBSTACLE,
			     box_pos[i], q_zero, box_size[i]);
	mw_object_add_to_group(obj, box);
    }
    box = mw_object_new2(mw, MWO_RECTANGLE, MWF_OBSTACLE,
			 box_pos[i], q_zero, box_size[i]);
    mw_object_add_to_group(obj, box);
    
    return obj;
}



mw_object mwo_create_wall(mob_world mw)
{
    mw_object obj;
    static vector3 box_size = {	+1.0, +0.200, +3.0 };
    static vector3 box_pos = { 0.0,  0.0, +1.5 };

    obj = mw_object_new2(mw, MWO_CUBOID, MWF_OBSTACLE,
			 box_pos, q_zero, box_size);
    return obj;
}

mw_object mwo_create_low_wall(mob_world mw)
{
    mw_object obj;
    static vector3 box_size = {	+1.0, +0.200, +1.5 };
    static vector3 box_pos = { 0.0,  0.0, +0.75 };

    obj = mw_object_new2(mw, MWO_CUBOID, MWF_OBSTACLE,
			 box_pos, q_zero, box_size);
    return obj;
}


mw_object mwo_create_floor(mob_world mw)
{
    mw_object obj;
    static vector3 box_size = {	+20.0, +20.0, +1.0 };

    obj = mw_object_new2(mw, MWO_RECTANGLE, MWF_OBSTACLE,
			 v3_zero, q_zero, box_size);
    return obj;
}

mw_object mwo_create_ceiling(mob_world mw)
{
    mw_object obj;
    static vector3 box_size = {	+20.0, +20.0, +1.0 };
    static vector3 box_pos = { 0.0,  0.0, +3.0 };

    obj = mw_object_new2(mw, MWO_RECTANGLE, MWF_OBSTACLE,
			 box_pos, q_zero, box_size);
    return obj;
}


mw_object mwo_create_low_desk(mob_world mw)
{
    mw_object obj;
    static vector3 box_size = {	+1.0, +0.4, +1.0 };
    static vector3 box_pos = { 0.0,  0.0, +0.5 };

    obj = mw_object_new2(mw, MWO_CUBOID, MWF_OBSTACLE,
			 box_pos, q_zero, box_size);
    return obj;
}

mw_object mwo_create_high_desk(mob_world mw)
{
    mw_object obj;
    static vector3 box_size = {	+1.0, +0.4, +2.0 };
    static vector3 box_pos = { 0.0,  0.0, +1.0 };

    obj = mw_object_new2(mw, MWO_CUBOID, MWF_OBSTACLE,
			 box_pos, q_zero, box_size);
    return obj;
}


mw_object mwo_create_closed_door(mob_world mw)
{
    mw_object obj, box;
    quaternion q;
    int i;
    static vector3 box_size[4] = {
	{+1.000, +0.200, +1.000},
	{+1.000, +0.100, +2.000},
	{+0.200, +0.100, +90.000},
	{+0.200, +0.100, +90.000}
    };
    static vector3 box_pos[5] = {
	{+0.000,  0.000, +2.500},
	{+0.000,  0.000, +1.000},
	{-0.400, -0.100, +1.000},
	{-0.400, +0.100, +1.000},
	{-1.000, +0.000, +0.000}
    };
    

    obj = mw_object_new(mw, MWO_GROUP, MWF_OBSTACLE);
    for (i=0; i<2; i++) {
	box = mw_object_new2(mw, MWO_CUBOID, MWF_OBSTACLE,
			     box_pos[i], q_zero, box_size[i]);
	mw_object_add_to_group(obj, box);
    }
    box = mw_object_new2(mw, MWO_TRIANGLE, MWF_OBSTACLE,
			 box_pos[2], q_zero, box_size[2]);
    mw_object_add_to_group(obj, box);
    q_set_av(&q, PI, box_pos[4]);
    box = mw_object_new2(mw, MWO_TRIANGLE, MWF_OBSTACLE,
			 box_pos[3], q, box_size[3]);
    mw_object_add_to_group(obj, box);
    
    return obj;
}


mw_object mwo_create_opened_door(mob_world mw)
{
    mw_object obj, box;
    quaternion q;
    static vector3 box_size[4] = {
	{+1.000, +0.200, +1.000},
	{+1.000, +0.100, +2.000},
	{+0.200, +0.100, +90.000},
	{+0.200, +0.100, +90.000}
    };
    static vector3 box_pos[7] = {
	{+0.000,  0.000, +2.500},
	{-0.450,  0.000, +1.000},
	{-0.850, -0.100, +1.000},
	{-0.850, +0.100, +1.000},
	{-1.000, +0.000, +0.000},
	{+0.000, +0.000, +1.000},
	{+0.450, +0.000, +0.000}
    };
    

    obj = mw_object_new(mw, MWO_GROUP, MWF_OBSTACLE);
    box = mw_object_new2(mw, MWO_CUBOID, MWF_OBSTACLE,
			 box_pos[1], q_zero, box_size[1]);
    mw_object_add_to_group(obj, box);

    box = mw_object_new2(mw, MWO_TRIANGLE, MWF_OBSTACLE,
			 box_pos[2], q_zero, box_size[2]);
    mw_object_add_to_group(obj, box);
    q_set_av(&q, PI, box_pos[4]);
    box = mw_object_new2(mw, MWO_TRIANGLE, MWF_OBSTACLE,
			 box_pos[3], q, box_size[3]);
    mw_object_add_to_group(obj, box);

    box = obj;
    q_set_av(&q, PI/2.0, box_pos[5]);
    mw_object_set_position(box, box_pos[6], q); 

    obj = mw_object_new(mw, MWO_GROUP, MWF_OBSTACLE);
    mw_object_add_to_group(obj, box);

    box = mw_object_new2(mw, MWO_CUBOID, MWF_OBSTACLE,
			 box_pos[0], q_zero, box_size[0]);
    mw_object_add_to_group(obj, box);
    
    return obj;
}

mw_object mwo_create(mob_world mw, int type)
{
    switch(type) {
    case MWOT_ROBUTER:     return mwo_create_robuter(mw);
    case MWOT_CHAIR:       return mwo_create_chair(mw);
    case MWOT_TABLE:       return mwo_create_table(mw);
    case MWOT_WALL:        return mwo_create_wall(mw);
    case MWOT_HUMAN:       return mwo_create_human(mw);
    case MWOT_FLOOR:       return mwo_create_floor(mw);
    case MWOT_CEILING:     return mwo_create_ceiling(mw);
    case MWOT_DOOR_OPEN:   return mwo_create_opened_door(mw);
    case MWOT_DOOR_CLOSED: return mwo_create_closed_door(mw);
    case MWOT_DESK_HIGH:   return mwo_create_high_desk(mw);
    case MWOT_DESK_LOW:    return mwo_create_low_desk(mw);
    case MWOT_WALL_LOW:    return mwo_create_low_wall(mw);
    default:
	return 0;
    }
}
