/**************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: mwo_mob.h
 *  TYPE		: module definition
 *  Time-stamp: <95/02/13 16:58:43 stolz>
 **************************************************************************
 *  MODULE              : mwo_mob.c
 *
 *  FUNCTION
 *
 *  NOTE
 *
 *  HISTORY
 *  21-Apr-94   Creation date.
 *  06-Jun-94   Add some mob-specific functions
 *  13-Jun-94   Write code for add/remove objects from space.
 *  16-Jun-94   Add mob_world_is_empty() and mob_soft_reset()
 *  19-Jun-94   Split mob_world into mob_world+mw_object+mw_mob.
 *
 **************************************************************************/
#ifndef MWO_MOB_H
#define MWO_MOB_H

#ifndef MW_OBJECT_H
#include "mw_object.h"
#endif

#ifndef TEXT_H
#include "text.h"
#endif




/**************************************************************************
 * special functions for the mob-object
 */

enum mob_type_enum {MOB_STOPPED, MOB_MOVING, MOB_HALTED};
typedef struct {
    double	x, y, angle, v, w, a, u;
    double	x_real, y_real, angle_real;
    int		type;
} mwo_mob_state;


mw_object mob_create(mob_world mw);

void    mob_send_command(mw_object mob, char *command);
void	mob_send_cmd(mw_object mob, double t, char *command);
void	mob_get_state(mw_object, mwo_mob_state *);

void	mob_get_io(mw_object, char **text, int *len);
void	mob_clear_io(mw_object);

void    mob_emergency_halt(mw_object mob);
void    mob_emergency_resume(mw_object mob);
void	mob_soft_reset(mw_object mob);
int 	mob_motion_type(mw_object obj);


enum mob_actuators_enum {ACT_MOB, ACT_MOB_WHEEL_FL, ACT_MOB_WHEEL_FR};


void mwo_mob_inform_actuator(mw_object obj, mw_object controller);
void mwo_mob_req_actuator(mw_object obj, mw_object controller);


#endif
