/**************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: polygon_test.c
 *  TYPE		: module implementation
 *  Time-stamp: <95/02/13 16:59:46 stolz>
 **************************************************************************
 *  MODULE              : polygon_test
 *
 *  IMPLEMENTATION
 *
 *  NOTE
 *      This are module-test-routines for the polygon-module.
 *
 *  HISTORY
 *  11-Aug-94   Creation date
 *
 **************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include "basic.h"
#include "polygon.h"


vector3 simple_poly[5] = {
	-25.0,   5.0,   0.0,
	-10.0,  15.0,   0.0,
	 10.0,  10.0,   0.0,
	  0.0, -10.0,   0.0,
	-15.0, -10.0,   0.0
};

int simple_result[3][3] = {
        0, 1, 2,
	0, 2, 3,
	0, 3, 4
};


vector3 complex_poly[11] = {
	 30.0,   5.0,   0.0,
	 45.0,  10.0,   0.0,
	 60.0,   0.0,   0.0,
	 70.0,  15.0,   0.0,
	 80.0,   0.0,   0.0,
	 95.0,  -5.0,   0.0,
	 75.0, -15.0,   0.0,
	 70.0,   5.0,   0.0,
	 55.0, -15.0,   0.0,
	 45.0,   5.0,   0.0,
	 35.0, -10.0,   0.0
};

int complex_result[9][3] = {
	4, 5, 6,
	3, 4, 6,
	3, 6, 7,
	2, 3, 7,
	2, 7, 8,
	1, 2, 8,
	1, 8, 9,
	0, 1, 9,
	0, 9, 10
};



int failed = FALSE;
int errors = 0;


void triangle_check(vector3 *p, vector3 *poly, int result[][3], int nr, int i)
{
    if (i==0) failed = FALSE;

    printf("Triangle %d. (%2.1f %2.1f), (%2.1f %2.1f), (%2.1f %2.1f)",
	   i+1,
	   p[0][0], p[0][1], p[1][0], p[1][1], p[2][0], p[2][1]);

    if (i>nr ||
        p[0][0] != poly[result[i][0]][0] ||
        p[0][1] != poly[result[i][0]][1] ||
        p[0][2] != poly[result[i][0]][2] ||
        p[1][0] != poly[result[i][1]][0] ||
        p[1][1] != poly[result[i][1]][1] ||
        p[1][2] != poly[result[i][1]][2] ||
        p[2][0] != poly[result[i][2]][0] ||
        p[2][1] != poly[result[i][2]][1] ||
        p[2][2] != poly[result[i][2]][2])
    {
	failed = TRUE;
	errors++;
	printf(" : WRONG\n");
    } 
    else printf(" : CORRECT\n");
}


void simple_test(vector3 *p)
{
    static int i=0;

    triangle_check(p, simple_poly, simple_result, 4, i);
    i++;
}

void complex_test(vector3 *p)
{
    static int i=0;

    triangle_check(p, complex_poly, complex_result, 9, i);
    i++;
}


int polygon_test()
{
    char index;
    vector3 normal = {0.0, 0.0, 1.0};
    int i;

    index = 'a';
    printf("POLYGON-TEST from %s %s\n", __DATE__, __TIME__);


    printf("%c) simple polygon triangulation\n", index++);
    printf("  the polygon looks like:\n");
    for (i=0; i<5; i++)
        printf("  (%2.1f %2.1f)", simple_poly[i][0], simple_poly[i][1]);
    printf("\n  triangulation results:\n");

    convex_polygon3_triangulation(simple_poly, 5, simple_test);
    printf("  simple triangulation test : %s\n",
	   (failed ? "FAILED" : "PASSED"));

    printf("%c) convex/concave polygon triangulation\n", index++);
    printf("  the polygon looks like:\n");
    for (i=0; i<11; i++)
        printf("  (%2.1f %2.1f)", complex_poly[i][0], complex_poly[i][1]);
    printf("\n  triangulation results:\n");

    polygon3_triangulation(complex_poly, 11, normal, complex_test);
    printf("  complex triangulation test : %s\n",
	   (failed ? "FAILED" : "PASSED"));

    i = errors;

    v3_neg(normal, normal);
    printf("%c) convex/concave polygon triangulation with wrong normal!!\n", index++);

    polygon3_triangulation(complex_poly, 11, normal, complex_test);

    errors = i;
    failed = !failed;
    if (failed) errors++;

    printf("  triangulation test with wrong normal: %s\n",
	   (failed ? "FAILED" : "PASSED"));

    printf("Total amount of errors: %d. Test %s!\n\n",
	   errors, (errors ? "FAILED" : "PASSED"));

    return errors;
}


int main(int argc, char **argv)
{
    return polygon_test();
}

