#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "rob.h"

#include "mobs_interface.h"



static rob_args  rob_options = {0, TRUE, -1, 0, 0};


rob_args *rob_parse_args(int argc, char **argv)
{
    int i;

    if (argc>0) {
	rob_options.argv = malloc(argc*sizeof(char *));
	if (rob_options.argv == 0) {
	    fprintf(stderr, "rob_additions: rob_parse_args() - can't alloc memory\n");
	    exit(10);
	}
        rob_options.argv[0] = argv[0];
        rob_options.argc++;

	for (i=1; i<argc; i++) {
	    if (strcmp(argv[i], "-sim")==0 || 
		strcmp(argv[i], "-s")==0 ) {
		rob_options.start_simulator = TRUE;
            }
	    else if (strcmp(argv[i], "-simtid")==0) {
                if (i+1>=argc) {
                    fprintf(stderr, "rob_additions: rob_parse_args() - tid-number after '%s' missing!\n",
			    argv[i]);
                    exit(10);
		}
		i++;
		rob_options.start_simulator = TRUE;
		rob_options.simulator_tid = strtol(argv[i], 0, 0);
            }
	    else if (strcmp(argv[i], "-rob")==0 ||
		     strcmp(argv[i], "-r")==0) {
                if (i+1>=argc) {
                    fprintf(stderr, "rob_additions: rob_parse_args() - name-string after '%s' missing!\n",
			    argv[i]);
                    exit(10);
		}
		i++;
		rob_options.rob_name = argv[i];
            }
            else {
		rob_options.argv[rob_options.argc] = argv[i];
	        rob_options.argc++;
	    }
        }

	for (i=rob_options.argc; i<argc; i++)
	    rob_options.argv[i] = 0;

    }

    return &rob_options;
}

int rob_pvm_start()
{
    return RobOk;
}

int rob_pvm_halt()
{
    return RobOk;
}

int rob_init(char* robname)
{
    return (int)mi_open(robname, rob_options.simulator_tid);
}


int rob_exit(int rob)
{
    mi_close((mobs_obj)rob);

    return RobOk;
}


int rob_command(int rob, char *command, char *reply)
{
    mi_send((mobs_obj)rob, command, reply, ROBOREPLYLENGTH-1);

    return reply ? strlen(reply) : 0;
}


int rob_camera_frame(unsigned char *frame, int rob, int camera_nr,
		     int width, int height, int color)
{
    if (frame && rob && camera_nr>=0 && width>0 && height>0 &&
	mi_get_camera_frame(frame, (mobs_obj)rob,
			    camera_nr, width, height, color)) {
	return RobOk;
    }
    else return RobCameraFail;
}


int rob_save_frame(char *fname, 
		   unsigned char *frame, int width, int height, int color)
{
    FILE *fh = fopen(fname, "wb");

    if (fh==0) return RobFileFail;
    if (color) {
	 fprintf(fh, "P6\n%d %d\n255\n", width, height);
	 fwrite(frame, width*height*3, 1, fh);
	 fclose(fh);
     }
     else {
	 fprintf(fh, "P5\n%d %d\n255\n", width, height);
	 fwrite(frame, width*height, 1, fh);
	 fclose(fh);
    }
    return RobOk;
}

int rob_start(int ttyflag)
{
  return(RobOk);
}

int rob_halt()
{
  return(RobOk);
}

