#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "rob.h"

unsigned char frame[CAMERA_DEF_WIDTH][CAMERA_DEF_HEIGHT];
int rob;
char reply[ROBOREPLYLENGTH+1];
char fname[200];
int camera_request = 0;

void camera(int frames, int waits)
{
    static int nr=0;
    int i;

    if (camera_request==0) return;


    for (;frames>0; frames--) {
	if (RobOk==rob_camera_frame((unsigned char *)frame, rob, 0, CAMERA_DEF_WIDTH, CAMERA_DEF_HEIGHT, 0)) {
	    
	    sprintf(fname, "frame%03d.pgm", nr++);
	    
	    printf("*** save frame to file '%s'\n", fname);
	    rob_save_frame(fname, (unsigned char*)frame, 
			   CAMERA_DEF_WIDTH, CAMERA_DEF_HEIGHT, 0);
	}
	else {
	    printf("*** error. no line to the camera.\n");
	    camera_request = 0;
	}
	for (i=waits; i>0; i--)
	    rob_command(rob, "odom", reply);
    }
}

int main(int argc, char **argv)
{  
    char *rob_name = "porthos";
    rob_args *args;

    args = rob_parse_args(argc, argv);
    if (args->rob_name) rob_name = args->rob_name;

    argc = args->argc;
    argv = args->argv;
    if (argc>1) 
	camera_request = 1;


    rob_pvm_start();
    rob = rob_init(rob_name);
    if (rob<=0) {
	printf("*** error. can't connect to robot '%s'.\n", rob_name);
	rob_pvm_halt();
	return 10;
    }

    printf("*** Connected to robot '%s'.\n", rob_name);

    if (rob_camera_frame((unsigned char *)frame, rob, 0,
			 CAMERA_DEF_WIDTH, CAMERA_DEF_HEIGHT, 0)==RobOk) {

	printf("*** sucessfuly grabed a frame.\n");
    }
    else {
	printf("*** error. no line to the camera.\n");
    }
    
    rob_command(rob, "motn on", reply);
    rob_command(rob, "odom on", reply);

    camera(1, 0);
    
    rob_command(rob, "motn wa", reply);
    rob_command(rob, "motn fo d=3000", reply);
    camera(6, 9);
    rob_command(rob, "motn wa", reply);
    rob_command(rob, "motn ri a=90", reply);
    camera(4, 9);
    rob_command(rob, "motn wa", reply);
    rob_command(rob, "motn fo d=5200", reply);
    camera(11, 9);
    rob_command(rob, "motn wa", reply);
    rob_command(rob, "motn le a=90", reply);
    camera(4, 9);
    rob_command(rob, "motn wa", reply);
    rob_command(rob, "motn fo d=5800", reply);
    camera(12, 9);
    rob_command(rob, "motn wa", reply);

    camera(1, 0);
    

/* backwards */
    rob_command(rob, "motn ba d=5800", reply);
    rob_command(rob, "motn wa", reply);
    rob_command(rob, "motn ri a=90", reply);
    rob_command(rob, "motn wa", reply);
    rob_command(rob, "motn ba d=5200", reply);
    rob_command(rob, "motn wa", reply);
    rob_command(rob, "motn le a=90", reply);
    rob_command(rob, "motn wa", reply);
    rob_command(rob, "motn ba d=3000", reply);
    rob_command(rob, "motn wa", reply);

    rob_command(rob, "motn off", reply);


    
    rob_exit(rob);
    rob_pvm_halt();

    return 0;
}
