#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifndef CLOCKS_PER_SEC
#define CLOCKS_PER_SEC  1000000L
#endif

#include "rob.h"

int main(int argc, char **argv)
{  
    char *rob_name = "porthos";
    int rob;
    rob_args *args;
    int color = 0, save = 0;
    int i;
    int width=128, height=128;
    unsigned char *frame = 0;
    int nr = 0;
    int count=1;
    int time=0;
    clock_t tstart, tstop;

    args = rob_parse_args(argc, argv);
    if (args->rob_name) rob_name = args->rob_name;

    argc = args->argc;
    argv = args->argv;

    (void)clock();

    for (i=0; i<argc; i++) {
	if (strcmp(argv[i], "-color")==0)
	    color = 1;
	else if (strcmp(argv[i], "-save")==0)
	    save = 1;
	else if (strcmp(argv[i], "-time")==0)
	    time = 1;
	else if (strcmp(argv[i], "-width")==0) {
	    i++;
	    if (i<argc) width = atoi(argv[i]);
	    else printf("%s: missing number after option '%s'.\n", 
			argv[0], argv[i-1]);
	    
	}
	else if (strcmp(argv[i], "-height")==0) {
	    i++;
	    if (i<argc) height = atoi(argv[i]);
	    else printf("%s: missing number after option '%s'.\n", 
			argv[0], argv[i-1]);
	}
	else if (strcmp(argv[i], "-camera")==0) {
	    i++;
	    if (i<argc) nr = atoi(argv[i]);
	    else printf("%s: missing number after option '%s'.\n", 
			argv[0], argv[i-1]);
	}
	else if (strcmp(argv[i], "-count")==0) {
	    i++;
	    if (i<argc) count = atoi(argv[i]);
	    else printf("%s: missing number after option '%s'.\n", 
			argv[0], argv[i-1]);
	}
	else if (argv[i][0] == '?' || argv[i][0] == 'h' ||
		 argv[i][0] == 'H' || strncmp(argv[i], "-h", 2)==0 ||
		 strncmp(argv[i], "-H", 2)==0) {

	    /* print Help-text
	     */
	    printf("Usage: %s [rob-options] [-width <def.128>] [-height <def.128>]\n   [-camera <def.0>] [-color] [-save]\n", argv[0]);
	    printf("  Request a frame with the size widthxheight of a rob-camera.\n  And save the image to the file 'frame.p[gp]m'.\n");
	    return 0;
	}
    }

    if (width<1 || height<1 || nr<0) {
	printf("*** error. illegal image-size or camera-nr.\n");
	return 10;
    }

    frame = (unsigned char*)malloc(width*height*(color ? 3 : 1)
				   *sizeof(unsigned char));

    if (frame==0) {
	printf("*** error. can't alloc memory for requested image-size.\n");
	return 10;
    }

    rob_pvm_start();
    rob = rob_init(rob_name);
    if (rob<=0) {
	printf("*** error. can't connect to robot '%s'.\n", rob_name);
	rob_pvm_halt();
	return 10;
    }

    printf("*** Connected to robot '%s'.\n", rob_name);

    for (i=0; i<count; i++) {
	tstart = clock();
	if (rob_camera_frame(frame, rob, nr, width, height, color)==RobOk) {
	    
	    tstop = clock();

	    printf("*** sucessfuly grabed a frame.\n");

	    if (time) {
		printf("*** grabing time was %2.3fsec\n", 
		       ((float)(tstop - tstart))/
		       ((float)CLOCKS_PER_SEC));
	    }

	    if (save) {
		char fname[100];

		if (count>1)
		    sprintf(fname, "frame%03d.p%cm", count, 
			    (color ? 'p' : 'g'));
		else
		    sprintf(fname, "frame.p%cm", (color ? 'p' : 'g'));

		printf("*** save %sframe into file '%s'.\n",
		       (color ? "color ": ""), fname);

		rob_save_frame(fname, frame, width, height, color);
	    }
	}
	else {
	    printf("*** error. no line to the camera.\n");
	    rob_exit(rob);
	    rob_pvm_halt();
	    return 10;
	}
    }

    rob_exit(rob);
    rob_pvm_halt();

    return 0;
}
