#include <stdio.h>
#include <stdlib.h>

#include "rob.h"

#define ARGUMENTS "r:0:1:2:3:4:5:6:7:8:9:f:sh"

extern char *optarg;                    /* getopt(3) control vars */


static int multi_init(int *robots, int num, char *names[])
{
    int i;
    int succ=0;
    
    for (i=0; i<num; i++) {
	if (names[i]) {
	    robots[i] = rob_init(names[i]);
	    if (robots[i] > 0) {
		printf("*** connecting robot#%d '%s'\n", i, names[i]);
		succ++;
	    } else {
		printf("*** ERROR: no connection to robot '%s'\n", names[i]);
	    }
	}
    }
    fflush(stdout);
    return succ;
}


static void multi_command(int robots[], int num, char *command)
{
    char reply[ROBOMSGLENGTH+1];
    int i;

    printf("*** send command '%s' to multiple robots\n", 
	   command ? command : "empty?");

    if (command) {
	for (i=0; i<num; i++) {
	    if (robots[i]>0)
		rob_command(robots[i], command, reply);
	}
    }
    fflush(stdout);
}

static void multi_exit(int robots[], int num)
{
    int i;
    
    for (i=0; i<num; i++)
	if (robots[i]>0) {
	    printf("*** close connection of robot#%d\n", i);
	    rob_exit(robots[i]);
	}
    fflush(stdout);
}

int main(int argc, char* argv[])
{
  int c, i;
  int robot[11];
  char *name[11];
  char *fname=0;
  int mode = RobViaServer;

  for (i=0; i<11; i++) {
      robot[i] = 0;
      name[i] = 0;
  }

  while ((c = getopt(argc, argv, ARGUMENTS)) != -1)
    switch (c) {
    case '0':
    case '1':
    case '2':
    case '3':
    case '4':
    case '5':
    case '6':
    case '7':
    case '8':
    case '9':
    case 'r':
	if (c=='r') name[10]= optarg;
	else name[c-'0'] = optarg;
	break;
    case 's':
	mode = RobViaSimulator;
	break;
    case 'f':
	fname = optarg;
	break;
    case 'h':
    case '?':
	printf("usage: %s [-s] [-(r|0|1..|9) <robotname>] [-f <cmdfile>]\n",
	       argv[0]);
	printf("description: send commands to multiple robots\n");
	printf("             default is a static motion with breaks\n");
	exit(0);
	break;
    default:
	break;
    }

  printf("*** Multi-Robot-Application ***\n");
  printf("*** using mode=%d\n", mode);
  
  fflush(stdout);

  rob_start(mode);
  if (multi_init(robot, 11, name)>0) {

      if (fname) {
	  FILE *fh;
	  char cmd[ROBOCMDLENGTH+10];
	  
	  printf("*** executing commands from file '%s'\n", fname);
	  fflush(stdout);
	  
	  if (fname[0]=='-' && fname[1]==0) fh = stdin;
	  else fh = fopen(fname, "r");

	  if (fh) {
	      i = 0;
	      while ((c=fgetc(fh))!=EOF) {
		  if (i<ROBOCMDLENGTH) {
		      cmd[i] = c;
		      i++;
		  }
		  if (c=='\n') {
		      cmd[i] = '\0';
		      multi_command(robot, 11, cmd);
		      i = 0;
		  }
	      }
	      if (fh!=stdin) fclose(fh);
	  }
	  else {
	      printf("*** ERROR: can`t open file for reading\n");
	  }
      }
      else {
	  printf("*** driving 1m forward an back with breaks of 5 secs\n");
	  sleep(5);
	  multi_command(robot, 11, "MOTV ON");
	  sleep(5);
	  multi_command(robot, 11, "MOTV LA v=1000 d=1000");
	  sleep(5);
	  multi_command(robot, 11, "MOTV LA v=-1000 d=1000");
	  sleep(5);
	  multi_command(robot, 11, "MOTV OFF");
	  sleep(5);
      }
      printf("*** done\n");
  }
  else {
      printf("*** WARNING - no robot selected\n");
  }

  fflush(stdout);

  multi_exit(robot, 11);
  rob_halt();
  exit(0);
}
