#include <stdio.h>
#include <stdlib.h>

#include "rob.h"
#include "rob_us_sensor.h"

#ifndef min
#define min(a,b) (((a)<(b))?(a):(b))
#endif


const int FREE_DIST = 500;
const int NOM_DIST = 50;
const int MAX_DISTANCE = 500;


void us_sensor_drive(int rob)
{
    char reply[ROBOREPLYLENGTH+1];
    rus sen;
    int V, VL, VR, L, R;
    long view;
    int home = 10;

    rob_command(rob, "motv on", reply);
    sen = rus_open(rob);
    
    V = rus_sensor(sen, PANEL_FRONT, 0);
    VL = rus_sensor(sen, PANEL_FRONT_LEFT, 0);
    VR = rus_sensor(sen, PANEL_FRONT_RIGHT, 0);
    L = rus_sensor(sen, PANEL_FRONT_LEFT, 90);
    R = rus_sensor(sen, PANEL_FRONT_RIGHT, -90);
    rus_select(sen, V);
    rus_select(sen, VL);
    rus_select(sen, VR);
    rus_select(sen, R);
    rus_select(sen, L);
    
    do {
	
	rus_read(sen);

	printf("\nUS-Sensor-Results: (in mm)\n");
	printf("     VL=%ld    V=%ld   VR=%ld\n",
	       rus_result(sen, VL),
	       rus_result(sen, V),
	       rus_result(sen, VR));
	printf("L=%ld                                 R=%ld\n",
	       rus_result(sen, L),
	       rus_result(sen, R));

	view = (rus_result(sen, L) > FREE_DIST) ? 1 : 0;
	view <<= 4;
	view |= (rus_result(sen, VL) > FREE_DIST) ? 1 : 0;
	view <<= 4;
	view |= (rus_result(sen, V) > FREE_DIST) ? 1 : 0;
	view <<= 4;
	view |= (rus_result(sen, VR) > FREE_DIST) ? 1 : 0;
	view <<= 4;
	view |= (rus_result(sen, R) > FREE_DIST) ? 1 : 0;


	printf("->FREE VIEW 0x%05lx\n", view);

	switch(view) {
	case 0x11111:
	    printf("Drive forward(fast).\n");
	    rob_command(rob, "MOTV LA V=200 D=1000", reply);
	    break;

	case 0x00100:
	    printf("Drive forward(slow).\n");
	    rob_command(rob, "MOTV LA V=50 D=100", reply);
	    break;

	case 0x01110:
	    printf("Drive forward.\n");
	    rob_command(rob, "MOTV LA V=100 D=200", reply);
	    break;

	case 0x01100:
	case 0x01101:
	case 0x11100:
	case 0x11110:
	case 0x11101:
	    printf("Drive left.\n");
	    rob_command (rob,"MOTV LA  V=100 W=250 t=6", reply);
	    break;

	case 0x00110:
	case 0x10110:
	case 0x00111:
	case 0x10111:
	case 0x01111:
	    printf("Drive right.\n");
	    rob_command(rob,"MOTV LA  V=100 W=-250 t=6", reply);
	    break;

	case 0x00001:
	case 0x00011:
	case 0x10001:
	case 0x10011:
	    printf("Turn right.\n");
	    rob_command (rob,"MOTV LA  V=0 W=-250 t=6", reply);
	    break;

	case 0x10000:
	case 0x11000:
	case 0x11001:
	    printf("Turn left.\n");
	    rob_command (rob,"MOTV LA  V=0 W=250 t=6", reply);
	    break;

	case 0x00000:
	    home--;
	    printf("I want to go home!\n");
	    break;
	    
	default:
	    printf("Don't known what to do! Turn left.\n");
	    rob_command (rob,"MOTV LA  V=0 W=250 t=6", reply);
	    break;
	}
	fflush(stdout);

    } while(view || home>=0);

     rus_close(sen);

     rob_command (rob, "motv ki", reply); 
     rob_command (rob, "motv off", reply); 
}






int main(int argc, char **argv)
{  
    char *rob_name = "porthos";
    rob_args *args;
    int rob;

    args = rob_parse_args(argc, argv);
    if (args->rob_name) rob_name = args->rob_name;

    rob_pvm_start();
    rob = rob_init(rob_name);
    if (rob<=0) {
	printf("*** error. can't connect to robot '%s'.\n", rob_name);
	rob_pvm_halt();
	return 10;
    }

    us_sensor_drive(rob);

    rob_exit(rob);
    rob_pvm_halt();

    return 0;
}
