#include <stdio.h>
#include <stdlib.h>

#include "rob.h"
#include "rob_us_sensor.h"


#define MAX_NR_SENSORS 24

#define MAX_NR_NODES   6
#define MAX_NR_SENSORS_PER_NODE 6


typedef struct {
    rus_sensor_info info;
    int node;
    int nr;
    long result;
} us_sensor;

struct rus_adt {
    int rob;
    int available;
    int selected;
    long active_nodes;
    long active_sensors[MAX_NR_SENSORS_PER_NODE];
    int sensor_nr[MAX_NR_NODES][MAX_NR_SENSORS_PER_NODE];
    us_sensor sensors[MAX_NR_SENSORS];
    double result_scale;
};
typedef struct rus_adt rus_adt;


static char reply[ROBOREPLYLENGTH+1];
static char rob_cmd[100];

static long us24_pos[24][3] = {
    {741, 000, 263},
    {741, 217, 263},
    {723, 263, 263},
    {693, 304, 263},
    {657, 334, 263},
    {567, 340, 263},
    {228, 340, 263},
    {110, 340, 263},
    {200, 334, 263},
    {236, 304, 263},
    {266, 263, 263},
    {284, 217, 263},
    {284, 000, 263},
    {284, 217, 263},
    {266, 263, 263},
    {236, 304, 263},
    {200, 334, 263},
    {110, 340, 263},
    {228, 340, 263},
    {567, 340, 263},
    {657, 334, 263},
    {693, 304, 263},
    {723, 263, 263},
    {741, 217, 263}
};


#define set_us(Msensor,Mnode,Madr,Mpanel, Mangle) \
    {r->sensor_nr[Mnode-1][Madr-1] = Msensor-1; \
	 r->sensors[Msensor-1].info.angle = Mangle; \
	 r->sensors[Msensor-1].info.panel = Mpanel; \
	 r->sensors[Msensor-1].node = Mnode -1; \
	 r->sensors[Msensor-1].nr = Madr -1;}
	    


rus  rus_open(int rob)
{
    rus r;
    int i, j;

    r = (rus)malloc(sizeof(rus_adt));
    if (r==0) {
	fprintf(stderr, "rob_us_sensor: rus_open() - can't alloc memory\n");
	exit(10);
    }
    r->rob = rob;

    r->result_scale = 0.412178;
    r->available = 0;
    r->selected  = 0;
    r->active_nodes = 0;
    for (i=0; i<MAX_NR_SENSORS_PER_NODE; i++) {
	r->active_sensors[i] = 0;
    }

    for (i=0; i<MAX_NR_NODES; i++) 
	for (j=0; j<MAX_NR_SENSORS_PER_NODE; j++)
	    r->sensor_nr[i][j] = -1;
    
    for (i=0; i<MAX_NR_SENSORS; i++) {
	r->sensors[i].info.panel = PANEL_FRONT;
	r->sensors[i].info.xpos = 0;
	r->sensors[i].info.ypos = 0;
	r->sensors[i].info.zpos = 0;
	r->sensors[i].info.angle = 0;
	r->sensors[i].result = -1;
	r->sensors[i].node = -1;
	r->sensors[i].nr = -1;
    }


    r->available = 24;
    for (i=0; i<24; i++) {
	r->sensors[i].info.xpos = us24_pos[i][0];
	r->sensors[i].info.ypos = us24_pos[i][1];
	r->sensors[i].info.zpos = us24_pos[i][2];
    }
	
    set_us( 1, 6, 2, PANEL_FRONT, 0);
    set_us( 2, 5, 2, PANEL_FRONT_RIGHT, 0);
    set_us( 3, 4, 2, PANEL_FRONT_RIGHT, -30);
    set_us( 4, 3, 2, PANEL_FRONT_RIGHT, -45);
    set_us( 5, 2, 2, PANEL_FRONT_RIGHT, -60);
    set_us( 6, 5, 3, PANEL_FRONT_RIGHT, -90);
    set_us( 7, 6, 3, PANEL_RIGHT, -90);
    set_us( 8, 1, 4, PANEL_BACK_RIGHT, -90);
    set_us( 9, 2, 4, PANEL_BACK_RIGHT, -120);
    set_us(10, 3, 4, PANEL_BACK_RIGHT, -135);
    set_us(11, 4, 4, PANEL_BACK_RIGHT, -150);
    set_us(12, 5, 4, PANEL_BACK_RIGHT, 180);
    set_us(13, 6, 4, PANEL_BACK, 180);
    set_us(14, 1, 1, PANEL_BACK_LEFT, 180);
    set_us(15, 2, 1, PANEL_BACK_LEFT, 150);
    set_us(16, 3, 1, PANEL_BACK_LEFT, 135);
    set_us(17, 4, 1, PANEL_BACK_LEFT, 120);
    set_us(18, 5, 1, PANEL_BACK_LEFT, 90);
    set_us(19, 6, 1, PANEL_LEFT, 90);
    set_us(20, 1, 2, PANEL_FRONT_LEFT, 90);
    set_us(21, 4, 3, PANEL_FRONT_LEFT, 60);
    set_us(22, 3, 3, PANEL_FRONT_LEFT, 45);
    set_us(23, 2, 3, PANEL_FRONT_LEFT, 30);
    set_us(24, 1, 3, PANEL_FRONT_LEFT, 0);

    rob_command(rob, "RNOD C=000000", reply);

    return r;
}

int  rus_available(rus r)
{
    return r->available;
}


void rus_select(rus r, int sensor)
{
    int node, nr;
    
    if (sensor>=0 && sensor <r->available) {
	node = r->sensors[sensor].node;
	nr = r->sensors[sensor].nr;
	r->active_nodes |= (1<<node);
	
	if (!(r->active_sensors[node] & (1<<nr))) {
	    r->active_sensors[node] |= (1<<nr);
	    r->selected += 1;
	}
    }
}

void rus_select_all(rus r)
{
    int i;

    for (i=0; i<r->available; i++)
	rus_select(r, i);
}

int  rus_select_count(rus r)
{
    return r->selected;
}

void rus_deselect(rus r, int sensor)
{
    int node, nr;
    
    if (sensor>=0 && sensor <r->available) {
	node = r->sensors[sensor].node;
	nr = r->sensors[sensor].nr;

	if (r->active_sensors[node] & (1<<nr)) {
	    r->active_sensors[node] -= (1<<nr);
	    r->selected -= 1;

	    if (r->active_sensors == 0)
		r->active_nodes -= (1<<node);
	}
    }
}

void rus_deselect_all(rus r)
{
    int i;

    for (i=0; i<r->available; i++)
	rus_select(r, i);
}

void rus_read(rus r)
{
    int node, nr;
    long mask;
    char *string;

    strcpy(rob_cmd, "READ C=1,000000,000000");

    for (node=0; node<MAX_NR_NODES; node++) {
	if (r->active_nodes & (1<<node)) {
	    
	    rob_cmd[7] = '1' + node;
	    mask = r->active_sensors[node];
	    for (nr=0; nr<MAX_NR_SENSORS_PER_NODE; nr++) {
		rob_cmd[9+nr] = ((mask & 1)) ? '1' : '0';
		mask >>= 1;
	    }

	    rob_command(r->rob, rob_cmd, reply);
	    string = reply;

	    mask = r->active_sensors[node];
	    for (nr=0; nr<MAX_NR_SENSORS_PER_NODE; nr++) {
		if ((mask & 1)) {
		    r->sensors[r->sensor_nr[node][nr]].result = (long)
			((double)strtol(string, &string, 10) * r->result_scale); 
		}
		else (void)(strtol(string, &string, 10));

		mask >>= 1;
	    }
	}
    }
}

long rus_result(rus r, int sensor)
{
    if (sensor>=0 && sensor<r->available)
	return r->sensors[sensor].result;
    else
	return 1;
}


void rus_close(rus r)
{
    if (r->active_nodes) {
	rob_command(r->rob, "RNOD C=******", reply);
    }
    
    free(r);
}


int  rus_sensor(rus r, rus_panel panel, int angle)
{
    int i;

    for (i=0; i<r->available; i++) {
	
	if (r->sensors[i].info.panel == panel &&
	    r->sensors[i].info.angle == angle) {

	    return i;
	}
    }

    return -1;
}


void rus_select_sensor(rus r, rus_panel panel, int angle)
{
    rus_select(r, rus_sensor(r, panel, angle));
}


rus_sensor_info *rus_sensor_information(rus r, int sensor)
{
    if (sensor>=0 && sensor<r->available) {
	
	return &(r->sensors[sensor].info);
    }
    else return 0;
}


