/********************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: space.h
 *  TYPE		: module definition
 *  Time-stamp: <95/02/13 17:01:28 stolz>
 ********************************************************************************
 *  MODULE              : space
 *
 *  FUNCTION
 *
 *	This module provide a 3d-space with triangles as objects.
 *	Low level routines for triangle intersection are exported.
 *
 *
 *  NOTE
 *
 *  HISTORY
 *  02-May-94   Creation date.
 *  07-Jun-94   Add triangles_do() and triangle_edges().
 *  04-Jul-94   Change Collision-Test. Add space_new2().
 *  03-Oct-94   Add connection-mechanism.
 *
 ********************************************************************************/
#ifndef SPACE_H
#define SPACE_H


#ifndef BASIC_H
#include "basic.h"
#endif

#ifndef VECTOR_H
#include "vector.h"
#endif

#ifndef LIST_H
#include "list.h"
#endif

#ifndef TRIANGLE_H
#include "triangle.h"
#endif

#ifndef TRIANGLE_PROJECTIONH
#include "triangle_projection.h"
#endif

typedef struct adt_space*	space;
typedef struct adt_triangle*	triangle;
typedef struct adt_tp*		triangle_projection;


/* space
 */
space		space_new();
space		space_new2(vector3 ref, vector3 edges, int dim[3]);
void		space_free();

/* triangle
 */
triangle	triangle_new(space, vector3 *edges);
void		triangle_move(triangle, vector3 *edges);
void 		triangle_free(triangle);
vector3*	triangle_edges(triangle);
triangle3	triangle_triangle3(triangle t);
void 		triangles_do(space s, void (*doit)(triangle t, vector3 *edges));

/* collision
 */
boolean		triangle_collision(triangle);


/* picking
 */
triangle	triangle_pick(double *distance, space, vector3 position, vector3 direction);


/* triangle projection
 */
triangle_projection	tp_new(vector3 *edges, vector3 focus);
void			tp_free(triangle_projection tp);
vector3*		tp_focus(triangle_projection tp);
vector3*		tp_edges(triangle_projection tp);
triangle_projection	tp_root(triangle_projection tp);
triangle		tp_mirror_of(triangle_projection tp);
t3p                     tp_t3p(triangle_projection tp);
void			project_triangle(list tp_cut, list tp_rest,
					space s, triangle_projection tp);

/* user-data
 */
void		triangle_set_user_data(triangle, void *userdata);
void*		triangle_get_user_data(triangle);


/* grouping
 */
void		triangle_set_group_nr(triangle t, int nr);
int		triangle_get_group_nr(triangle t);


void		triangle_set_connections(triangle t,
                                         triangle t0, int side0,
                                         triangle t1, int side1,
                                         triangle t2, int side2);

/* printing on stdout (for debugging)
 */
void 		triangle_print(triangle t);
void 		triangles_print(list tl);
void 		tp_print(triangle_projection tp);
void 		space_print(space s);

#endif
