/**************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: space_private.h
 *  TYPE		: module definition
 *  Time-stamp: <95/02/13 17:01:34 stolz>
 **************************************************************************
 *  MODULE              : space
 *
 *  FUNCTION
 *
 *  NOTE
 *
 *  HISTORY
 *  25-Aug-94   Creation date.
 *
 **************************************************************************/
#ifndef SPACE_PRIVATE_H
#define SPACE_PRIVATE_H

#ifndef SPACE_H
#include "space.h"
#endif

#ifndef LIST_H
#include "list.h"
#endif

#ifndef VECTOR_H
#include "vector.h"
#endif

#ifndef TRIANGLE_H
#include "triangle.h"
#endif

#ifndef TRIANGLE_PROJECTION_H
#include "triangle_projection.h"
#endif



/****************************************************************************
 * definitions for the adt's and intern used structures.
 */

/* partition
 */
typedef struct {
    list        triangles;	/* triangles which are part of this part. */
    vector3     position;	/* starting-position of the partition */
    vector3     edges;          /* size of the partition */
    vector3	max_edge;	/* = position+max_edge */
} partition;


/* bounding box, (holds a 3d-array of partitions)
 */
typedef struct adt_bounding_box*  bounding_box;
typedef struct adt_bounding_box adt_bounding_box;
struct adt_bounding_box {
    vector3		position;         /* abs pos of */
    vector3		edges;            /* edges>0 */
    vector3             end_pos;          /* = position+edges */
    int                 dim[3];		  /* part the edges in the */
					  /* x/y/z-dimesion */
    long                nr_partitions;    /* x_dim*y_dim*z_dim */
    vector3             partition_edges;  /* edges / (xyz_dim) */
    long		nr_triangles;	  /* total number of triangles */
    partition*          partitions;       /* x_dim*y_dim*z_dim lists */
    list                outside_triangles; /* triangles outside the box */
};

/* abbrevation to acces a special partition
 * b = box, x,y,z are the coordinates of the partition
 */
#define part(b,x,y,z) ((b)->partitions[(x)+((b)->dim[0]*(y+((b)->dim[1]*(z))))])



/* the adt space
 */
typedef struct adt_space adt_space;
struct adt_space {
    bounding_box	box;		/* space-box */
    list                triangles;	/* triangles in this space */
};


/* the adt triangle
 * a triangle is an exclusive member of one specific space!
 *
 * many struct-elements are for efficient purposes.
 */
typedef struct adt_triangle adt_triangle;
struct adt_triangle {
    space	  space;             /* element of space ... */
    triangle3	  triangle;          /* the triangle */
    void*	  user_data;         /* pointer to user-specific data */
    list   	  member_boxes;      /* is member of the partitions ... */
    int           group_nr;
    vector3	  center;
};


typedef struct adt_tp adt_tp;
struct adt_tp {
    t3p         ptriangle;
    adt_tp	*root;
    triangle	mirror_on;
};

#endif
