/**************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: space_test.c
 *  TYPE		: module implementation
 *  Time-stamp: <95/02/13 17:01:51 stolz>
 **************************************************************************
 *  MODULE              : space_test
 *
 *  IMPLEMENTATION
 *
 *  NOTE
 *
 *  HISTORY
 *  25-Aug-94   Creation date.
 *  28-Aug-94   Add projection-test-data.
 *
 **************************************************************************/

#include <stdio.h>

#include "space_private.h"

/****************************************************************************
 * Module Test
 * for triangle_in_partition(), triangle_clip(), triangle_new() and
 * straight_line_through_triangle()
 */



/* test-datas
 */
static vector3 d1[3] = {
	+0.0, -0.1, 0.0,
	+0.2, -0.1, 0.0,
	+0.1, +0.1, 0.0
};
static vector3 d2[3] = {
	+0.4, +0.1, 0.2,
	+0.4, +0.3, 0.2,
	+0.2, +0.3, 0.2
};
static vector3 d3[3] = {
	+0.0, +0.4999, 0.0,
	+0.2, +0.4999, 0.0,
	+0.0, +0.3, 0.0
};
static vector3 d4[3] = {
	+0.7, +0.4, 0.0,
	+0.7, +0.8, 0.0,
	+0.3, +0.8, 0.0
};
static vector3 d5[3] = {
	+0.6, -0.4, 0.1,
	+0.6, +0.6, 0.1,
	-1.0, +0.6, 0.1
};
static vector3 d6[3] = {
	+0.3, +0.6, 0.0,
	+0.4, +0.4, 0.0,
	+0.2, +0.4, 0.0
};



vector3 pick_start[5] = {
	{0.59, 0.59, 0.2},
	{0.59, 0.59, 0.05},
	{0.35, 0.25, -1.0},
	{0.1, 0.1, 0.05},
        {-0.01, 0.25, 0.6}
};

vector3 pick_index[5] = {
	{0.0, 0.0, -1.0},
	{0.0, 0.0, -1.0},
	{0.0, 0.0, +1.0},
	{1.0, 1.0, 0.0},
	{0.4, 0.0, -0.6}
};

int pick_result[5] = {
	4, 3, 4, -1, 1
};

double pick_dist_result[5] = {
	0.1, 0.05, 1.1, 0.0, 0.4807401
};


/****************************************************************************
 * test datas for the triangle-projection.
 */

vector3 tp_triangle_focus = { 0.7, -0.5, 0.5 };
vector3 tp_triangle[3] = {
	{ 0.6, 0.3, 0.9 },
	{ 0.9, 0.2, 0.5 },
	{ 0.6, 0.3, 0.1 }
};

#define TP_TEST_TRIANGLES 4

vector3 tp_triangles[TP_TEST_TRIANGLES][3] = {
    {
	{ 0.8, 0.6, 0.4 },
	{ 0.9, 0.3, 0.4 },
	{ 0.9, 0.3, 0.2 }
     },
    {
	{ 0.6, 0.6, 0.6 },
	{ 0.7, 0.6, 0.5 },
	{ 0.6, 0.6, 0.4 }
     },
    {
	{ 0.6, 0.4, 0.7 },
	{ 0.7, 0.4, 0.5 },
	{ 0.6, 0.4, 0.3 }
     },
    {
	{ 0.8, 1.2, 1.1 },
	{ 1.1, 1.1, 0.6 },
	{ 0.8, 1.2, 0.6 }
     }
};

/* correct results!
 */

double tp_acc = 0.05;

#define TP_POLY_SIZE	12
vector3 tp_rest_poly[TP_POLY_SIZE] = {
	{0.6, 0.3, 0.9},
	{0.7, 2.33, 7.5},
	{0.7, 2.33, 0.69},
	{0.83, 0.23, 0.6},
	{0.9, 0.2, 0.5},
	{0.42, 0.22, 0.4},
	{0.8, 0.23, 0.4},
	{0.8, 0.23, 0.37},
	{0.6, 0.3, 0.1},
	{0.6, 0.3, 0.3},
	{0.7, 0.26, 0.5},
	{0.6, 0.3, 0.7}
};





int space_test()
{
    int failed = FALSE;
    int errors = 0;
    char index;

    triangle t[6];
    space s;
    list l;
    int i;
    double dist = 0.0;
    triangle picked_t = 0;
    triangle_projection tp;

    index = 'a';
    printf("SPACE-TEST from %s %s\n"
            "  %c) generate an empty space via space_new(..)\n",
	    __DATE__, __TIME__, index++);

    s = space_new();

#if 0
    printf("The empty space looks like:\n");

    space_print(s);
#endif
    printf("\n\n"
	   "  %c) add four test triangles to the empty space", index++);

    failed = FALSE;
    for (i=0; i<TP_TEST_TRIANGLES; i++) {
        t[i] = triangle_new(s, tp_triangles[i]);
        failed = failed || (t[i] == 0);
	if (failed) errors++;
    }
    printf(" : %s\n", (failed ? "FAILED" : "PASSED"));


    printf("\nThe space after adding the triangles is:\n");

    space_print(s);


    tp = tp_new(tp_triangle, tp_triangle_focus);
    if (failed = (tp == 0)) errors++;
    printf("  %c) Generate a triangle_projection : %s\n", index++,
	   (failed ? "FAILED" : "PASSED"));
    if (!failed) {
        list tp_cut = list_new();
        list tp_rest = list_new();

    	printf("  %c) Project the triangle in the space.\n", index++);

        project_triangle(tp_cut, tp_rest, s, tp);

	printf("  Result - Rest-faces of the triangle:\n");
	list_for_all(tp_rest, tp_print);

	printf("  Result - Projection on other triangles:\n");
	list_for_all(tp_cut, tp_print);

	list_for_all(tp_cut, tp_free);
	list_for_all(tp_rest, tp_free);
	list_free(tp_cut);
	list_free(tp_rest);
    }
    if (tp) tp_free(tp);


    printf("  %c) clear space\n", index++);
    space_free(s);
    s = space_new();



#if 0
    printf("\n\n"
		    "  %c) add six test triangles to the empty space\n",
	    index++);

    t[0] = triangle_new(s, d1);
    t[1] = triangle_new(s, d2);
    t[2] = triangle_new(s, d3);
    t[3] = triangle_new(s, d4);
    t[4] = triangle_new(s, d5);
    t[5] = triangle_new(s, d6);

    printf("\nThe space after adding the triangles is:\n");

    space_print(s);

/*
    printf("\nPLEASE CHECK THE OUTPUT:\n"
	 	    "The triangles list of the partition should look like:\n"
		    "[0][0][0] = %p %p %p %p %p\n"
		    "[0][1][0] = %p %p %p\n"
		    "[1][0][0] = %p %p\n"
		    "[1][1][0] = %p %p\n"
		    "outside   = %p %p\n"
		    "the other partitions should be empty!\n\n",
                    t[0],t[1],t[2],t[4],t[5],
		    t[3],t[4],t[5],
		    t[3],t[4],
		    t[3],t[4],
		    t[0],t[4]);
*/

    printf("\nPLEASE, ALSO CHECK THE OUTPUT:\n"
	 	    "The triangles list of the partition should look like:\n");


    l = list_new();
    list_add(l, t[0]);
    list_add(l, t[1]);
    list_add(l, t[2]);
    list_add(l, t[4]);
    list_add(l, t[5]);
    if (lists_equal(part(s->box,0,0,0).triangles, l)) { failed = FALSE; }
    else { failed = TRUE; errors++; }
    list_free(l);
    printf("[0][0][0] = %p %p %p %p %p : %s\n", t[0], t[1], t[2], t[4], t[5],
	    (failed ? "FAILED" : "PASSED"));

    if (list_size(part(s->box,0,0,1).triangles) == 0) { failed = FALSE; }
    else { failed = TRUE; errors++; }
    printf("[0][0][1] = - : %s \n",
	    (failed ? "FAILED" : "PASSED"));

    l = list_new();
    list_add(l, t[3]);
    list_add(l, t[4]);
    list_add(l, t[5]);
    if (lists_equal(part(s->box,0,1,0).triangles, l)) { failed = FALSE; }
    else { failed = TRUE; errors++; }
    list_free(l);
    printf("[0][1][0] = %p %p %p : %s\n", t[3], t[4], t[5],
	    (failed ? "FAILED" : "PASSED"));

    if (list_size(part(s->box,0,1,1).triangles) == 0) { failed = FALSE; }
    else { failed = TRUE; errors++; }
    printf("[0][1][1] = - : %s \n",
	    (failed ? "FAILED" : "PASSED"));

    l = list_new();
    list_add(l, t[3]);
    list_add(l, t[4]);
    if (lists_equal(part(s->box,1,0,0).triangles, l)) { failed = FALSE; }
    else { failed = TRUE; errors++; }
    list_free(l);
    printf("[1][0][0] = %p %p : %s\n", t[3], t[4],
	    (failed ? "FAILED" : "PASSED"));

    if (list_size(part(s->box,1,0,1).triangles) == 0) { failed = FALSE; }
    else { failed = TRUE; errors++; }
    printf("[1][0][1] = - : %s \n",
	    (failed ? "FAILED" : "PASSED"));


    l = list_new();
    list_add(l, t[3]);
    list_add(l, t[4]);
    if (lists_equal(part(s->box,1,1,0).triangles, l)) { failed = FALSE; }
    else { failed = TRUE; errors++; }
    list_free(l);
    printf("[1][1][0] = %p %p : %s\n", t[3], t[4],
	    (failed ? "FAILED" : "PASSED"));


    if (list_size(part(s->box,1,1,1).triangles) == 0) { failed = FALSE; }
    else { failed = TRUE; errors++; }
    printf("[1][1][1] = - : %s \n",
	    (failed ? "FAILED" : "PASSED"));


    l = list_new();
    list_add(l, t[0]);
    list_add(l, t[4]);
    if (lists_equal(s->box->outside_triangles, l)) { failed = FALSE; }
    else { failed = TRUE; errors++; }
    list_free(l);
    printf("outside   = %p %p : %s\n", t[0], t[4],
	    (failed ? "FAILED" : "PASSED"));


    printf("  %c) picking test via triangle_pick()\n", index++);

    for (i=0; i<5; i++) {

	    printf("     %d. pick-line (%1.2f %1.2f %1.2f) ->+ (%1.2f %1.2f %1.2f)\n",
		    i+1,
                    pick_start[i][0], pick_start[i][1], pick_start[i][2],
                    pick_index[i][0], pick_index[i][1], pick_index[i][2]);

	    picked_t = triangle_pick(&dist, s, pick_start[i], pick_index[i]);

	    failed = (((pick_result[i]>=0) ? t[pick_result[i]] : 0)
		     != picked_t);

            if (!failed && (pick_result[i]>=0 &&
		fabs(pick_dist_result[i] - dist)>0.0001))
		failed = TRUE;

	    if (failed) errors++;

	    printf("     result: triangle=%p, distance=%1.3f, "
		    "expected t=%p,d=%1.3f : %s\n",
		    picked_t, dist,
		    ((pick_result[i]>=0) ? t[pick_result[i]] : 0 ),
                    pick_dist_result[i],
		    (failed ? "FAILED" : "PASSED"));

    }

#endif

    space_free(s);

    printf("Total amount of errors: %d. Test %s!\n\n",
			errors, errors ? "FAILED" : "PASSED");


    return errors;
}



int main(int argc, char **argv)
{
    return space_test();
}
