/**************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: triangle.h
 *  TYPE		: module definition
 *  Time-stamp: <95/02/13 17:02:33 stolz>
 **************************************************************************
 *  MODULE              : triangle
 *
 *  FUNCTION
 *
 *  NOTE
 *
 *  HISTORY
 *  07-Mar-94   Creation date.
 *  27-May-94   Add triangle3_distance().
 *  23-Jun-94   triangle3_new() returns now 0 if the triangle is to small!
 *  03-Oct-94   Add connection-mechanism.
 *
 **************************************************************************/
#ifndef TRIANGLE_H
#define TRIANGLE_H

#ifndef BASIC_H
#include "basic.h"
#endif

#ifndef VECTOR_H
#include "vector.h"
#endif

#ifndef LIST_H
#include "list.h"
#endif



typedef struct adt_triangle3*   triangle3;

/* creation/deletion of a triangle
 */
triangle3	triangle3_new(vector3 *edges);
void 		triangle3_free(triangle3 t);
void 		triangle3_list_free(list tl);

/* request shape-information of the triangle
 */
vector3*	triangle3_abs_edges(triangle3 t);
vector3*	triangle3_rel_edges(triangle3 t);
double*		triangle3_edges_length(triangle3 t);
void		triangle3_normal(vector3 *normal, triangle3 t);


boolean		triangle3_equal(triangle3 t1, triangle3 t2);

boolean 	triangle3_piercing(triangle3 t1, triangle3 t2);
list 		triangle3_clip(list triangle3, int xyz_select,
			       double min, double max);
boolean 	straight_line_trough_triangle(vector3 line_start,
					      vector3 line_index,
                                              triangle3 t);

boolean		triangle3_distance(double *factor, vector3 line_start,
                                   vector3 line_index, triangle3 t);
boolean		line3_in_t3_plane(double *factor,
				  vector2 *point2,
				  vector3 *point3,
		                  vector3 line_start, vector3 line_index,
			          triangle3 t);
void		point3_in_basis_t3(vector3 *v3, vector2 v2, triangle3 basis);
void		triangle3_of_triangle2(vector3 *t3, vector2 t2[3],
					triangle3 basis);

void		triangle3_set_connections(triangle3 t,
                                          triangle3 t0, int side0,
                                          triangle3 t1, int side1,
                                          triangle3 t2, int side2);

void		triangle3_connection(triangle3 *ct, int *cside, triangle3 t, int side);

void 		triangle3_print(triangle3 t);
void 		triangle3_print_list(list tl);


#endif
