/**************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: triangle_test.c
 *  TYPE		: module implementation
 *  Time-stamp: <95/02/13 17:03:15 stolz>
 **************************************************************************
 *  MODULE              : triangle_test
 *
 *  IMPLEMENTATION
 *
 *  NOTE
 *      This are module-test-routines for the triangle-module.
 *
 *  HISTORY
 *  19-Jul-94   Creation date
 *
 **************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>


#include "triangle_projection.h"

#ifdef USE_IV
#include "iv_draw.h"
#endif



#define ITEMS 27

#define DISTANCE 1e+10   /* 1e+10 */


vector3 focus[ITEMS] = {  /* initial-focus, and focus of the rest-faces */
    {0.0, 0.0, DISTANCE},
    {0.0, 0.0, DISTANCE},
    {0.0, 0.0, DISTANCE},
    {0.0, 0.0, DISTANCE},
    {0.0, 0.0, DISTANCE},
    {0.0, 0.0, DISTANCE},
    {0.0, 0.0, DISTANCE},
    {0.0, 0.0, DISTANCE},
    {0.0, 0.0, DISTANCE},
    {0.0, 0.0, DISTANCE},

    {0.0, 0.0, DISTANCE},
    {0.0, 0.0, DISTANCE},
    {0.0, 0.0, DISTANCE},
    {0.0, 0.0, DISTANCE},
    {0.0, 0.0, DISTANCE},
    {0.0, 0.0, DISTANCE},
    {1.5936408333706207, 0.81000000000000005, 0.26300000000000001},
    {1.1736408333706212, 1.04, 0.26300000000000001},
    {1.8036408333706209, 1.25, 0.26300000000000001},
    {1.483641, 0.830000, 0.263000},

    {0.923641, 0.830000, 0.263000},
    {3.289904, 3.228050, 0.263000},
    {2.723641, 0.830000, 0.263000},
    {-0.863641, 0.830000, 0.263000},
    {0.912897, 2.606532, 0.263000},
    {-1.830500, 1.914641, 0.263000},
    {-0.056418, -0.662645, 0.263000}
};

vector3 focus2[ITEMS] = { /* focus for the cut-triangles (the mirror projection) */
    {0.0, 0.0, -DISTANCE},
    {0.0, 0.0, -DISTANCE},
    {0.0, 0.0, -DISTANCE},
    {0.0, 0.0, -DISTANCE},
    {0.0, 0.0, -DISTANCE},
    {0.0, 0.0, -DISTANCE},
    {0.0, 0.0, -DISTANCE},
    {0.0, 0.0, -DISTANCE},
    {0.0, 0.0, -DISTANCE},
    {0.0, 0.0, -DISTANCE},

    {0.0, 0.0, -DISTANCE},
    {0.0, 0.0, -DISTANCE},
    {0.0, 0.0, -DISTANCE},
    {0.0, 0.0, -DISTANCE},
    {0.0, 0.0, -DISTANCE},
    {0.0, 0.0, -DISTANCE},
    {-1.5936408333706207, 0.81000000000000005, 0.26300000000000001}, 
	/* ^WRONG!(dosn't matter - no rest-triangles!) */
    {-2.891641, 1.040000, 0.263000},
    {1.803641, 0.570000, 0.263000},
    {-3.136841, 0.830000, 0.263000},

    {0.923641, 0.830000, 0.263000},
    {3.289904, 3.228050, 1.737000},
    {2.723641, 0.830000, 0.747000},
    {-0.863641, 0.830000, -0.263000},
    {0.912897, 2.606532, 0.263000},
    {-1.830500, 1.914641, 0.263000},
    {-0.06, -9.0, 0.263}
};

static vector3 t3p_edges[ITEMS][3] = {
    {
        {-4.0, +2.5, +0.1},
        {-3.0, +4.0, +0.1},
        {-3.0, +2.5, +0.1}
    }, {
        {-2.0, +2.5, +0.1},
        {-0.5, +5.0, +0.1},
        {+1.5, +2.5, +0.1}
    }, {
        {+1.0, +3.5, +0.1},
        {+7.0, +3.5, +0.1},
        {+4.0, +2.0, +0.1}
    }, {
        {-4.5, -1.0, +0.1},
        {-3.0, +2.0, +0.1},
        {-0.5,  0.0, +0.1}
    }, {
        {+1.0, -1.0, +0.1},
        {+3.0, +1.0, +0.1},
        {+3.0, -2.5, +0.1}
    }, {
        {+4.0, -1.0, +0.1},
        {+6.0, +1.0, +0.1},
        {+6.0, -2.0, +0.1}
    }, {
        {-5.5, -3.5, +0.1},
        {-4.0, -2.0, +0.1},
        {-2.5, -3.5, +0.1}
    }, {
        {-1.5, -4.0, +0.1},
        {-0.5, -2.5, +0.1},
        {-0.5, -3.5, +0.1}
    }, {
	{+2.0, -3.0, +0.1},
	{+5.0, -3.0, +0.1},
	{+5.0, -5.0, +0.1}
    }, {
	{+8.0, +1.5, +0.1},
	{+9.5, +3.0, +0.1},
	{+11.0, +1.5, +0.1}
    }, {
	{+9.0, -0.5, +0.1},
	{+10.5, +1.0, +0.1},
	{+9.5, -1.0, +0.1}
    }, {
	{-3.0,  0.0, +0.1},
	{-1.0,  0.0, +0.1},
	{-3.0, -1.5, +0.1}
    }, {
	{+1.0,  0.0, +0.1},
	{+3.0,  0.0, +0.1},
	{+1.0, -2.0, +0.1}
    }, {
	{+6.0,  0.0, +0.1},
	{+6.5, -1.0, +0.1},
	{+5.5, -1.0, +0.1}
    }, {
	{+9.0,  0.0, +0.1},
	{+10.5, 0.0, +0.1},
	{+10.5, -1.0, +0.1}
    }, {
	{-2.0,  0.0, +0.1},
	{-0.5, -1.5, +0.1},
	{-2.5, -1.5, +0.1}
    }, {
	{0.31999999999999995, 0.55404609865892751, 0.51895390134107244}, 
	{0.31999999999999995, 0.80227359985214008, 0.25527359985214004},
	{0.31999999999999995, 0.80227359985214008, 0.27072640014785998}
    }, {
	{0.12, 0.8282578236756396, 0.051257823675638026},
	{0.12, 0.93529116760488795, 0.2707754083461717},
	{0.12, 0.93529116760488795, 0.25522459165382833}
    }, {
	{0.29000000000000004, 1.1107967543845894, 0.2406982029843826},
	{0.29000000000000004, 1.1011636288533766, 0.25976145939747447},
	{0.29000000000000026, 1.1008356881063215, 0.2564192215341024}
    }, {
	{0.310000, 0.594142, 0.498858},
	{0.310000, 0.960000, 0.027143},
	{0.310000, 0.690295, 0.385940}
    }, {
	{0.030000, 0.969628, 0.134622},
	{0.030000, 0.970410, 0.131270},
	{0.030000, 0.970410, 0.134748}
    }, {
	{2.063013, 0.556987, 1.000000},
	{1.777846, 1.050911, 1.000000},
	{1.897084, 0.844385, 0.668142}
    }, {
	{0.930000, 0.775236, 0.376522},
	{0.930000, 0.703687, 0.288395},
	{0.930000, 0.691065, 0.385262}
    }, {
	{-0.789000, 0.849211, 0.241033},
	{-0.789000, 0.845000, 0.240772},
	{-0.789000, 0.845000, 0.248000}
    }, {
	{0.649891, 1.093002, -0.000001},
	{0.723156, 1.019736, 0.073265},
	{0.919999, 0.822893, 0.019799}
    }, {
	{-1.895741, 1.590000, 0.328241},
	{-1.895741, 1.590000, 0.197759},
	{-1.887348, 1.590000, 0.254720}
    }, {
	{-0.08, -0.74, 0.28}, 
	{-0.05, -0.74, 0.28},
	{-0.05, -0.74, 0.25}
    }
};

static vector3 t3_edges[ITEMS][3] = {
    {
	{-5.5, +2.5, 0.0},
	{-5.5, +4.5, 0.0},
	{-2.5, +4.5, 0.0}
    }, {
	{-1.0, +3.5, 0.0},
	{+0.5, +3.5, 0.0},
	{0.0, +1.5, 0.0}
    }, {
	{+2.5, +1.5, 0.0},
	{+4.0, +4.5, 0.0},
	{+5.5, +1.5, 0.0}
    }, {
	{-5.0, +1.5, 0.0},
	{-2.0, +0.5, 0.0},
	{-6.0, -0.5, 0.0}
    }, {
	{+1.5, +1.0, 0.0},
	{+2.5, -1.0, 0.0},
	{+1.5, -1.0, 0.0}
    }, {
	{+5.0, +1.0, 0.0},
	{+7.0, -1.0, 0.0},
	{+5.0, -1.0, 0.0}
    }, {
	{-5.5, -2.5, 0.0},
	{-2.5, -2.5, 0.0},
	{-4.0, -4.0, 0.0}
    }, {
	{-2.0, -4.5, 0.0},
	{-1.0, -2.0, 0.0},
	{+1.5, -3.5, 0.0}
    }, {
	{+3.5, -3.5, 0.0},
	{+4.5, -3.5, 0.0},
	{+4.5, -4.0, 0.0}
    }, {
	{+8.0, +3.5, 0.0},
	{+11.0, +3.5, 0.0},
	{+9.5, +2.0, 0.0}
    }, {
	{+8.0,  0.0, 0.0},
	{+11.0,  0.0, 0.0},
	{+9.5, -1.5, 0.0}
    }, {
	{-3.0, -1.0, 0.0},
	{-3.0,  0.0, 0.0},
	{-2.0,  0.0, 0.0}
    }, {
	{+2.0,  0.0, 0.0},
	{+2.0, -1.0, 0.0},
	{+1.0, -1.0, 0.0}
    }, {
	{+4.5,  0.0, 0.0},
	{+6.5,  0.0, 0.0},
	{+6.5, -2.0, 0.0}
    }, {
	{+8.0,  0.0, 0.0},
	{+11.0,  0.0, 0.0},
	{+11.0, -2.0, 0.0}
    }, {
	{-3.0,  0.0, 0.0},
	{-1.0,  0.0, 0.0},
	{-1.0, -2.0, 0.0}
    }, {
	{-0.87900000000000011, 0.79500000000000004, 0.27800000000000002},
	{-0.87900000000000011, 0.82500000000000007, 0.27800000000000002},
	{-0.87900000000000011, 0.82500000000000007, 0.248}
    }, {
	{-0.8590000000000001, 0.80800000000000005, 0.27800000000000002},
	{-0.8590000000000001, 0.80800000000000005, 0.248},
	{-0.8590000000000001, 0.83800000000000008, 0.248}
    }, {
	{-1.6764999999999999, 0.90999999999999992, 0.27800000000000002},
	{-1.6764999999999999, 0.90999999999999992, 0.248},
	{-1.6465000000000001, 0.90999999999999992, 0.248}
    }, {
	{-0.826600, 0.555000, 0.005000}, 
	{-0.826600, 1.105000, 0.005000}, 
	{-0.826600, 0.555000, 0.505000}
    }, {
	{-1.776600, 1.105000, 0.005000}, 
	{-1.776600, 1.105000, 0.505000}, 
	{-0.826600, 1.105000, 0.005000}
    }, {
	{1.196987, 0.056987, 1.000000},
	{2.063013, 0.556987, 1.000000},
	{0.696987, 0.923013, 1.000000}
    }, {
	{-1.776600, 0.555000, 0.505000},
	{-0.826600, 0.555000, 0.505000}, 
	{-1.776600, 1.105000, 0.505000}
    }, {
	{0.030000, 0.060000, 0.000000}, 
	{0.030000, 1.060000, 0.000000},
	{-0.170000, 0.060000, 0.000000}
    }, {
	{-0.789000, 0.815000, 0.278000},
	{-0.789000, 0.845000, 0.278000},
	{-0.789000, 0.845000, 0.248000}
    }, {
	{-1.945893, 1.314607, 0.278000}, 
	{-1.945893, 1.314607, 0.248000}, 
	{-1.967107, 1.293393, 0.248000}
    }, {
	{5.00, -5.00, -3.50}, 
	{5.00, -5.00, 6.50}, 
	{-5.00, -5.00, -3.50}
    }
};





static int tp_cut_triangles[ITEMS] = {
    0, 2, 3, 1, 2, 3, 4, 1, 1, 2,
    2, 1, 1, 1, 1, 3, 0, 1, 2, 2,
    0, 0 /* t&tp has one equal (1 before) */ , 2, 1, 0, 0, 0
};
static int tp_cut_index[ITEMS] = {
    -1, 0, 2, 5, 6, 8, 11, 15, 16, 17,
    19, 21, 22, 23, 24, 25, -1, 28, 29, 31,
    -1, 33, 34, 36, -1, -1, -1
};


static vector3 tp_cut_results[][3] = {
    /* 2. cut */
    {
	{0.25, 2.50, 0.00},
	{-1.00, 3.50, 0.00},
	{-0.50, 2.50, 0.00}
    }, {
	{0.25, 2.50, 0.00},
	{0.50, 3.50, 0.00},
	{-1.00, 3.50, 0.00}

	/* 3. CUT: */
    }, {
	{3.50, 3.50, 0.00},
	{5.00, 2.50, 0.00},
	{4.50, 3.50, 0.00}
    }, {
	{3.50, 3.50, 0.00},
	{4.00, 2.00, 0.00},
	{5.00, 2.50, 0.00}
    }, {
	{3.50, 3.50, 0.00},
	{3.00, 2.50, 0.00},
	{4.00, 2.00, 0.00}

	/* 4. CUT: */
    }, {
	{-4.00, 0.00, 0.00},
	{-2.00, 0.50, 0.00},
	{-3.50, 1.00, 0.00}

	/* 5. CUT: */
    }, {
	{1.50, -0.50, 0.00},
	{2.50, -1.00, 0.00},
	{2.00, 0.00, 0.00}
    }, {
	{1.50, -0.50, 0.00},
	{1.50, -1.00, 0.00},
	{2.50, -1.00, 0.00}

	/* 6. CUT: */
    }, {
	{5.00, 0.00, 0.00},
	{6.00, 0.00, 0.00},
	{5.50, 0.50, 0.00}
    }, {
	{5.00, 0.00, 0.00},
	{6.00, -1.00, 0.00},
	{6.00, 0.00, 0.00}
    }, {
	{5.00, 0.00, 0.00},
	{5.00, -1.00, 0.00},
	{6.00, -1.00, 0.00}

	/* 7. CUT: */
    }, {
	{-5.00, -3.00, 0.00},
	{-3.50, -2.50, 0.00},
	{-4.50, -2.50, 0.00}
    }, {
	{-5.00, -3.00, 0.00},
	{-3.00, -3.00, 0.00},
	{-3.50, -2.50, 0.00}
    }, {
	{-5.00, -3.00, 0.00},
	{-3.50, -3.50, 0.00},
	{-3.00, -3.00, 0.00}
    }, {
	{-5.00, -3.00, 0.00},
	{-4.50, -3.50, 0.00},
	{-3.50, -3.50, 0.00}

	/* 8. CUT: */
    }, {
	{-1.50, -4.00, 0.00},
	{-0.50, -2.50, 0.00},
	{-0.50, -3.50, 0.00}

	    /* 9. CUT: */
    }, {
	{3.50, -3.50, 0.00},
	{4.50, -3.50, 0.00},
	{4.50, -4.00, 0.00}

	/* 10. CUT: */
    }, {
	{9.00, 2.50, 0.00},
	{10.00, 2.50, 0.00},
	{9.50, 3.00, 0.00}
    }, {
	{9.00, 2.50, 0.00},
	{9.50, 2.00, 0.00},
	{10.00, 2.50, 0.00}

	/* 11. CUT: */
    }, {
	{9.00, -0.50, 0.00},
	{10.00, 0.00, 0.00},
	{9.50, -1.00, 0.00}
    }, {
	{9.00, -0.50, 0.00},
	{9.50, 0.00, 0.00},
	{10.00, 0.00, 0.00}

	/* 12. CUT: */
    }, {
	{-3.00, -1.00, 0.00},
	{-3.00, 0.00, 0.00},
	{-2.00, 0.00, 0.00}

	/* 13. CUT: */
    }, {
	{2.00, 0.00, 0.00},
	{2.00, -1.00, 0.00},
	{1.00, -1.00, 0.00}

	/* 14. CUT: */
    }, {
	{6.00, 0.00, 0.00},
	{6.50, -1.00, 0.00},
	{5.50, -1.00, 0.00}

    /* 15. CUT: */
    }, {
	{9.00, 0.00, 0.00},
	{10.50, 0.00, 0.00},
	{10.50, -1.00, 0.00}

	/* 16. CUT: */
    }, {
	{-2.00, 0.00, 0.00},
	{-1.50, -1.50, 0.00},
	{-2.25, -0.75, 0.00}
    }, {
	{-2.00, 0.00, 0.00},
	{-1.00, -1.50, 0.00},
	{-1.50, -1.50, 0.00}
    }, {
	{-2.00, 0.00, 0.00},
	{-1.00, -1.00, 0.00},
	{-1.00, -1.50, 0.00}
    }, {
	/* 18. CUT: */
	{-0.8590000000000001, 0.8233724688038242, 0.24800000000000003}, 
	{-0.8590000000000001, 0.83800000000000008, 0.248},
	{-0.8590000000000001, 0.82816692332909336, 0.25783307667090671}
    }, {
	/* 19. CUT: */
	{-1.676500, 0.910000, 0.251196},
	{-1.654102, 0.910000, 0.255602},
	{-1.646500, 0.910000, 0.248000}
    }, {
	{-1.676500, 0.910000, 0.251196},
	{-1.646500, 0.910000, 0.248000},
	{-1.676500, 0.910000, 0.248000}
    }, {
	/* 20. CUT: */
	{-0.826600, 0.555000, 0.483236},
	{-0.826600, 0.925916, 0.005000},
	{-0.826600, 0.930846, 0.005000}
    }, {
	{-0.826600, 0.555000, 0.483236},
	{-0.826600, 0.930846, 0.005000},
	{-0.826600, 0.555000, 0.505000}
    }, {
	/* 22. CUT: */
	{2.063013, 0.556987, 1.000000},
	{1.349075, 0.144795, 1.000000},
	{1.602511, 0.680378, 1.000000}
    }, {
	/* 23. CUT: */
	{-1.099945, 0.713257, 0.505000},
	{-1.776600, 0.555000, 0.505000},
	{-0.826613, 0.555000, 0.505000}
    }, {
	{-1.099945, 0.713257, 0.505000},
	{-1.776600, 0.657827, 0.505000},
	{-1.776600, 0.555000, 0.505000}
    }, {
	/* 24. CUT: */
	{0.030000, 1.060004, 0.000000},
	{0.019506, 1.007479, 0.000000},
	{0.030000, 1.009588, 0.000000}
    }
};


static int tp_rest_triangles[ITEMS] = {
    0, 5, 2, 4, 4/*count 5*/, 3/*count 4*/, 3, 0, 6, 3, 
    1, 2, 3, 0, 0, 2, 0, 2, 2, 2, 
    0, 0 /* t&tp has one equal edge 2 before*/, 2, 1, 0, 0, 0
};
static int tp_rest_index[ITEMS] = {
    -1, 0, 5, 7, 11, 16, 20, -1, 23, 29, 
    32, 33, 35, -1, -1, 38, -1, 40, 42, 44,
    -1, 46, 48, 50, -1, -1, -1
};

static vector3 tp_rest_results[][3] = {
	/* II. rest */
    {
	{-2.00, 2.50, 0.10},
	{-1.00, 3.50, 0.10},
	{-0.50, 2.50, 0.10}
    }, {
	{-2.00, 2.50, 0.10},
	{-0.50, 5.00, 0.10},
	{-1.00, 3.50, 0.10}
    }, {
	{-0.50, 5.00, 0.10},
	{0.50, 3.50, 0.10},
	{-1.00, 3.50, 0.10}
    }, {
	{-0.50, 5.00, 0.10},
	{1.50, 2.50, 0.10},
	{0.50, 3.50, 0.10}
    }, {
	{1.50, 2.50, 0.10},
	{0.25, 2.50, 0.10},
	{0.50, 3.50, 0.10}


	/* 3. REST: */
    }, {
	{7.00, 3.50, 0.10},
	{5.00, 2.50, 0.10},
	{4.50, 3.50, 0.10}
    }, {
	{1.00, 3.50, 0.10},
	{3.50, 3.50, 0.10},
	{3.00, 2.50, 0.10}

    /* 4. REST: */
    }, {
	{-4.50, -1.00, 0.10},
	{-2.00, 0.50, 0.10},
	{-0.50, 0.00, 0.10}
    }, {
	{-2.00, 0.50, 0.10},
	{-3.00, 2.00, 0.10},
	{-0.50, 0.00, 0.10}
    }, {
	{-2.00, 0.50, 0.10},
	{-3.50, 1.00, 0.10},
	{-3.00, 2.00, 0.10}
    }, {
	{-4.50, -1.00, 0.10},
	{-4.00, 0.00, 0.10},
	{-2.00, 0.50, 0.10}

	/* 5. REST: */
    }, {
	{1.00, -1.00, 0.10},
	{2.50, -1.00, 0.10},
	{3.00, -2.50, 0.10}
    }, {
	{2.50, -1.00, 0.10},
	{3.00, 1.00, 0.10},
	{3.00, -2.50, 0.10}
    }, {
	{2.50, -1.00, 0.10},
	{2.00, 0.00, 0.10},
	{3.00, 1.00, 0.10}
    }, {
	{1.00, -1.00, 0.10},
	{1.50, -0.50, 0.10},
	{1.50, -1.00, 0.10}
    }, {
	{1.00, -1.00, 0.10},		/* zero-triangle */
	{1.50, -1.00, 0.10},
	{2.50, -1.00, 0.10}

	/* 6. REST: */
    }, {
	{6.00, 1.00, 0.10},
	{6.00, 0.00, 0.10},
	{5.50, 0.50, 0.10}
    }, {
	{4.00, -1.00, 0.10},
	{5.00, 0.00, 0.10},
	{5.00, -1.00, 0.10}
    }, {
	{4.00, -1.00, 0.10},
	{6.00, -1.00, 0.10},
	{6.00, -2.00, 0.10}
    }, {                                   /* zero-triangle */
	{4.00, -1.00, 0.10},
	{5.00, -1.00, 0.10},
	{6.00, -1.00, 0.10}

	/* 7. REST: */
    }, {
	{-2.50, -3.50, 0.10},
	{-3.50, -3.50, 0.10},
	{-3.00, -3.00, 0.10}
    }, {
	{-4.00, -2.00, 0.10},
	{-3.50, -2.50, 0.10},
	{-4.50, -2.50, 0.10}
    }, {
	{-5.50, -3.50, 0.10},
	{-5.00, -3.00, 0.10},
	{-4.50, -3.50, 0.10}

    /* 8. REST: */
    /* 9. REST: */
    }, {
	{5.00, -5.00, 0.10},
	{3.50, -3.50, 0.10},
	{4.50, -4.00, 0.10}
    }, {
	{5.00, -5.00, 0.10},
	{2.00, -3.00, 0.10},
	{3.50, -3.50, 0.10}
    }, {
	{5.00, -3.00, 0.10},
	{4.50, -4.00, 0.10},
	{4.50, -3.50, 0.10}
    }, {
	{5.00, -3.00, 0.10},
	{5.00, -5.00, 0.10},
	{4.50, -4.00, 0.10}
    }, {
	{2.00, -3.00, 0.10},
	{4.50, -3.50, 0.10},
	{3.50, -3.50, 0.10}
    }, {
	{2.00, -3.00, 0.10},
	{5.00, -3.00, 0.10},
	{4.50, -3.50, 0.10}

	/* 10. REST: */
    }, {
	{8.00, 1.50, 0.10},
	{9.50, 2.00, 0.10},
	{11.00, 1.50, 0.10}
    }, {
	{9.50, 2.00, 0.10},
	{10.00, 2.50, 0.10},
	{11.00, 1.50, 0.10}
    }, {
	{8.00, 1.50, 0.10},
	{9.00, 2.50, 0.10},
	{9.50, 2.00, 0.10}

	/* 11. REST: */
    }, {
	{10.50, 1.00, 0.10},
	{10.00, 0.00, 0.10},
	{9.50, 0.00, 0.10}

	/* 12. REST: */
    }, {
	{-1.00, 0.00, 0.10},
	{-3.00, -1.00, 0.10},
	{-2.00, 0.00, 0.10}
    }, {
	{-1.00, 0.00, 0.10},
	{-3.00, -1.50, 0.10},
	{-3.00, -1.00, 0.10}

    /* 13. REST: */
    }, {
	{1.00, -2.00, 0.10},
	{1.00, -1.00, 0.10},
	{2.00, -1.00, 0.10}
    }, {
	{1.00, 0.00, 0.10},
	{2.00, -1.00, 0.10},
	{1.00, -1.00, 0.10}
    }, {
	{1.00, 0.00, 0.10},
	{3.00, 0.00, 0.10},
	{2.00, -1.00, 0.10}

	/* 14. REST: */
	/* 15. REST: */
	/* 16. REST: */
    }, {
	{-2.50, -1.50, 0.10},
	{-2.25, -0.75, 0.10},
	{-1.50, -1.50, 0.10}
    }, {
	{-0.50, -1.50, 0.10},
	{-1.00, -1.50, 0.10},
	{-1.00, -1.00, 0.10}
    }, {
	/* 18. REST: */
 	{0.120000, 0.828258, 0.051258},
	{0.120000, 0.927709, 0.255225},
	{0.120000, 0.935291, 0.255225}
    }, {
	{0.120000, 0.935291, 0.255225},
	{0.120000, 0.930194, 0.260322},
	{0.120000, 0.935291, 0.270775}
    }, {
	/* 19. REST: */
	{0.290000, 1.110797, 0.240698},
	{0.290000, 1.102122, 0.257866},
	{0.290000, 1.102122, 0.256476}
    }, {
	{0.290000, 1.110797, 0.240698},
	{0.290000, 1.102122, 0.256476},
	{0.290000, 1.100836, 0.256419}
    }, {
	/* 20. REST: */
	{0.310000, 0.960000, 0.027143},
	{0.310000, 0.881231, 0.131932},
	{0.310000, 0.878727, 0.131932}
    }, {
	{0.310000, 0.594142, 0.498858},
	{0.310000, 0.690295, 0.374884},
	{0.310000, 0.690295, 0.385940} 
    }, {
	/* 21. REST: */
	/* 22. REST: */
	{1.777846, 1.050911, 1.000000},
	{1.810484, 0.994381, 0.909164},
	{2.063013, 0.556987, 1.000000}
    }, {
	{1.897084, 0.844385, 0.668142},
	{2.063013, 0.556987, 1.000000},
	{1.845591, 0.933574, 0.811456}
    }, {
	/* 23. REST: */
	{0.930000, 0.703687, 0.288395},
	{0.930000, 0.691065, 0.385262},
	{0.930000, 0.720394, 0.359453}
    }, {
	{0.930000, 0.703687, 0.288395},
	{0.930000, 0.720394, 0.359453},
	{0.930000, 0.761378, 0.359453}
    }, {
	/* 24. REST: */
	{-0.789000, 0.845000, 0.248000},
	{-0.789000, 0.849211, 0.241033},
	{-0.789000, 0.845000, 0.241033}
    }
};


int t3p_list_verify(int nr,
		    int *nr_t, int *index, vector3 edges[][3], vector3 focus,
		    list cut)
{
    t3p tp_result, tp;
    int errors = 0;
    int i;
    list cut2;
    boolean failed;

    /* copy the cut-list
     */
    cut2 = list_new();
    tp = list_first(cut);
    while (tp) {
	list_add(cut2, tp);
	tp = list_next(cut);
    }

    failed = (list_size(cut) != nr_t[nr]);
    if (failed) {
	errors++;
	printf("t3p_list_verify() - "
	       "WRONG number of triangles, %d != %d(correct)\n",
	       list_size(cut), nr_t[nr]);
    }

    if (index[nr]>=0) for (i=0; i<nr_t[nr]; i++) {
	tp_result = t3p_new(edges[index[nr] + i], focus);

	if (tp_result==0) {
	    printf("t3p_list_verify() - "
		   "WARNING result contain a zero-triangle\n"
		   " (%f %f %f) (%f %f %f) (%f %f %f)\n",
		   v3_elems(edges[index[nr]+i][0]),
		   v3_elems(edges[index[nr]+i][1]),
		   v3_elems(edges[index[nr]+i][2]));
	    continue;
	}

	tp = list_first(cut);
	while (tp) {
	    if (t3p_equal(tp_result, tp)) {
		list_remove(cut2, tp);
		break;
	    }
	    tp = list_next(cut);
	}
	if (tp==0) {
	    failed = TRUE;
	    errors++;
	    printf("t3p_list_verify() - MISSING projection\n:");
	    t3p_print(tp_result);
	}

	t3p_free(tp_result);
    }

    tp = list_first(cut2);
    while (tp) {
        failed = TRUE;
	errors++;
	printf("t3p_list_verify() - WRONG projection in list\n:");
	t3p_print(tp);
	tp = list_next(cut2);
    }

    list_free(cut2);
    
    return errors;
}



int t3p_test(int item_start, int item_stop)
{
    int failed = FALSE;
    int errors = 0;
    char index = 'a';

    triangle3 t[ITEMS];
    t3p tp[ITEMS];
    int i;
    list cut;
    list rest;
    time_t now;
    char *time_string = "No time available!";


    now = time(0);
    if (now!=-1) time_string = ctime(&now);

    printf("TRIANGLE3-PROJECTION-TEST at %s"
	   "Test code compiled on: %s %s\n",
	   time_string, __DATE__, __TIME__);

    printf("\n%c) generate %d triangles and %d projection-triangles.\n",
	    index++, ITEMS, ITEMS);
    for (i=0; i<ITEMS; i++) {
	t[i] = triangle3_new(&t3_edges[i][0]);
        if (t[i]==0) {
	    printf("ERROR! Can't create %d.triangle\n", i+1);
	    errors++;
	}
 	tp[i] = t3p_new(&t3p_edges[i][0], focus[i]);
        if (tp[i]==0) {
	    printf("ERROR! Can't create %d.proj-triangle\n", i+1);
	    errors++;
	}
    }

    cut = list_new();
    rest = list_new();

    if (item_start<0) item_start = 0;
    if (item_stop>ITEMS) item_stop = ITEMS;
    if (item_start>=item_stop) {
	item_start = 0;
	item_stop = ITEMS;
    }

    printf("%c) Projection test\n", index++);
    for (i=item_start; i<item_stop; i++) {
	printf("%d. t3 against t3p. Result : ", i+1);
	
	triangle3_projection(cut, rest, tp[i], t[i]);

#ifdef USE_IV
	iv_color(1.0, 1.0, 1.0);
	iv_draw_t3(t[i]);
	iv_color(1.0, 0.0, 0.0);
	iv_draw_t3p(tp[i]);
	iv_color(0.0, 1.0, 0.0); 
	list_for_all(rest, iv_draw_t3p); 
	iv_color(0.0, 0.0, 1.0);
	list_for_all(cut, iv_draw_t3p); 
#endif

        printf("%d cuts and %d rests\n", list_size(cut), list_size(rest));

        printf("...check cut-triangles...\n");
	failed = t3p_list_verify(i,
				 tp_cut_triangles,
				 tp_cut_index,
				 tp_cut_results,
				 focus2[i],
				 cut);

	errors+=failed;

        printf("...check rest-triangles...\n");
	failed = t3p_list_verify(i,
				 tp_rest_triangles,
				 tp_rest_index,
				 tp_rest_results,
				 focus[i],
				 rest);

	errors += failed;

	printf("... : %s\n", (failed ? "FAILED" : "PASSED"));

	list_for_all(cut, t3p_free);
	list_for_all(rest, t3p_free);
        list_remove_all(cut);
        list_remove_all(rest);
    }

    printf("%c) freeing all triangles.\n", index++);
    for (i=0; i<ITEMS; i++) {
	if (t[i]) triangle3_free(t[i]);
	if (tp[i]) t3p_free(tp[i]);
    }

    list_free(cut);
    list_free(rest);

    printf("Total amount of errors: %d. Test %s!\n\n",
			errors, errors ? "FAILED" : "PASSED");


    return errors;
}



int main(int argc, char **argv)
{
    int rc;
    int start = 0;
    int stop = ITEMS;


    if (argc>1) {
	start = atoi(argv[1]);
    }
    if (argc>2) {
	stop = atoi(argv[2]);
    }

#ifdef USE_IV
    if (argc>3)
	iv_init(argc, argv, 0);
    else
	iv_init(argc, argv, 1);
#endif

    rc = t3p_test(start, stop);

#ifdef USE_IV
    iv_main_loop();
#endif

    return rc;
}
