/**************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: triangle_test.c
 *  TYPE		: module implementation
 *  Time-stamp: <95/02/13 17:02:53 stolz>
 **************************************************************************
 *  MODULE              : triangle_test
 *
 *  IMPLEMENTATION
 *
 *  NOTE
 *      This are module-test-routines for the triangle-module.
 *
 *  HISTORY
 *  19-Jul-94   Creation date
 *
 **************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "triangle.h"


/****************************************************************************
 * Module Test
 * for triangle_clip() and
 * straight_line_through_triangle()
 */


/* test-datas for clipping
 */
static vector3 da[3] = {
	-7.0, +1.0, 0.0,
	-1.0, +1.0, 0.0,
	-5.0, +5.0, 0.0
};

static vector3 db[3] = {
	-5.0, -5.0, 0.0,
	-2.0, -5.0, 0.0,
	-3.0, -1.0, 0.0
};

static vector3 dc[3] = {
	+1.0, -2.0, 0.0,
	+4.0, -2.0, 0.0,
	+1.0, +1.0, 0.0
};

static vector3 dd[3] = {
	+1.0, +4.0, 0.0,
	+3.0, +6.0, 0.0,
	-1.0, +6.0, 0.0
};


/* test-datas for straight-line...
 */
static vector3 dl[3] = {
	-1.0, +3.0, 2.0,
	+0.0, +6.0, 2.0,
	+4.0, +2.0, 2.0
};

static vector3 l_start[11] = {
	+2.0, +8.0, +3.0,
	+2.0, +3.0, +1.0,
	+1.0, +4.0, +3.0,
	+0.0, +4.0, -2.0,
	+4.0, +4.0, +4.0,
	+1.0, +6.0, +4.0,
	-2.0, +4.0, +4.0,
	-2.0, +6.0, -1.0,
	+3.0, +4.0, +3.0,
	+4.01, +4.0, +4.0,
        +2.0, +4.0, 2.0
};

static vector3 l_index[11] = {
	+0.0, -3.0, +0.0,
	+0.0, +0.0, +3.0,
	+0.0, +0.0, +3.0,
	+0.0, +0.0, +3.0,
	-2.0, -2.0, -4.0,
	+0.0, +0.0, -4.0,
	+0.0, +0.0, -3.0,
	+2.0, -1.0, +3.0,
        +0.0, +0.0, -2.0,
	-2.0, -2.0, -4.0,
	+0.0, +0.0, -4.0
};

static boolean lt_res[11] = {
	FALSE, TRUE, FALSE, FALSE, TRUE, FALSE, FALSE, TRUE, FALSE, FALSE, TRUE
};


static vector3 line_start = {-4.0, 4.0, 2.0};
static vector3 line_index = { 1.0, 0.0, 2.0};


int triangle3_test()
{
    int failed = FALSE;
    int errors = 0;
    char index;

    list t_list;
    list ct_list;
    triangle3 t;
    double d;
    int i;

    index = 'a';
    fprintf(stderr, "TRIANGLE3-TEST from %s %s\n"
            "  %c) Clipping test.\n",
	    __DATE__, __TIME__, index++);


    fprintf(stderr, "     generate four test triangles\n");

    t_list = list_new();
    list_add(t_list, (t = triangle3_new(da)));
    list_add(t_list, triangle3_new(db));
    list_add(t_list, triangle3_new(dc));
    list_add(t_list, triangle3_new(dd));

    triangle3_print_list(t_list);

    fprintf(stderr, "  %c) compute factor for line-index to pierce triangle T1(%p).\n"
		    "     with a call of triangle3_distance()\n",
	    index++, t);

    if (triangle3_distance(&d, line_start, line_index, t)) {
	fprintf(stderr, "     ->Result, factor=%f\n (should be -1!)", d);
	if (fabs(d + 1.0)>0.0001) {
	    failed = TRUE;
	    errors ++;
	}
    }
    else {
	failed = TRUE;
	errors ++;
    }

    fprintf(stderr, "    test of beetwen T1 and line "
	  	    "    (%1.0f %1.0f %1.0f) ->+ (%1.0f %1.0f %1.0f)\n"
		    "    test : %s\n",
                    line_start[0], line_start[1], line_start[2],
		    line_index[0], line_index[1], line_index[2],
                    (failed ? "FAILED" : "PASSED"));

    fprintf(stderr, "  %c) clip triangles on y=+/-3\n",
	    index++);

    ct_list = triangle3_clip(t_list, 1, -3.0, +3.0);

    fprintf(stderr, "      The result of the clipping is:\n");

    triangle3_print_list(ct_list);

    fprintf(stderr, "CHECK THE RESULT (this test return no error-nr)!!!!\n"
		    "It should be consist of the triangles:\n"
		    "I.   (-7 1) (-1 1) (-3 3) and (-7 1) (-3 3) (-6 3)\n"
		    "II.  (-2.5 -3) (-3 -1) (-4 -3)\n"
		    "III. (1 -2) (4 -2) (1 1)\n\n");

    triangle3_list_free(t_list);
    triangle3_list_free(ct_list);


    fprintf(stderr, "\n  %c) Creating a triangle for the piercing test\n",
	    index++);

    t = triangle3_new(dl);

    triangle3_print(t);

    for (i=0; i<11; i++) {

        if (lt_res[i] == straight_line_trough_triangle(l_start[i],
            l_index[i], t)) {

            failed = FALSE;
	}
	else {
	    failed = TRUE;
            errors++;
	}

	fprintf(stderr, "    test if %s with line %d. "
			" (%1.0f %1.0f %1.0f) ->+ (%1.0f %1.0f %1.0f)\n"
			"    test : %s\n",
			(lt_res[i] ? "piercing" : "not piercing"),
			i+1,
                        l_start[i][0], l_start[i][1], l_start[i][2],
			l_index[i][0], l_index[i][1], l_index[i][2],
                        (failed ? "FAILED" : "PASSED"));
    }


    fprintf(stderr, "Total amount of errors: %d. Test %s!\n\n",
			errors, errors ? "FAILED" : "PASSED");


    return errors;
}



int main(int argc, char **argv)
{
    return triangle3_test();
}
