/********************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: vector.h
 *  TYPE		: module definition
 *  Time-stamp: <95/02/13 17:03:51 stolz>
 ********************************************************************************
 *  MODULE              : vector
 *
 *  FUNCTION
 *
 *  NOTE
 *
 *  HISTORY
 *  07-Mar-94   Creation date.
 *
 ********************************************************************************/
#ifndef VECTOR_H
#define VECTOR_H

#ifndef BASIC_H
#include "basic.h"
#endif



typedef double vector2[2];

extern vector2  v2_zero;

#define v2_let(a,b)     {(a)[0]=(b)[0];(a)[1]=(b)[1];}
#define v2_set(a,x,y)   {(a)[0]=x;(a)[1]=y;}

#define v2_product(a,b)   ((a)[0]*(b)[0]+(a)[1]*(b)[1])
#define v2_sum(a)         sqrt((a)[0]*(a)[0]+(a)[1]*(a)[1])
#define v2_scalar(a,b,k)  {(a)[0]=(b)[0]*(k);(a)[1]=(b)[1]*(k);}
#define v2_norm(a,b)      v2_skalar(a,b,1.0/v2_betrag(b))
#define v2_add(a,b,c)     {(a)[0]=(b)[0]+(c)[0];(a)[1]=(b)[1]+(c)[1];}
#define v2_sub(a,b,c)     {(a)[0]=(b)[0]-(c)[0];(a)[1]=(b)[1]-(c)[1];}
#define v2_inc(a,b)       {(a)[0]+=(b)[0];(a)[1]+=(b)[1];}
#define v2_dec(a,b)       {(a)[0]-=(b)[0];(a)[1]-=(b)[1];}
#define v2_mul(a,b,c)     {(a)[0]=(b)[1]*(c)[0]-(b)[0]*(c)[1];(a)[1]=(b)[0]*(c)[1]-(b)[1]*(c)[0];}
#define v2_neg(a,b)       {(a)[0]=-(b)[0];(a)[1]=-(b)[1];}
#define v2_addscal(a,b,k,c) {(a)[0]=(b)[0]+(k)*(c)[0];(a)[1]=(b)[1]+(k)*(c)[1];}
#define v2_subscal(a,b,k,c) {(a)[0]=(b)[0]-(k)*(c)[0];(a)[1]=(b)[1]-(k)*(c)[1];}
#define v2_vmul(a,b,c)	  {(a)[0]=(b)[0]*(c)[0];(a)[1]=(b)[1]*(c)[1];}

#define v2_is_in_range(l,x,h)	((l)[0]<=(x)[0]&&(x)[0]<=(h)[0]&&(l)[1]<=(x)[1]&&(x)[1]<=(h)[1])
#define v2_is_positive(x)	((x)[0]>=0.0&&(x)[1]>=0.0)
#define v2_is_negative(x)	((x)[0]<0.0&&(x)[1]<0.0)
#define v2_is_zero(x)		(fasb((x)[0])<ZERO_MAX&&fabs((x)[1])<ZERO_MAX)
#define v2_is_not_zero(x)	(fasb((x)[0])>=ZERO_MAX&&fabs((x)[1])>=ZERO_MAX)

#define v2_out_of_range(l,x,h)  ((l)[0]>(x)[0]||(x)[0]>(h)[0]||(l)[1]>(x)[1]||(x)[1]>(h)[1])

#define v2_elems(x)		(x)[0],(x)[1]
#define v2_elem(x,i)		(x)[i]
#define v2_distance_quadrat(a,b) (((a)[0]-(b)[0])*((a)[0]-(b)[0])+((a)[1]-(b)[1])*((a)[1]-(b)[1]))




typedef double vector3[3];


#define v3_product(a,b)   ((a)[0]*(b)[0]+(a)[1]*(b)[1]+(a)[2]*(b)[2])
#define v3_sum(a)         sqrt((a)[0]*(a)[0]+(a)[1]*(a)[1]+(a)[2]*(a)[2])
#define v3_scalar(a,b,k)  {(a)[0]=(b)[0]*(k);(a)[1]=(b)[1]*(k);(a)[2]=(b)[2]*(k);}
#define v3_norm(a,b)      v3_skalar(a,b,1.0/v3_betrag(b))
#define v3_add(a,b,c)     {(a)[0]=(b)[0]+(c)[0];(a)[1]=(b)[1]+(c)[1];(a)[2]=(b)[2]+(c)[2];}
#define v3_sub(a,b,c)     {(a)[0]=(b)[0]-(c)[0];(a)[1]=(b)[1]-(c)[1];(a)[2]=(b)[2]-(c)[2];}
#define v3_inc(a,b)       {(a)[0]+=(b)[0];(a)[1]+=(b)[1];(a)[2]+=(b)[2];}
#define v3_dec(a,b)       {(a)[0]-=(b)[0];(a)[1]-=(b)[1];(a)[2]-=(b)[2];}
#define v3_mul(a,b,c)     {(a)[0]=(b)[1]*(c)[2]-(b)[2]*(c)[1];(a)[1]=(b)[2]*(c)[0]-(b)[0]*(c)[2];(a)[2]=(b)[0]*(c)[1]-(b)[1]*(c)[0];}
#define v3_let(a,b)       {(a)[0]=(b)[0];(a)[1]=(b)[1];(a)[2]=(b)[2];}
#define v3_neg(a,b)       {(a)[0]=-(b)[0];(a)[1]=-(b)[1];(a)[2]=-(b)[2];}
#define v3_set(a,x,y,z)   {(a)[0]=x;(a)[1]=y;(a)[2]=z;}
#define v3_addscal(a,b,k,c) {(a)[0]=(b)[0]+(k)*(c)[0];(a)[1]=(b)[1]+(k)*(c)[1];(a)[2]=(b)[2]+(k)*(c)[2];}
#define v3_subscal(a,b,k,c) {(a)[0]=(b)[0]-(k)*(c)[0];(a)[1]=(b)[1]-(k)*(c)[1];(a)[2]=(b)[2]-(k)*(c)[2];}
#define v3_vmul(a,b,c)     {(a)[0]=(b)[0]*(c)[0];(a)[1]=(b)[1]*(c)[1];(a)[2]=(b)[2]*(c)[2];}

#define v3_is_in_range(l,x,h) ((l)[0]<=(x)[0]&&(x)[0]<=(h)[0]&&(l)[1]<=(x)[1]&&(x)[1]<=(h)[1]&&(l)[2]<=(x)[2]&&(x)[2]<=(h)[2])
#define v3_is_positive(x)	((x)[0]>=0.0&&(x)[1]>=0.0&&(x)[2]>=0.0)
#define v3_is_negative(x)	((x)[0]<0.0&&(x)[1]<0.0&&(x)[2]<0.0)
#define v3_is_zero(x)	(fasb((x)[0])<ZERO_MAX&&fabs((x)[1])<ZERO_MAX&&fabs((x)[2])<ZERO_MAX)
#define v3_is_not_zero(x)	(fasb((x)[0])>=ZERO_MAX&&fabs((x)[1])>=ZERO_MAX&&fabs((x)[2])>=ZERO_MAX)
#define v3_equal(a,b)		(fabs((a)[0]-(b)[0])<ZERO_MAX&&fabs((a)[1]-(b)[1])<ZERO_MAX&&fabs((a)[2]-(b)[2])<ZERO_MAX)

#define v3_out_of_range(l,x,h)  ((l)[0]>(x)[0]||(x)[0]>(h)[0]||(l)[1]>(x)[1]||(x)[1]>(h)[1]||(l)[2]>(x)[2]||(x)[2]>(h)[2])
#define v3_elems(x)		(x)[0],(x)[1],(x)[2]
#define v3_elem(x,i)		(x)[i]

extern vector3  v3_zero;
extern vector3  v3_one[3];

typedef double	quaternion[4];

#define	q_set(q,q0,q1,q2,q3) (q)[0]=(q0);(q)[1]=(q1);(q)[2]=(q2);(q)[3]=(q3);
#define q_let(a,b)	{(a)[0]=(b)[0];(a)[1]=(b)[1];(a)[2]=(b)[2];(a)[3]=(b)[3];}
#define q_neg(a,b)      {(a)[0]=(b)[0];(a)[1]=-(b)[1];(a)[2]=-(b)[2];(a)[3]=-(b)[3];}
#define q_elems(q)      (q)[0],(q)[1],(q)[2],(q)[3]
#define q_elem(q,i)      (q)[i]

void q_set_av(quaternion *q, double angle, vector3 axis);
void q_get_av(double *angle, vector3 *axis, quaternion q);

void v3_qrot(vector3 *y, vector3 v, quaternion q);

void q_mul(quaternion *q, quaternion a, quaternion b);


extern quaternion q_zero;


typedef double hom_matrix[4][4];

void hm_mul(hom_matrix *x, hom_matrix a, hom_matrix b);
void hm_set_v3q(hom_matrix *x, vector3 translation, quaternion rotation);
void hm_let(hom_matrix *x, hom_matrix a);
void hm_vmul(vector3 *y, hom_matrix t, vector3 x);

extern hom_matrix hm_zero;
#endif


