/**************************************************************************
 *  PROJECT		: DA mobile robot simulation
 *  AUTHOR(S)           : Horst Stolz (HUS)
 *  FILE		: xvmobs_stubs.c
 *  TYPE		: module implementation
 *  Time-stamp: <95/04/07 16:23:51 stolz>
 **************************************************************************
 *  MODULE              : xvmobs_stubs
 *
 *  IMPLEMENTATION
 *	Subs-Implementation of the graphical user interface of MOBS.
 *
 *  HISTORY
 *  13-Jan-95  Remove automatic startup of a ivd-server
 *             Start remote task via pvm, catch output to stdout.
 *             Extract machine and args of the task-string.
 *  20-Feb-95  Add ivd_lock/unlock-support and automatic camera-following
 *             of the current robot.
 *             Autosetting of the remote-task-string with parameter.
 *  01-Mar-95  Remote Callback for the control and config-windows.
 *             Change remote/manual-switch to spawn/kill-buttons
 *             Change remote-task-parameters to -s -r <name>
 *  10-Mar-95  Add sync-Switch 
 *  11-Mar-95  Remove sync-Switch.
 **************************************************************************/

/*
 * xvmobs_stubs.c - Notify and event callback function stubs.
 * This file was generated by `gxv' from `xvmobs.G'.
 */


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include "xvmobs_ui.h"
#include <math.h>
#include <X11/Intrinsic.h>
#include <signal.h>
#include <memory.h>

#include "mob_world.h"
#include "xdraw.h"
#include "list.h"
#include "mwo.h"
#include "rev.h"
#include "pvm_server.h"
#include "ivd.h"
#include "triangle_projection.h"
#include <group.h>

#ifndef NO_PVM
#include "pvm3.h"
#endif

#define MAIN 1

void	insert_door_closed();
void	insert_robot();
void	insert_chair();
void	insert_cuboid();
void	insert_door();
void	insert_human();
void	insert_rectangle();
void		insert_sign();
void	insert_table();
void	insert_triangle();
void	insert_wall();
void	insert_wall_low();
void	insert_floor();
void	insert_desk();
void	insert_ceiling();
void	object_cut();
void	object_group();
void	object_copy();
void	object_paste();
void	object_ungroup();
void	zoom_in();
void	zoom_out();
void	object_deselect_all();
void	quit_mobs();
void	object_edit_open();
void	sim_step();
void	set_display_speed();
void	debug_space_notify();
void	us_results_notify();
void	motion_tracking_notify();
void	object_names_notify();
void	real_us_notify();
void	no_edge_reflection_notify();
void	set_display_scale();
void		Select_2D();
void		Adjust_2D();
void	set_robot_ypos();
void	set_robot_angle();
void	remote_spawn();
void	remote_kill();
void	robot_reset();
void	robot_resume();
void	do_file();
void	new_world();
void	editor_config_apply();
void	sim_config_apply();
void	env_config_apply();
void	object_edit_done();
void	sync_mode_notify();

/*
 * Global object definitions.
 */
xvmobs_window1_objects	*Xvmobs_window1;
xvmobs_robot_control_objects	*Xvmobs_robot_control;
xvmobs_file_error_window_objects	*Xvmobs_file_error_window;
xvmobs_about_window_objects	*Xvmobs_about_window;
xvmobs_file_window_objects	*Xvmobs_file_window;
xvmobs_attention_window_objects	*Xvmobs_attention_window;
xvmobs_config_editor_window_objects	*Xvmobs_config_editor_window;
xvmobs_config_sim_window_objects	*Xvmobs_config_sim_window;
xvmobs_config_env_window_objects	*Xvmobs_config_env_window;
xvmobs_object_window_objects	*Xvmobs_object_window;

#ifdef MAIN

/*
 * Instance XV_KEY_DATA key.  An instance is a set of related
 * user interface objects.  A pointer to an object's instance
 * is stored under this key in every object.  This must be a
 * global variable.
 */
Attr_attribute	INSTANCE;


#define CLIP_FILENAME	".mobs_clipboard.mof"

#define MIN_PIXELS_PER_METER 10.0
#define MAX_PIXELS_PER_METER 1000.0


enum File_action_mode {
    FAM_NONE, FAM_LOAD_WORLD, FAM_INSERT_OBJECTS,
    FAM_SAVE_WORLD, FAM_SAVE_OBJECTS, FAM_SAVE_MOB_PROTOCOL
};

mob_world	world;
static Window mywindow;
static Display *mydisplay = 0;
static GC mygc;
static int mywidth, myheight;
static char timestring[80] = { "Time:" };
static int robot_window_up = 0;
static vector2 window_center = {0.0, 0.0};
static double frames_per_second = 10.0;
static double pixels_per_meter = 100.0;
static double sim_time = 0.0;
static mw_object     act_mob = 0;
static mw_object     act_obj = 0;
static short mouse_x, mouse_y;
static vector3 mouse_real;
static list selected_objs;
static boolean snap_grid = TRUE;
static double  snap_position = 0.01; /* 10mm */
static double  snap_angle = PI/12;  /* 15 Grad*/
static int center_radius = 25; /* 25 pixel */
static int file_action_mode = FAM_NONE;
static int pick_height = 2900;

/* display-modes:
 */
enum display_flags {DF_US_RESULTS=1,
			DF_SPACE=2,
			DF_MOTION=4,
			DF_NAMES=8,
			DF_REAL_US=16,
			DF_NO_EDGE_REFL=32,
		        DF_SYNCHRON=64};

static int display_modes = 0;


static char *host;

char *about_text1 = "Welcome to MOBS,\nthe Mobile rOBot Simulator.\n\n";
char *about_text2 = "\n\nUse ";

char *about_text3 = " as simulator-task-id for\nconnecting robot applications to\nthe simulator. -1 indicates that\nPVM is not started!\n\nFor bug-fixes contact Horst Stolz.\nemail address:\nstolz@hermes.informatik.uni-stuttgart.de\n";
char about_string[300];



/* forward-declarations
 */
void object_window_repaint();
void object_changed();

void terminate()
{
    ivd_free();
    if (world) mob_world_free(world);
    if (selected_objs) list_free(selected_objs);
    world = 0;
    selected_objs = 0;

    printf("Terminate!\n");
    fflush(stdout);
    exit(0);
}



int main(argc, argv)
     int	argc;
     char	**argv;
{
    int ivd_server = -1;                  /* vars for arg-parsing */
    char *renderhost = 0; /* getenv("HOST"); */
    char *renderdisplay = getenv("DISPLAY");
    int i;
    list files = list_new();
    char *ivfile = 0;
    char *fname;


    /* remember host-name 
     */
    host = getenv("HOST");


    /* Argument-parsing
     */
    for (i=1; i<argc; i++) {
	if (strcmp(argv[i], "-ivd") == 0) {

	    /* ivd-tid option
	     */
	    i++;
	    if (i<argc) ivd_server = atoi(argv[i]);
	    else printf("%s: missing pvm-tid-number after option '%s'.\n", 
			argv[0], argv[i-1]);
	}
	else if (strcmp(argv[i], "-renderhost") == 0) {

	    /* host of the render-server (ivd_server)
	     */
	    i++;
	    if (i<argc) renderhost = argv[i];
	    else printf("%s: missing host-name after option '%s'.\n", 
			argv[0], argv[i-1]);
	}
	else if (strcmp(argv[i], "-renderdisplay") == 0) {

	    /* display of the render-server (ivd_server)
	     */
	    i++;
	    if (i<argc) renderdisplay = argv[i];
	    else printf("%s: missing host-name after option '%s'.\n", 
			argv[0], argv[i-1]);
	}
	else if (strcmp(argv[i], "-renderfile") == 0) {

	    /* iv-file to hang at root-node.
	     */
	    i++;
	    if (i<argc) ivfile = argv[i];
	    else printf("%s: missing filename after option '%s'.\n", 
			argv[0], argv[i-1]);
	}
	else if (argv[i][0] == '?' || argv[i][0] == 'h' ||
		 argv[i][0] == 'H' || strncmp(argv[i], "-h", 2)==0 ||
		 strncmp(argv[i], "-H", 2)==0) {

	    /* print Help-text
	     */
	    printf("Usage: %s [-ivd <tid-nr>] [-renderhost <hostname>]\n"
		   "  [-renderdisplay <display>  [-renderfile <filename>]\n"
		   "  <mobs-object-files> ...\n"
		   "Project-Id: %s\n"
		   "Project-Version: %s\n",
		   argv[0], project_id_string, project_version_string);
	    
	    list_free(files);
	    exit(0);
	}
	else {
	    list_add(files, argv[i]);
	}
    }
    

    selected_objs = list_new();
    world = mob_world_new();
    
    if (ivd_server == -1)
	ivd_new(IVD_SERVER, renderhost, renderdisplay);
    else
	ivd_new2(ivd_server, renderdisplay);

    ivd_lock();

    if (ivfile) ivd_insert_file(ivfile);
    else ivd_insert_file("iv/lights.iv");

    mw_object_set_extern(ivd_object_new, 
			 ivd_object_free, ivd_object_change);

    for (fname=list_first(files); fname; fname=list_next(files)) {
	/* insert a MOF-File
	 */
	if (mob_world_insert_objects(world, fname)) {
	    fprintf(stderr, "Error while reading file '%s'.\n%s\n",
		    fname, mob_world_io_error_string);
	}
    }

    ivd_unlock();

#ifndef NO_PVM
    /* redirect output of pvm-tasks
     */
    pvm_catchout(stdout); 
#endif

    xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);

    /*
     * Initialize XView.
     */

    INSTANCE = xv_unique_key();
	
	/*
	 * Initialize user interface components.
	 * Do NOT edit the object initializations by hand.
	 */
	Xvmobs_window1 = xvmobs_window1_objects_initialize(NULL, NULL);
	Xvmobs_robot_control = xvmobs_robot_control_objects_initialize(NULL, Xvmobs_window1->window1);
	Xvmobs_file_error_window = xvmobs_file_error_window_objects_initialize(NULL, Xvmobs_robot_control->robot_control);
	Xvmobs_about_window = xvmobs_about_window_objects_initialize(NULL, Xvmobs_robot_control->robot_control);
	Xvmobs_file_window = xvmobs_file_window_objects_initialize(NULL, Xvmobs_robot_control->robot_control);
	Xvmobs_attention_window = xvmobs_attention_window_objects_initialize(NULL, Xvmobs_robot_control->robot_control);
	Xvmobs_config_editor_window = xvmobs_config_editor_window_objects_initialize(NULL, Xvmobs_window1->window1);
	Xvmobs_config_sim_window = xvmobs_config_sim_window_objects_initialize(NULL, Xvmobs_window1->window1);
	Xvmobs_config_env_window = xvmobs_config_env_window_objects_initialize(NULL, Xvmobs_window1->window1);
	Xvmobs_object_window = xvmobs_object_window_objects_initialize(NULL, Xvmobs_window1->window1);



	textsw_insert(Xvmobs_about_window->about_text, about_text1,
		      strlen(about_text1));
	textsw_insert(Xvmobs_about_window->about_text, project_id_string,
		      strlen(project_id_string));
	textsw_insert(Xvmobs_about_window->about_text, " ", 1);
	textsw_insert(Xvmobs_about_window->about_text, project_version_string,
		      strlen(project_version_string));
	textsw_insert(Xvmobs_about_window->about_text, about_text2,
		      strlen(about_text2));

#ifndef NO_PVM
	sprintf(about_string, "%d", ps_server_tid());
#else
	sprintf(about_string, "no pvm available");
#endif

	textsw_insert(Xvmobs_about_window->about_text, about_string,
		      strlen(about_string));
	textsw_insert(Xvmobs_about_window->about_text, about_text3,
		      strlen(about_text3));



	xv_set(Xvmobs_robot_control->text_io, 
	       TEXTSW_MEMORY_MAXIMUM, 50000000, 
	       TEXTSW_WRAPAROUND_SIZE, 50000000,
	       NULL);

        signal(SIGTERM, terminate);
        signal(SIGKILL, terminate);
        signal(SIGQUIT, terminate);

	/*
	 * Turn control over to XView.
	 */
	xv_main_loop(Xvmobs_window1->window1);

	ivd_free();
	mob_world_free(world);
	list_free(selected_objs);

	exit(0);

	return 0;
}
#endif






void repaint_rob_window()
{
    static mw_object prev_mob = 0;
    static long prev_x, prev_y, prev_angle, prev_len;
    static int prev_type;
    static char prev_name[100];
    static int prev_tid=0;

    long x, y, angle;
    char *name;
    int tid;
    
    mwo_mob_state state;
    char *text;
    int len;
    

    if (mydisplay == 0) return;

    if (act_mob==0) {
	
	if (act_mob != prev_mob)
	    xv_set(Xvmobs_robot_control->robot_name, PANEL_VALUE, 
		   "no robot selected", NULL);

	prev_mob = 0;

	return;
    }
    
    mob_get_state(act_mob, &state);
    mob_get_io(act_mob, &text, &len);
    name = mw_object_get_name(act_mob);
    
    x = dround(state.x*1000.0);
    y = dround(state.y*1000.0);
    angle = dround(state.angle*180.0/PI) % 360;
    tid = mw_object_get_remote_tid(act_mob);

    if (act_mob!=prev_mob || prev_x != x)
	xv_set(Xvmobs_robot_control->xpos, PANEL_VALUE, x, NULL);

    if (act_mob!=prev_mob || prev_y != y)
	xv_set(Xvmobs_robot_control->ypos, PANEL_VALUE, y , NULL);
    
    if (act_mob!=prev_mob || prev_angle != angle)
	xv_set(Xvmobs_robot_control->angle, PANEL_VALUE, angle, NULL);

    if (act_mob!=prev_mob || 
	(name && strcmp(name, prev_name)!=0)) {
	char exe[(name ? strlen(name) : 0)+
		 (host ? strlen(host) : 0)+40];
	
	sprintf(exe, "%s@task -s -r %s", 
		host ? host : "host?", 
		name ? name : "robot?");
	
	xv_set(Xvmobs_robot_control->remote_task, PANEL_VALUE, 
	       exe, NULL);
	xv_set(Xvmobs_robot_control->robot_name, PANEL_VALUE, name, NULL);
    }

    if (act_mob!=prev_mob || prev_type != state.type) {
	switch (state.type) {
	case MOB_STOPPED:
	    xv_set(Xvmobs_robot_control->robot_status, 
		   PANEL_LABEL_STRING, "stopped", NULL);
	    break;

	case MOB_MOVING:
	    xv_set(Xvmobs_robot_control->robot_status, 
		   PANEL_LABEL_STRING, "moving", NULL);
	    break;

	case MOB_HALTED:
	    xv_set(Xvmobs_robot_control->robot_status, 
		   PANEL_LABEL_STRING, "halted", NULL);
	    break;

	default:
	    xv_set(Xvmobs_robot_control->robot_status, 
		   PANEL_LABEL_STRING, "Unknown status?", NULL);
	    break;
	}
    }

    if (act_mob!=prev_mob || prev_len > len) {

	textsw_delete(Xvmobs_robot_control->text_io, 
		      0, TEXTSW_INFINITY);
	
	if (len>0) textsw_insert(Xvmobs_robot_control->text_io, text, len);
    }
    else {
	textsw_insert(Xvmobs_robot_control->text_io, 
		      &text[prev_len], len-prev_len);
    }

    if (tid != prev_tid) {
	if (tid<0) {
		xv_set(Xvmobs_robot_control->robocmd, 
		       PANEL_INACTIVE, FALSE, NULL);
		xv_set(Xvmobs_robot_control->remote_task, 
		       PANEL_INACTIVE, FALSE, NULL);
		xv_set(Xvmobs_robot_control->remote_spawn, 
		       PANEL_INACTIVE, FALSE, NULL);
		xv_set(Xvmobs_robot_control->remote_kill, 
		       PANEL_INACTIVE, TRUE, NULL);
		xv_set(Xvmobs_robot_control->control_mode, 
		       PANEL_VALUE, "manual", NULL);
	}
	else {
		xv_set(Xvmobs_robot_control->robocmd, 
		       PANEL_INACTIVE, TRUE, NULL);
		xv_set(Xvmobs_robot_control->remote_task, 
		       PANEL_INACTIVE, TRUE, NULL);
		xv_set(Xvmobs_robot_control->remote_spawn, 
		       PANEL_INACTIVE, TRUE, NULL);
		xv_set(Xvmobs_robot_control->remote_kill, 
		       PANEL_INACTIVE, FALSE, NULL);
		xv_set(Xvmobs_robot_control->control_mode, 
		       PANEL_VALUE, "remote", NULL);
	}
    }
    
    prev_mob = act_mob;
    prev_x = x;
    prev_y = y;
    prev_angle = angle;
    prev_len = len;
    if (name) strncpy(prev_name, name, 90); else prev_name[0] = 0;
    prev_type = state.type;
    prev_tid = tid;
}


void repaint_2d_display()
{
    double time_round;
    long hours, mins, secs, msecs;


    /* don't draw if this display is not set
     */
    if (mydisplay == 0) return;

    /* convert sim_time into hours,mins,secs and msecs.
     */
    sim_time = mob_world_get_time(world);
    time_round = sim_time + 0.0005;
    secs = time_round;
    msecs = ((long)((time_round - secs) * 1000.0))%1000 ;
    mins = (secs / 60) % 60;
    hours = (secs / 3600) % 60;
    secs = secs % 60;

    if (hours) {
	sprintf(timestring, "Time: %ldh %ldm %lds %ldms",
	       hours, mins, secs, msecs);
    }
    else if (mins) {
	sprintf(timestring, "Time: %ldm %lds %ldms",
	       mins, secs, msecs);
    }
    else {
	sprintf(timestring, "Time: %lds %ldms",
	       secs, msecs);
    }

    xv_set(Xvmobs_window1->time_msg, PANEL_LABEL_STRING, &timestring[0], NULL);

    
    draw2d_set(mydisplay, mywindow, mygc, mywidth, myheight,
               pixels_per_meter, pixels_per_meter, window_center);

    if (display_modes & DF_SPACE)
	draw2d_space(world);
    else 
	draw2d(world);
}


void repaint_display()
{
    static mob_world old_world = 0;
    static int editor = 1;

    if (mydisplay == 0) return;

    if (world != old_world && world) {
	/* update config-windows
	 */
	xv_set(Xvmobs_config_env_window->air_speed, PANEL_VALUE,
	       dround(mob_world_get(world, MWS_AIR_SPEED)), NULL);
	xv_set(Xvmobs_config_env_window->air_damping, PANEL_VALUE,
	       dround(mob_world_get(world, MWS_US_AIR_DAMPING)*1000.0), NULL);
	xv_set(Xvmobs_config_env_window->reflexion_count, PANEL_VALUE,
	       dround(mob_world_get(world, MWS_MAX_REFLEXION_COUNT)), NULL);
	xv_set(Xvmobs_config_sim_window->collision_rate, PANEL_VALUE,
	       dround(mob_world_get(world, MWS_COLLISION_STEP)*1000.0), NULL);

	old_world = world;
    }
    if (editor) {
	xv_set(Xvmobs_config_editor_window->snap_coord, PANEL_VALUE,
	       dround(1000.0*snap_position), NULL);
	xv_set(Xvmobs_config_editor_window->snap_angle, PANEL_VALUE, 
	       dround(180.0/PI*snap_angle), NULL);
	xv_set(Xvmobs_config_editor_window->center_radius, PANEL_VALUE,
	       (long)center_radius, NULL);
	xv_set(Xvmobs_config_editor_window->pick_height, PANEL_VALUE,
	       (long)pick_height, NULL);
	xv_set(Xvmobs_config_editor_window->mouse_snap, PANEL_VALUE,
	       (long)(snap_grid ? 0 : 1), NULL);
	editor = 0;
    }

    repaint_2d_display();

    if (robot_window_up) repaint_rob_window();

    object_window_repaint();
}


void clear_select_status()
{
    act_mob = 0;
    act_obj = 0;
    list_remove_all(selected_objs);
}

void unhighlight_objects()
{
    for (act_obj=list_first(selected_objs); 
	 act_obj; 
	 act_obj=list_next(selected_objs)) {

	(void)mw_object_highlight(act_obj, FALSE);
    }

    list_remove_all(selected_objs);
}


void simulation_step()
{
    sim_time += 1.0/frames_per_second;
    ivd_lock();
    mob_world_simulate_to(world, sim_time);
    if (act_obj && mw_object_function(act_obj) == MWF_MOB) {
	mw_object camera = mw_object_get_sensor(act_obj, CAMERA_OFFSET);
	
	if (camera) ivd_camera_view(camera);
    }
    ivd_unlock();
    repaint_display();

}


Notify_value timer_draw()
{
    simulation_step();
    return NOTIFY_DONE;
}


/*
 * Menu handler for `menu_insert (robot)'.
 */
Menu_item
xvmobs_menu_insert_item0_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_insert_item0_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * User-defined action for `menu_insert'.
 */
void
insert_robot(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*    Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */

    act_mob = mob_create(world);
    mw_object_set_name(act_mob, "Robuter");
    mw_object_set_position(act_mob, mouse_real, q_zero);

    repaint_display();

}

/*
 * Menu handler for `menu_2D (Cut)'.
 */
Menu_item
xvmobs_menu_2D_item1_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_2D_item1_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		object_cut(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * User-defined action for `menu_2D'.
 */
void
object_cut(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*    xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
    
    if (list_is_not_empty(selected_objs)) {

	if (mw_objects_save(selected_objs, CLIP_FILENAME)) {
	    fprintf(stderr, "Error while writing selected objects to the\n"
			    "clipboard file '%s'.\n%s\n",
		    CLIP_FILENAME, mw_object_io_error_string);
	}

	act_obj = list_first(selected_objs);
	while (act_obj) {
	    if (act_obj == act_mob) act_mob = 0;

	    mw_object_free(act_obj);

	    act_obj = list_next(selected_objs);
	}
	list_remove_all(selected_objs);

	repaint_display();
    }
}



/*
 * Notify callback function for `sim_start'.
 */
void
sim_start(item, event)
	Panel_item	item;
	Event		*event;
{
    static struct itimerval timer;
/*    xvmobs_window1_objects *ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: sim_start\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(Xvmobs_window1->file, PANEL_INACTIVE, TRUE, NULL);
	
	xv_set(Xvmobs_window1->sim_start, PANEL_INACTIVE, TRUE, NULL);
	
	xv_set(Xvmobs_window1->sim_stop, PANEL_INACTIVE, FALSE, NULL);
	
	xv_set(Xvmobs_window1->sim_step, PANEL_INACTIVE, TRUE, NULL);
	
	/* gxv_end_connections */


    timer.it_value.tv_usec = 1000000 / (int)(frames_per_second);
/*    timer.it_value.tv_usec = 1000000 / 10; */
    timer.it_interval.tv_usec = timer.it_value.tv_usec;
    
    notify_set_itimer_func(Xvmobs_window1->window1, timer_draw, 
			   ITIMER_REAL, &timer, NULL);
}

/*
 * Notify callback function for `sim_stop'.
 */
void
sim_stop(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_window1_objects *ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: sim_stop\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(Xvmobs_window1->sim_start, PANEL_INACTIVE, FALSE, NULL);
	
	xv_set(Xvmobs_window1->file, PANEL_INACTIVE, FALSE, NULL);
	
	xv_set(Xvmobs_window1->sim_stop, PANEL_INACTIVE, TRUE, NULL);
	
	xv_set(Xvmobs_window1->sim_step, PANEL_INACTIVE, FALSE, NULL);
	
	/* gxv_end_connections */


	notify_set_itimer_func(Xvmobs_window1->window1, NOTIFY_FUNC_NULL, 
			       ITIMER_REAL, NULL, NULL);

}


/*
 * Notify callback function for `sim_step'.
 */
void
xvmobs_window1_sim_step_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_window1_objects *ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_window1_sim_step_notify_callback\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	sim_step(item, event);
	
	/* gxv_end_connections */

}


/*
 * User-defined action for `sim_step'.
 */
void
sim_step(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG
    fputs("xvmobs: sim_step\n", stderr);
#endif
    simulation_step();
}

/*
 * Notify callback function for `sim_reset'.
 */
void
sim_reset(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_window1_objects *ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: sim_reset\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(Xvmobs_window1->sim_start, PANEL_INACTIVE, FALSE, NULL);
	
	xv_set(Xvmobs_window1->sim_stop, PANEL_INACTIVE, TRUE, NULL);
	
	xv_set(Xvmobs_window1->sim_step, PANEL_INACTIVE, FALSE, NULL);
	
	xv_set(Xvmobs_window1->file, PANEL_INACTIVE, FALSE, NULL);
	
	/* gxv_end_connections */

	notify_set_itimer_func(Xvmobs_window1->window1, NOTIFY_FUNC_NULL, 
			       ITIMER_REAL, NULL, NULL);


	mob_world_reset(world);
	
	repaint_display();

}

/*
 * Notify callback function for `display_speed'.
 */
void
xvmobs_window1_display_speed_notify_callback(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
/*	xvmobs_window1_objects *ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: xvmobs_window1_display_speed_notify_callback: value: %d\n", value);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	set_display_speed(item, value, event);
	
	xv_set(Xvmobs_window1->display_2D, XV_SHOW, TRUE, NULL);
	
	/* gxv_end_connections */

}

/*
 * User-defined action for `display_speed'.
 */
void
set_display_speed(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
#ifdef GUI_DEBUG
    fputs("xvmobs: set_display_speed\n", stderr);
#endif

    if (value>0)
	frames_per_second = value;
    else
	frames_per_second = 10.0;
}

/*
 * Notify callback function for `sim_robot'.
 */
void
xvmobs_window1_sim_robot_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_window1_objects *ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_window1_sim_robot_notify_callback\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	robot_window_up = 1;

	repaint_rob_window();
}

/*
 * Notify callback function for `display_scale'.
 */
void
xvmobs_window1_display_scale_notify_callback(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
/*	xvmobs_window1_objects *ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: xvmobs_window1_display_scale_notify_callback: value: %d\n", value);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(Xvmobs_window1->display_2D, XV_SHOW, TRUE, NULL);
	
	set_display_scale(item, value, event);
	
	/* gxv_end_connections */

}

/*
 * User-defined action for `display_scale'.
 */
void
set_display_scale(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: set_display_scale\n", stderr);
#endif

	pixels_per_meter = (double)(value);

	repaint_display();
}

/*
 * Event callback function for `display_2D'.
 */
Notify_value
display_events(win, event, arg, type)
	Xv_window	win;
	Event		*event;
	Notify_arg	arg;
	Notify_event_type type;
{
/*	xvmobs_window1_objects *ip = (xvmobs_window1_objects *) xv_get(xv_get(win, CANVAS_PAINT_CANVAS_WINDOW), XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: display_events: event %d\n", event_id(event));
#endif

	
	if (event_action(event) == ACTION_MENU && event_is_down(event)) {
	    mouse_x = event_x(event);
	    mouse_y = event_y(event);
	    draw2d_get_real_coord(&mouse_real, (int)mouse_x, (int)mouse_y);
	}
	
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	if (event_action(event) == ACTION_MENU && event_is_down(event)) {
		Menu	menu = (Menu) xv_get(win, WIN_MENU);

		if (menu)
			menu_show(menu, win, event, 0);
	}

	if (event_action(event) == ACTION_SELECT)
	{
		Select_2D(win, event, arg, type);
	}
	
	if (event_action(event) == ACTION_ADJUST)
	{
		Adjust_2D(win, event, arg, type);
	}
	
	/* gxv_end_connections */

	return notify_next_event_func(win, (Notify_event) event, arg, type);
}

/*
 * Repaint callback function for `display_2D'.
 */
void
xvmobs_window1_display_2D_repaint_callback(canvas, paint_window, display, xid, rects)
	Canvas		canvas;
	Xv_window	paint_window;
	Display		*display;
	Window		xid;
	Xv_xrectlist	*rects;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_window1_display_2D_repaint_callback\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

    mydisplay = display;
    mywindow = xid;
    mygc = DefaultGC(display, DefaultScreen(display));
    mywidth = (int)xv_get(paint_window, XV_WIDTH);
    myheight = (int)xv_get(paint_window, XV_HEIGHT);

    repaint_display();

}

/*
 * Resize callback function for `display_2D'.
 */
void
display_resize(canvas, width, height)
	Canvas		canvas;
	int		width;
	int		height;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: display_resize\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	/* gxv_end_connections */

	repaint_display();
}


/*
 * User-defined action for `display_2D'.
 */
void
Select_2D(win, event, arg, type)
	Xv_window	win;
	Event		*event;
	Notify_arg	arg;
	Notify_event_type type;
{
/*	xvmobs_window1_objects *ip = (xvmobs_window1_objects *) xv_get(xv_get(win, CANVAS_PAINT_CANVAS_WINDOW), XV_KEY_DATA, INSTANCE); */
    vector3 pick_start, pick_direction;
    static vector3 mouse_start = { 0.0, 0.0, 0.0};
    static vector3 mouse_end = { 0.0, 0.0, 0.0};
    static old_x=0, old_y=0;
	
#ifdef GUI_DEBUG
    fprintf(stderr, "xvmobs: Select_2D: event %d\n", event_id(event));
#endif
    
    
    if (event_is_down(event)) {
	
	/* find object under the mouse-pointer
	 */
	mouse_x = event_x(event);
	mouse_y = event_y(event);
	
#ifdef GUI_DEBUG
	printf("mouse x=%d y=%d, real:(%f %f %f)\n",
	       mouse_x, mouse_y, mouse_real[0], mouse_real[1], mouse_real[2]);
#endif
	
	draw2d_get_real_coord(&mouse_start, (int)mouse_x, (int)mouse_y);
	
	draw2d_get_pick_parameters(&pick_start, &pick_direction,
				   (int)mouse_x, (int)mouse_y);
	pick_start[2] = (double)pick_height/1000.0;
	act_obj = mw_object_pick(world, pick_start, pick_direction);
	if (act_obj) act_obj = mw_object_top_head(act_obj);
#ifdef GUI_DEBUG
	printf("selected object: %p\n", act_obj);
#endif
	old_x = mouse_x; 
	old_y = mouse_y;
	
	if (act_obj) {
	    
	    /* add object to the selected-list
	     */
	    
	    if (mw_object_function(act_obj)==MWF_MOB) {
		act_mob = act_obj;
	    }
	    
	    (void)mw_object_highlight(act_obj, TRUE);
	    
	    list_remove(selected_objs, act_obj);
	    list_add(selected_objs, act_obj);
	    
	    repaint_display();
	}
	else {
	    unhighlight_objects();
	    repaint_display();
	}
    }
    else {
	mouse_x = event_x(event);
	mouse_y = event_y(event);
	if (act_obj && (old_x!=mouse_x || old_y != mouse_y)) {
	    vector3 obj_pos;
	    quaternion obj_orient;
	    vector3 v;
	    
	    draw2d_get_real_coord(&mouse_end, (int)mouse_x, (int)mouse_y);
	    
	    mw_object_position(&obj_pos, &obj_orient, act_obj);
	    
	    if (draw2d_distance(obj_pos, mouse_start) > center_radius) {
		quaternion q;
		double angle;
		
		/* compute angle and rotation-axis
		 */
		v3_dec(mouse_start, obj_pos);
		v3_dec(mouse_end, obj_pos);
		
		draw2d_v3_norm(&mouse_start);
		draw2d_v3_norm(&mouse_end);
		
		angle = v3_product(mouse_end, mouse_start);
		angle /= v3_sum(mouse_start);
		angle /= v3_sum(mouse_end);
		if (angle>1.0) angle = 1.0;
		if (angle<-1.0) angle = -1.0;
		angle = acos(angle);
		
		if (snap_grid) {
		    angle = dcut(angle, snap_angle);
		}
		
		v3_mul(v, mouse_end, mouse_start);
		
		/* avaid negation of the rotation-axis.
		 */
		v3_neg(v, v);
		angle = -angle;
		
		/* rotate object
		 */
		q_set_av(&q, -angle, v);
		
#ifdef GUI_DEBUG
		printf("rotate object angle=%fgrad axe=(%f %f %f)\n",
		       angle*180/PI, v[0], v[1], v[2]);
#endif

		q_mul(&q, obj_orient, q);
		
#ifdef GUI_DEBUG
		printf("obj_orient=%f x (%f %f %f)\n", 
		       obj_orient[0], obj_orient[1], obj_orient[2], obj_orient[3]);
		printf("q=%f x (%f %f %f)\n", q[0], q[1], q[2], q[3]);

		printf("set orientation to angle=%f axe=(%f %f %f)\n",
		       angle, v[0], v[1], v[2]);
#endif
		
		mw_object_set_position(act_obj, obj_pos, q);
	    }
	    else {

		/* move object */
		
		v3_dec(mouse_end, mouse_start);
		
#ifdef GUI_DEBUG
		printf("move object (%f %f %f)\n",
		       mouse_end[0], mouse_end[1], mouse_end[2]);
#endif

		v3_inc(obj_pos, mouse_end);
		
		if (snap_grid) {
		    obj_pos[0] = dcut(obj_pos[0], snap_position);
		    obj_pos[1] = dcut(obj_pos[1], snap_position);
		    obj_pos[2] = dcut(obj_pos[2], snap_position);
		}
		
#ifdef GUI_DEBUG
		printf("set position to (%f %f %f)\n",
		       obj_pos[0], obj_pos[1], mouse_end[2]);
#endif
		
		mw_object_set_position(act_obj, obj_pos, obj_orient);
	    }
	    
	    object_changed();
	    repaint_display();

	}
    }
}


/*
 * User-defined action for `display_2D'.
 */
void
Adjust_2D(win, event, arg, type)
	Xv_window	win;
	Event		*event;
	Notify_arg	arg;
	Notify_event_type type;
{
/*    xvmobs_window1_objects *ip = (xvmobs_window1_objects *) xv_get(xv_get(win, CANVAS_PAINT_CANVAS_WINDOW), XV_KEY_DATA, INSTANCE); */
    static vector3 mouse_end = { 0.0, 0.0, 0.0};
    
#ifdef GUI_DEBUG
    fprintf(stderr, "xvmobs: Adjust_2D: event %d\n", event_id(event));
#endif
    
    if (event_is_down(event)) {
	mouse_x = event_x(event);
	mouse_y = event_y(event);
	draw2d_get_real_coord(&mouse_end, (int)mouse_x, (int)mouse_y);
	v2_let(window_center, mouse_end);
	repaint_display();
    }
}

/*
 * Done callback function for `robot_control'.
 */
void
robot_window_done(frame)
	Frame		frame;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: robot_window_done\n", stderr);
#endif
	xv_set(frame, XV_SHOW, FALSE, NULL);
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	robot_window_up = 0;
}

/*
 * Notify callback function for `xpos'.
 */
Panel_setting
set_robot_xpos(item, event)
	Panel_item	item;
	Event		*event;
{
/*    xvmobs_robot_control_objects *ip = (xvmobs_robot_control_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
    int	value = (int) xv_get(item, PANEL_VALUE);
	
#ifdef GUI_DEBUG
    fprintf(stderr, "xvmobs: set_robot_xpos: value: %d\n", value);
#endif
    
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	/* gxv_end_connections */

    if (act_mob) {
	vector3 position;
	quaternion orientation;
	
	mw_object_position(&position, &orientation, act_mob);
	v3_elem(position, 0) = 1e-3 *(double)value;
	mw_object_set_position(act_mob, position, orientation);

	object_changed();
	repaint_display();
    }
    
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `robot_control_mode'.
 */
void
xvmobs_robot_control_robot_control_mode_notify_callback(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
/*	xvmobs_robot_control_objects *ip = (xvmobs_robot_control_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: xvmobs_robot_control_robot_control_mode_notify_callback: value: %u\n", value);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

}

/*
 * Notify callback function for `ypos'.
 */
Panel_setting
xvmobs_robot_control_ypos_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_robot_control_objects *ip = (xvmobs_robot_control_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	int	value = (int) xv_get(item, PANEL_VALUE);
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: xvmobs_robot_control_ypos_notify_callback: value: %d\n", value);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	set_robot_ypos(item, event);
	
	/* gxv_end_connections */
	
    if (act_mob) {
	vector3 position;
	quaternion orientation;
	
	mw_object_position(&position, &orientation, act_mob);
	v3_elem(position, 1) = 1e-3 *(double)value;
	mw_object_set_position(act_mob, position, orientation);

	object_changed();
	repaint_display();
    }
	return panel_text_notify(item, event);
}

/*
 * User-defined action for `ypos'.
 */
void
set_robot_ypos(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: set_robot_ypos\n", stderr);
#endif
}

/*
 * Notify callback function for `angle'.
 */
Panel_setting
xvmobs_robot_control_angle_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_robot_control_objects *ip = (xvmobs_robot_control_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	int	value = (int) xv_get(item, PANEL_VALUE);
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: xvmobs_robot_control_angle_notify_callback: value: %d\n", value);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	set_robot_angle(item, event);
	
	/* gxv_end_connections */

    if (act_mob) {
	vector3 position;
	quaternion orientation;
	vector3 v;

	mw_object_position(&position, &orientation, act_mob);

#ifdef GUI_DEBUG
	printf("set robot angle to %d\n", value);
#endif
	v3_set(v, 0.0, 0.0, 1.0);
	q_set_av(&orientation, PI/180.0 *(double)value, v);
	mw_object_set_position(act_mob, position, orientation);

	object_changed();
	repaint_display();
    }

	return panel_text_notify(item, event);
}

/*
 * User-defined action for `angle'.
 */
void
set_robot_angle(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: set_robot_angle\n", stderr);
#endif
}

/*
 * Notify callback function for `robocmd'.
 */
Panel_setting
test_getcmd(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_robot_control_objects *ip = (xvmobs_robot_control_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	char *	value = (char *) xv_get(item, PANEL_VALUE);
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: test_getcmd: value: %s\n", value);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	/* gxv_end_connections */

	if (act_mob) {
	    mob_send_command(act_mob, value);
	    repaint_rob_window();
	}

	return panel_text_notify(item, event);
}



/*
 * Menu handler for `menu_insert (rectangle)'.
 */
Menu_item
xvmobs_menu_insert_item1_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_insert_item1_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_insert (triangle)'.
 */
Menu_item
xvmobs_menu_insert_item2_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_insert_item2_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_insert (cuboid)'.
 */
Menu_item
xvmobs_menu_insert_item3_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_insert_item3_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_insert (wall)'.
 */
Menu_item
xvmobs_menu_insert_item4_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_insert_item4_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_insert (chair)'.
 */
Menu_item
xvmobs_menu_insert_item5_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_insert_item5_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_insert (table)'.
 */
Menu_item
xvmobs_menu_insert_item6_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_insert_item6_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_insert (human)'.
 */
Menu_item
xvmobs_menu_insert_item7_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_insert_item7_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_insert (door)'.
 */
Menu_item
xvmobs_menu_insert_item8_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_insert_item8_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_insert (floor)'.
 */
Menu_item
xvmobs_menu_insert_item9_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_insert_item9_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_insert (sign)'.
 */
Menu_item
xvmobs_menu_insert_item10_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_insert_item10_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * User-defined action for `menu_insert'.
 */
void
insert_chair(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
    act_obj = mwo_create(world, MWOT_CHAIR);
    mw_object_move(act_obj, mouse_real, q_zero);
    act_obj = 0;

    repaint_2d_display();
}

/*
 * User-defined action for `menu_insert'.
 */
void
insert_cuboid(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*    Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */

    vector3 v;

    v3_let(v, mouse_real);
    v[2] = 0.5;

    act_obj = mw_object_new(world, MWO_CUBOID, MWF_OBSTACLE);
    mw_object_set_position(act_obj, v, q_zero);
    act_obj = 0;

    repaint_2d_display();
}

/*
 * User-defined action for `menu_insert'.
 */
void
insert_door(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
    act_obj = mwo_create(world, MWOT_DOOR_OPEN);
    mw_object_move(act_obj, mouse_real, q_zero);
    act_obj = 0;

    repaint_2d_display();
}

/*
 * User-defined action for `menu_insert'.
 */
void
insert_floor(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
    act_obj = mwo_create(world, MWOT_FLOOR);
    mw_object_move(act_obj, mouse_real, q_zero);
    act_obj = 0;
    repaint_2d_display();
}

/*
 * User-defined action for `menu_insert'.
 */
void
insert_human(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
    act_obj = mwo_create(world, MWOT_HUMAN);
    mw_object_move(act_obj, mouse_real, q_zero);
    act_obj = 0;
    repaint_2d_display();
}

/*
 * User-defined action for `menu_insert'.
 */
void
insert_rectangle(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*    Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */

    act_obj = mw_object_new(world, MWO_RECTANGLE, MWF_OBSTACLE);
    mw_object_move(act_obj, mouse_real, q_zero);
    act_obj = 0;

    repaint_2d_display();
	
}

/*
 * User-defined action for `menu_insert'.
 */
void
insert_sign(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
}

/*
 * User-defined action for `menu_insert'.
 */
void
insert_table(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */

    act_obj = mwo_create(world, MWOT_TABLE);
    mw_object_move(act_obj, mouse_real, q_zero);
    act_obj = 0; 
    repaint_2d_display();
	
}

/*
 * User-defined action for `menu_insert'.
 */
void
insert_triangle(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*    Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */

    act_obj = mw_object_new(world, MWO_TRIANGLE, MWF_OBSTACLE);
    mw_object_set_position(act_obj, mouse_real, q_zero);
    act_obj = 0;

    repaint_2d_display();
}

/*
 * User-defined action for `menu_insert'.
 */
void
insert_wall(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*    Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */

    act_obj = mwo_create(world, MWOT_WALL);
    mw_object_move(act_obj, mouse_real, q_zero);
    act_obj = 0;
    repaint_2d_display();
    
}

/*
 * Menu handler for `menu_2D (Copy)'.
 */
Menu_item
xvmobs_menu_2D_item2_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_2D_item2_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		object_copy(item, op);
		
		xv_set(Xvmobs_window1->sim_step, PANEL_INACTIVE, TRUE, NULL);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_2D (Paste)'.
 */
Menu_item
xvmobs_menu_2D_item3_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_2D_item3_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		object_paste(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_2D (Group)'.
 */
Menu_item
xvmobs_menu_2D_item4_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_2D_item4_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		object_group(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_2D (Ungroup)'.
 */
Menu_item
xvmobs_menu_2D_item5_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_2D_item5_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		object_ungroup(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_2D (Mark)'.
 */
Menu_item
xvmobs_menu_2D_item6_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_2D_item6_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		object_deselect_all(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_2D (Save)'.
 */
Menu_item
xvmobs_menu_2D_item7_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_2D_item7_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		xv_set(Xvmobs_file_error_window->file_error_window, XV_SHOW, FALSE, NULL);
		
		xv_set(Xvmobs_file_window->file_window, XV_SHOW, TRUE, NULL);
		
		xv_set(Xvmobs_file_window->file_button, PANEL_LABEL_STRING, "Save Objects", NULL);
		
		xv_set(Xvmobs_attention_window->attention_window, XV_SHOW, FALSE, NULL);
		
		/* gxv_end_connections */
		
		file_action_mode = FAM_SAVE_OBJECTS;


		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_2D (Zoom out)'.
 */
Menu_item
xvmobs_menu_2D_item8_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_2D_item8_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		zoom_in(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * User-defined action for `menu_2D'.
 */
void
object_group(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*    xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
    if (list_is_not_empty(selected_objs)) {

	(void)mw_object_group(world, selected_objs, MWF_OBSTACLE);
	
	list_remove_all(selected_objs);

	repaint_2d_display();
    }
}

/*
 * User-defined action for `menu_2D'.
 */
void
object_copy(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */

    if (list_is_not_empty(selected_objs)) {

	if (mw_objects_save(selected_objs, CLIP_FILENAME)) {
	    fprintf(stderr, "Error while writing selected objects to the\n"
			    "clipboard file '%s'.\n%s\n",
		    CLIP_FILENAME, mw_object_io_error_string);
	}
    }
}

/*
 * User-defined action for `menu_2D'.
 */
void
object_mark(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*    xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
    boolean b;
    
    if (list_is_not_empty(selected_objs)) {
	act_obj = list_first(selected_objs);
	while (act_obj) {
	    b = mw_object_mark(act_obj, TRUE);
	    b = mw_object_mark(act_obj, !b);
	    act_obj = list_next(selected_objs);
	}
	repaint_2d_display();
    }
}

/*
 * User-defined action for `menu_2D'.
 */
void
object_paste(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */

	if (mob_world_insert_objects(world, CLIP_FILENAME)) {
	    fprintf(stderr, "Error while reading from clipboard file '%s'.\n"
			    "%s\n", CLIP_FILENAME, mob_world_io_error_string);
	}

	repaint_display();
}

/*
 * user-defined action for `menu_2D'.
 */
void
object_ungroup(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*    xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
    

    if (list_is_not_empty(selected_objs)) {
	act_obj = list_first(selected_objs);
	while (act_obj) {
	    mw_object_ungroup(act_obj);
	    act_obj = list_next(selected_objs);
	}
	list_remove_all(selected_objs);
	
	repaint_2d_display();
    }
}

/*
 * User-defined action for `menu_2D'.
 */
void
zoom_in(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */

	pixels_per_meter *= 2.0;
	if (pixels_per_meter > MAX_PIXELS_PER_METER)
	    pixels_per_meter = MAX_PIXELS_PER_METER;

#ifdef GUI_DEBUG
	printf("mouse x=%d y=%d, real:(%f %f %f)\n",
	       mouse_x, mouse_y, mouse_real[0], mouse_real[1], mouse_real[2]);
#endif

	window_center[0] = mouse_real[0];
	window_center[1] = mouse_real[1];

	xv_set(Xvmobs_window1->display_scale, PANEL_VALUE, (long)pixels_per_meter, NULL);

	repaint_display();
}

/*
 * User-defined action for `menu_2D'.
 */
void
zoom_out(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	pixels_per_meter /= 2.0;
	if (pixels_per_meter < MIN_PIXELS_PER_METER)
	    pixels_per_meter = MIN_PIXELS_PER_METER;
	
	xv_set(Xvmobs_window1->display_scale, PANEL_VALUE, (long)pixels_per_meter, NULL);


	repaint_display();
}


void reset_display_modes()
{
    mob_world_setb(world, MWB_US_SENSOR_PROTOCOL, 
		   (display_modes & DF_US_RESULTS) ? TRUE : FALSE);
    mob_world_setb(world, MWB_MOB_PROTOCOL,
		   (display_modes & DF_MOTION) ? TRUE : FALSE);
    mob_world_setb(world, MWB_REAL_US, 
		   (display_modes & DF_REAL_US) ? TRUE : FALSE);
    draw2d_opt(D2D_DRAW_NAMES,
	       (display_modes & DF_NAMES) ? TRUE : FALSE);
    t3p_with_edge_reflexions = ((display_modes & DF_NO_EDGE_REFL) ? FALSE : TRUE);
    ps_synchron( (display_modes & DF_SYNCHRON) ? TRUE : FALSE);
}

/*
 * Notify callback function for `display_modes'.
 */
void
xvmobs_window1_display_modes_notify_callback(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
/*	xvmobs_window1_objects *ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	short	i;
	int old_display_modes = display_modes;
	
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_window1_display_modes_notify_callback\n", stderr);
#endif

	display_modes = 0;

	for (i = 0; i < 7; i++) {
		if (value & 01)
#ifdef GUI_DEBUG
			fprintf(stderr, "\t%dth item selected\n", i);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		if (i == 4 && (value & 01))
		{
			debug_space_notify(item, value, event);
		}
		
		if (i == 0 && (value & 01))
		{
			us_results_notify(item, value, event);
		}
		
		if (i == 3 && (value & 01))
		{
			motion_tracking_notify(item, value, event);
		}
		
		if (i == 5 && (value & 01))
		{
			object_names_notify(item, value, event);
		}
		
		if (i == 1 && (value & 01))
		{
			real_us_notify(item, value, event);
		}
		
		if (i == 2 && (value & 01))
		{
			no_edge_reflection_notify(item, value, event);
		}
		
		/* gxv_end_connections */

		value >>= 1;
	}

	if ((old_display_modes & DF_US_RESULTS) != 
	    (display_modes & DF_US_RESULTS))
	    mob_world_setb(world, MWB_US_SENSOR_PROTOCOL, 
			   (display_modes & DF_US_RESULTS) ? TRUE : FALSE);

	if ((old_display_modes & DF_MOTION) != 
	    (display_modes & DF_MOTION))
	    mob_world_setb(world, MWB_MOB_PROTOCOL,
			   (display_modes & DF_MOTION) ? TRUE : FALSE);

	if ((old_display_modes & DF_REAL_US) != 
	    (display_modes & DF_REAL_US))
	    mob_world_setb(world, MWB_REAL_US, 
			   (display_modes & DF_REAL_US) ? TRUE : FALSE);
	
	if ((old_display_modes & DF_NAMES) != 
	    (display_modes & DF_NAMES))
	    draw2d_opt(D2D_DRAW_NAMES,
		       (display_modes & DF_NAMES) ? TRUE : FALSE);
	
	t3p_with_edge_reflexions = ((display_modes & DF_NO_EDGE_REFL) ? FALSE : TRUE); 

	ps_synchron( (display_modes & DF_SYNCHRON) ? TRUE : FALSE);


	repaint_display();

}

/*
 * User-defined action for `display_modes'.
 */
void
debug_space_notify(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: debug_space_notify\n", stderr);
#endif

	display_modes |= DF_SPACE;

	repaint_display();
}




/*
 * Notify callback function for `robot_halt_resume'.
 */
void
robot_halt_resume(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_robot_control_objects *ip = (xvmobs_robot_control_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: robot_halt_resume\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	/* gxv_end_connections */

	if (act_mob) mob_emergency_halt(act_mob);

}

/*
 * Menu handler for `menu_file (Load...)'.
 */
Menu_item
xvmobs_menu_file_item1_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_file_item1_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		xv_set(Xvmobs_file_error_window->file_error_window, XV_SHOW, FALSE, NULL);
		
		xv_set(Xvmobs_file_window->file_window, XV_SHOW, TRUE, NULL);
		
		xv_set(Xvmobs_file_window->file_button, PANEL_LABEL_STRING, "Load", NULL);
		
		/* gxv_end_connections */
		
		file_action_mode = FAM_LOAD_WORLD;

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Notify callback function for `robot_resume'.
 */
void
xvmobs_robot_control_robot_resume_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_robot_control_objects *ip = (xvmobs_robot_control_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_robot_control_robot_resume_notify_callback\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	robot_resume(item, event);
	
	/* gxv_end_connections */

}

/*
 * User-defined action for `robot_resume'.
 */
void
robot_resume(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: robot_resume\n", stderr);
#endif

	if (act_mob) mob_emergency_resume(act_mob);
}

/*
 * Notify callback function for `robot_reset'.
 */
void
xvmobs_robot_control_robot_reset_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_robot_control_objects *ip = (xvmobs_robot_control_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_robot_control_robot_reset_notify_callback\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	robot_reset(item, event);
	
	/* gxv_end_connections */

}

/*
 * User-defined action for `robot_reset'.
 */
void
robot_reset(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: robot_reset\n", stderr);
#endif
	
	if (act_mob) mob_soft_reset(act_mob);
}




/*
 * Menu handler for `menu_file (New)'.
 */
Menu_item
xvmobs_menu_file_item0_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_file_item0_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		xv_set(Xvmobs_attention_window->attention_window, XV_SHOW, TRUE, NULL);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_file (Insert...)'.
 */
Menu_item
xvmobs_menu_file_item3_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_file_item3_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		xv_set(Xvmobs_file_error_window->file_error_window, XV_SHOW, FALSE, NULL);
		
		xv_set(Xvmobs_file_window->file_window, XV_SHOW, TRUE, NULL);
		
		xv_set(Xvmobs_file_window->file_button, PANEL_LABEL_STRING, "Insert", NULL);
		
		/* gxv_end_connections */

		file_action_mode = FAM_INSERT_OBJECTS;

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_file (About)'.
 */
Menu_item
xvmobs_menu_file_item4_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_file_item4_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		xv_set(Xvmobs_about_window->about_window, XV_SHOW, TRUE, NULL);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_file (Quit)'.
 */
Menu_item
xvmobs_menu_file_item5_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_file_item5_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		quit_mobs(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}



/*
 * Menu handler for `menu_file (Save)'.
 */
Menu_item
xvmobs_menu_file_item2_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_file_item2_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		xv_set(Xvmobs_file_error_window->file_error_window, XV_SHOW, FALSE, NULL);
		
		xv_set(Xvmobs_file_window->file_window, XV_SHOW, TRUE, NULL);
		
		xv_set(Xvmobs_file_window->file_button, PANEL_LABEL_STRING, "Save", NULL);
		
		/* gxv_end_connections */

		file_action_mode = FAM_SAVE_WORLD;

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}


/*
 * User-defined action for `menu_file'.
 */
void
quit_mobs(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */

	if (xv_destroy_safe(Xvmobs_window1->window1) == XV_OK) {
	    ivd_free();
	    mob_world_free(world);
	    list_free(selected_objs);
	    exit(0);
	}
	else fprintf(stderr, "Can't quit mobs because of xv_destroy_safe()!\n");
}


/*
 * Notify callback function for `close_about'.
 */
void
xvmobs_about_window_close_about_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_about_window_objects *ip = (xvmobs_about_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_about_window_close_about_notify_callback\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(Xvmobs_about_window->about_window, XV_SHOW, FALSE, NULL);
	
	/* gxv_end_connections */

}

/*
 * Notify callback function for `file_button'.
 */
void
xvmobs_file_window_file_button_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_file_window_objects *ip = (xvmobs_file_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_file_window_file_button_notify_callback\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(Xvmobs_file_window->file_window, XV_SHOW, FALSE, NULL);
	
	do_file(item, event);
	
	/* gxv_end_connections */

}

/*
 * User-defined action for `file_button'.
 */
void
do_file(item, event)
	Panel_item	item;
	Event		*event;
{
	mob_world	new_world;

        char *filename = (char *) xv_get(Xvmobs_file_window->file_name,
					 PANEL_VALUE);
#ifdef GUI_DEBUG
	fputs("xvmobs: do_file\n", stderr);
#endif

	ivd_lock();

	switch(file_action_mode) {
	case FAM_LOAD_WORLD:
	    unhighlight_objects();
	    clear_select_status();
	    new_world = mob_world_load(filename);
	    if (new_world) {
		mob_world_free(world);
		world = new_world;
	    }
	    else {
		xv_set(Xvmobs_file_window->file_window,
		       XV_SHOW, FALSE, NULL);
		xv_set(Xvmobs_file_error_window->file_error_text,
			PANEL_VALUE, mob_world_io_error_string, NULL);
		xv_set(Xvmobs_file_error_window->file_error_window,
		       XV_SHOW, TRUE, NULL);
	    }
	    reset_display_modes();
	    break;

	case FAM_SAVE_WORLD:
	    if (mob_world_save(world, filename)) {
		xv_set(Xvmobs_file_window->file_window,
		       XV_SHOW, FALSE, NULL);
		xv_set(Xvmobs_file_error_window->file_error_text,
			PANEL_VALUE, mob_world_io_error_string, NULL);
		xv_set(Xvmobs_file_error_window->file_error_window,
		       XV_SHOW, TRUE, NULL);
	    }
	    break;

	case FAM_INSERT_OBJECTS:
	    if (mob_world_insert_objects(world, filename)) {
		xv_set(Xvmobs_file_window->file_window,
		       XV_SHOW, FALSE, NULL);
		xv_set(Xvmobs_file_error_window->file_error_text,
			PANEL_VALUE, mob_world_io_error_string, NULL);
		xv_set(Xvmobs_file_error_window->file_error_window,
		       XV_SHOW, TRUE, NULL);
	    }
	    break;

	case FAM_SAVE_OBJECTS:
	    if (mw_objects_save(selected_objs, filename)) {
		xv_set(Xvmobs_file_window->file_window,
		       XV_SHOW, FALSE, NULL);
		xv_set(Xvmobs_file_error_window->file_error_text,
			PANEL_VALUE, mw_object_io_error_string, NULL);
		xv_set(Xvmobs_file_error_window->file_error_window,
		       XV_SHOW, TRUE, NULL);
	    }
	    break;
	default:
	    break;		
	}
	file_action_mode = FAM_NONE;
	repaint_display();

	ivd_unlock();
}

/*
 * Notify callback function for `cancel_file'.
 */
void
xvmobs_file_window_cancel_file_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_file_window_objects *ip = (xvmobs_file_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_file_window_cancel_file_notify_callback\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(Xvmobs_file_window->file_window, XV_SHOW, FALSE, NULL);
	
	/* gxv_end_connections */

	file_action_mode = FAM_NONE;
}

/*
 * Notify callback function for `file_error_button'.
 */
void
xvmobs_file_error_window_file_error_button_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_file_error_window_objects *ip = (xvmobs_file_error_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_file_error_window_file_error_button_notify_callback\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(Xvmobs_file_error_window->file_error_window, XV_SHOW, FALSE, NULL);
	
	/* gxv_end_connections */

}

/*
 * Notify callback function for `attention_cancel'.
 */
void
xvmobs_attention_window_attention_cancel_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_attention_window_objects *ip = (xvmobs_attention_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_attention_window_attention_cancel_notify_callback\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

}

/*
 * Notify callback function for `attention_continue'.
 */
void
xvmobs_attention_window_attention_continue_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_attention_window_objects *ip = (xvmobs_attention_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_attention_window_attention_continue_notify_callback\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	new_world(item, event);
	
	/* gxv_end_connections */

}

/*
 * Notify callback function for `robot_name'.
 */
Panel_setting
set_rob_name(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_robot_control_objects *ip = (xvmobs_robot_control_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	char *	value = (char *) xv_get(item, PANEL_VALUE);
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: set_rob_name: value: %s\n", value);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	/* gxv_end_connections */

	if (act_mob) {
	    mw_object_set_name(act_mob, value);
	    repaint_display();
	}

	
	return panel_text_notify(item, event);
}



/*
 * Event callback function for `display_modes'.
 */
void
display_modes_func(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_window1_objects *ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: display_modes_func: event %d\n", event_id(event));
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	/* gxv_end_connections */

	panel_default_handle_event(item, event);
}

/*
 * User-defined action for `display_modes'.
 */
void
us_results_notify(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: us_results_notify\n", stderr);
#endif

	display_modes |= DF_US_RESULTS;
}

/*
 * User-defined action for `display_modes'.
 */
void
motion_tracking_notify(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: motion_tracking_notify\n", stderr);
#endif

	display_modes |= DF_MOTION;
}

/*
 * Menu handler for `menu_2D (Zoom out)'.
 */
Menu_item
xvmobs_menu_2D_item9_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*
	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	*/

	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_2D_item9_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		zoom_out(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * User-defined action for `robot_control_mode'.
 */
void
set_manual_control(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: set_manual_control\n", stderr);
#endif
}

/*
 * User-defined action for `robot_control_mode'.
 */
void
set_remote_control(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: set_remote_control\n", stderr);
#endif
}

/*
 * Notify callback function for `remote_task'.
 */
Panel_setting
xvmobs_robot_control_remote_task_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*
	xvmobs_robot_control_objects *ip = (xvmobs_robot_control_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
*/
	
#ifdef GUI_DEBUG
	char *	value = (char *) xv_get(item, PANEL_VALUE);
	fprintf(stderr, "xvmobs: xvmobs_robot_control_remote_task_notify_callback: value: %s\n", value);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	return panel_text_notify(item, event);
}


/* returns 0 for fail and 1 for succ. spawn
 */
int spawn_remote_task(char *value)
{
    char *exe, nr[20], *machine, *task, *options, *cp;
    int len, i, tid;
    char **argv=0;
    int argmax=0;

#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: spawn remote task '%s'\n", value);
#endif

    len = strlen(value);
    if (len<1) return 0;
    exe = (char *)malloc((len+200)*sizeof(char));
    if (exe==0) return 0;
    
    
#ifdef NO_PVM
    /* old remote-process startup via system()
     */
    if (act_mob) {

	strcpy(exe, value);
	strcat(exe, " -simtid ");
	sprintf(nr, "%d", ps_server_tid());
	strcat(exe, nr);
	strcat(exe, " -rob ");
	strncat(exe, mw_object_get_name(act_mob), 100);
	strcat(exe, " &");

#ifdef GUI_DEBUG
	printf("xvmobs: start remote '%s' via system()\n", exe);
#endif

	system(exe);
	return 1;
    }
#else

    /* task-spawning with PVM
     * Split the exec-string "[host @]task[ options]"
     * into (host)machine, task and options.
     */
	
    strcpy(exe, value);
    
    /* extract host-machine and task-name
     */
    machine = host;
    task = exe;
    for (cp = exe; *cp; cp++) {
	if (*cp=='@') {
	    *cp='\0';
	    if (*exe) machine = exe;
	    task = cp + 1;
	    break;
	    } 
    }
    
    /* split options 
     */
    options = task;
    for (options=task; *options; options++) 
	if (*options==' ') {
	    *options = '\0';
	    options++;
	    break;
	}
    if (*options == 0) options=0;
    else {

	/* separate the args of the option-string an
         * save them to the 0-ended char*-array argv.
	 * use the '*' as escape-char to insert spaces
         * into a argument.
	 */
	argmax = 10; 
	argv = malloc((argmax+1)*sizeof(char *));
	i = 0;

	while(*options) {
	    if (i==argmax) {
		argmax += 10;
		argv = realloc(argv, (argmax+1)*sizeof(char *));
	    }
	    argv[i] = options;
	    i++;
	    while (*options) {
		if (*options == ' ') {
		    *options = '\0';
		    options++;
		    break;
		}
		else if (*options == '*' && *(options+1)) {
		    for (cp=options+1; *cp; cp++)
			*(cp-1) = *cp;
		    *(cp-1) = *cp;
		}
		options++;
	    }
	    
	}
	argv[i] = 0;
    }
    

#ifdef GUI_DEBUG
    printf("xvmobs: start remote task via pvm_spawn()\n");
    printf("machine = %s\n", machine);
    printf("task = %s\n", task);
    if (argv) {
	i = 0;
	while (argv[i]) {
	    printf("argv[%d] = %s\n", i, argv[i]);
	    i++;
	}
    }

#endif
    i = pvm_spawn(task, argv, PvmTaskHost, machine, 1,
		  &tid);
#ifdef GUI_DEBUG
    if (i==1) {
	printf("xvmobs: spawn task '%s' with tid %d\n", exe, tid);
    }
    else printf("xvmobs: can't spawn task '%s'\n", exe);
#endif
#endif
    free(exe);
    if (argv) free(argv);

    return (i==1) ? 1 : 0;
}

/*
 * Menu handler for `menu_insert (floor)'.
 */
Menu_item
xvmobs_menu_insert_item11_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_insert_item11_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_insert (ceiling)'.
 */
Menu_item
xvmobs_menu_insert_item12_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_insert_item12_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_insert (desk)'.
 */
Menu_item
xvmobs_menu_insert_item13_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_insert_item13_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_insert (desk(high))'.
 */
Menu_item
insert_desk_high(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: insert_desk_high: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_insert (sign)'.
 */
Menu_item
xvmobs_menu_insert_item15_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_insert_item15_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * User-defined action for `menu_insert'.
 */
void
insert_ceiling(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */

    act_obj = mwo_create(world, MWOT_CEILING);
    mw_object_move(act_obj, mouse_real, q_zero);
    act_obj = 0;
    repaint_2d_display();
}

/*
 * User-defined action for `menu_insert'.
 */
void
insert_desk(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
    act_obj = mwo_create(world, MWOT_DESK_LOW);
    mw_object_move(act_obj, mouse_real, q_zero);
    act_obj = 0;
    repaint_2d_display();
}

/*
 * User-defined action for `menu_insert'.
 */
void
insert_door_closed(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
    act_obj = mwo_create(world, MWOT_DOOR_CLOSED);
    mw_object_move(act_obj, mouse_real, q_zero);
    act_obj = 0;
    repaint_2d_display();
}

/*
 * User-defined action for `menu_insert'.
 */
void
insert_wall_low(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */

    act_obj = mwo_create(world, MWOT_WALL_LOW);
    mw_object_move(act_obj, mouse_real, q_zero);
    act_obj = 0;
    repaint_2d_display();
	
}

/*
 * User-defined action for `display_modes'.
 */
void
object_names_notify(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: object_names_notify\n", stderr);
#endif

	display_modes |= DF_NAMES;
}

/*
 * User-defined action for `display_modes'.
 */
void
real_us_notify(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: real_us_notify\n", stderr);
#endif
	
	display_modes |= DF_REAL_US;
}


/*
 * Menu handler for `menu_insert (camera)'.
 */
Menu_item
xvmobs_menu_insert_item16_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_insert_item16_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * User-defined action for `menu_insert'.
 */
void
insert_camera(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
}

/*
 * Notify callback function for `edit'.
 */
void
xvmobs_window1_edit_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_window1_objects *ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_window1_edit_notify_callback\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

}

/*
 * Notify callback function for `object_control_button'.
 */
void
xvmobs_window1_object_control_button_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/* 	xvmobs_window1_objects *ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_window1_object_control_button_notify_callback\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

}


/*
 * Notify callback function for `object_x_pos'.
 */
Panel_setting
xvmobs_object_window_object_x_pos_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
        int	value = (int) xv_get(item, PANEL_VALUE);
	fprintf(stderr, "xvmobs: xvmobs_object_window_object_x_pos_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `object_y_pos'.
 */
Panel_setting
xvmobs_object_window_object_y_pos_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	int	value = (int) xv_get(item, PANEL_VALUE);
	fprintf(stderr, "xvmobs: xvmobs_object_window_object_y_pos_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	return panel_text_notify(item, event);
}


/*
 * Notify callback function for `object_z_pos'.
 */
Panel_setting
xvmobs_object_window_object_z_pos_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */

#ifdef GUI_DEBUG
	int	value = (int) xv_get(item, PANEL_VALUE);
	fprintf(stderr, "xvmobs: xvmobs_object_window_object_z_pos_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `object_x_axis'.
 */
Panel_setting
xvmobs_object_window_object_x_axis_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	int	value = (int) xv_get(item, PANEL_VALUE);
	fprintf(stderr, "xvmobs: xvmobs_object_window_object_x_axis_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `object_y_axis'.
 */
Panel_setting
xvmobs_object_window_object_y_axis_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	int	value = (int) xv_get(item, PANEL_VALUE);
	fprintf(stderr, "xvmobs: xvmobs_object_window_object_y_axis_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	return panel_text_notify(item, event);
}


/*
 * Notify callback function for `object_z_axis'.
 */
Panel_setting
xvmobs_object_window_object_z_axis_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	int	value = (int) xv_get(item, PANEL_VALUE);
	fprintf(stderr, "xvmobs: xvmobs_object_window_object_z_axis_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `object_angle'.
 */
Panel_setting
xvmobs_object_window_object_angle_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/* 	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	int	value = (int) xv_get(item, PANEL_VALUE);
	fprintf(stderr, "xvmobs: xvmobs_object_window_object_angle_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	return panel_text_notify(item, event);
}


/*
 * Notify callback function for `object_3d_app'.
 */
Panel_setting
xvmobs_object_window_object_3d_app_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	char *	value = (char *) xv_get(item, PANEL_VALUE);
	fprintf(stderr, "xvmobs: xvmobs_object_window_object_3d_app_notify_callback: value: %s\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `color_red'.
 */
void
xvmobs_object_window_color_red_notify_callback(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
/*	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: xvmobs_object_window_color_red_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

}


/*
 * Notify callback function for `color_green'.
 */
void
xvmobs_object_window_color_green_notify_callback(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
/*	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: xvmobs_object_window_color_green_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

}


/*
 * Notify callback function for `color_blue'.
 */
void
xvmobs_object_window_color_blue_notify_callback(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
/*	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: xvmobs_object_window_color_blue_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

}


/*
 * Notify callback function for `object_dim1'.
 */
Panel_setting
xvmobs_object_window_object_dim1_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	int	value = (int) xv_get(item, PANEL_VALUE);
	fprintf(stderr, "xvmobs: xvmobs_object_window_object_dim1_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `object_dim2'.
 */
Panel_setting
xvmobs_object_window_object_dim2_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	int	value = (int) xv_get(item, PANEL_VALUE);
	fprintf(stderr, "xvmobs: xvmobs_object_window_object_dim2_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	return panel_text_notify(item, event);
}


/*
 * Notify callback function for `object_dim3'.
 */
Panel_setting
xvmobs_object_window_object_dim3_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	int	value = (int) xv_get(item, PANEL_VALUE);
	fprintf(stderr, "xvmobs: xvmobs_object_window_object_dim3_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	return panel_text_notify(item, event);
}


/*
 * Notify callback function for `object_dim4'.
 */
Panel_setting
xvmobs_object_window_object_dim4_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	int	value = (int) xv_get(item, PANEL_VALUE);
	fprintf(stderr, "xvmobs: xvmobs_object_window_object_dim4_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `object_dim5'.
 */
Panel_setting
xvmobs_object_window_object_dim5_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	int	value = (int) xv_get(item, PANEL_VALUE);
	fprintf(stderr, "xvmobs: xvmobs_object_window_object_dim5_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	return panel_text_notify(item, event);
}


/*
 * Menu handler for `menu_edit (robot)'.
 */
Menu_item
xvmobs_menu_edit_item0_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_item0_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		insert_robot(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_edit (rectangle)'.
 */
Menu_item
xvmobs_menu_edit_item1_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_item1_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		insert_rectangle(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_edit (triangle)'.
 */
Menu_item
xvmobs_menu_edit_item2_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_item2_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		insert_triangle(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_edit (cuboid)'.
 */
Menu_item
xvmobs_menu_edit_item3_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_item3_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		insert_cuboid(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_edit (wall)'.
 */
Menu_item
xvmobs_menu_edit_item4_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_item4_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		insert_wall(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_edit (wall(low))'.
 */
Menu_item
xvmobs_menu_edit_item5_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_item5_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		insert_wall_low(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_edit (chair)'.
 */
Menu_item
xvmobs_menu_edit_item6_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_item6_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		insert_chair(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_edit (table)'.
 */
Menu_item
xvmobs_menu_edit_item7_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_item7_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		insert_table(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_edit (human)'.
 */
Menu_item
xvmobs_menu_edit_item8_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_item8_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		insert_human(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_edit (door)'.
 */
Menu_item
xvmobs_menu_edit_item9_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_item9_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		insert_door(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_edit (door(closed))'.
 */
Menu_item
xvmobs_menu_edit_item10_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_item10_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		insert_door_closed(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_edit (floor)'.
 */
Menu_item
xvmobs_menu_edit_item11_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/* 	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_item11_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		insert_floor(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_edit (ceiling)'.
 */
Menu_item
xvmobs_menu_edit_item12_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_item12_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		insert_ceiling(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_edit (desk)'.
 */
Menu_item
xvmobs_menu_edit_item13_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_item13_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		insert_desk(item, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_edit (sign)'.
 */
Menu_item
xvmobs_menu_edit_item15_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_item15_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_edit (camera)'.
 */
Menu_item
xvmobs_menu_edit_item16_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_item16_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_2D (Insert)'.
 */
Menu_item
xvmobs_menu_2D_item0_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_2D_item0_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_control (Robot ...)'.
 */
Menu_item
xvmobs_menu_control_item0_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_control_item0_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		if (xv_get(Xvmobs_robot_control->robot_control, FRAME_CLOSED))
			xv_set(Xvmobs_robot_control->robot_control, FRAME_CLOSED, FALSE, NULL);

		xv_set(Xvmobs_robot_control->robot_control, XV_SHOW, TRUE, NULL);
		
		/* gxv_end_connections */

		robot_window_up = 1;
		repaint_rob_window();
		
		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_control (Object ...)'.
 */
Menu_item
xvmobs_menu_control_item1_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_control_item1_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		object_edit_open(item, op);
		
		if (xv_get(Xvmobs_object_window->object_window, FRAME_CLOSED))
			xv_set(Xvmobs_object_window->object_window, FRAME_CLOSED, FALSE, NULL);

		xv_set(Xvmobs_object_window->object_window, XV_SHOW, TRUE, NULL);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_config (Editor ...)'.
 */
Menu_item
xvmobs_menu_config_item0_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_config_item0_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		xv_set(Xvmobs_config_editor_window->config_editor_window, XV_SHOW, TRUE, NULL);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_config (Simulation ...)'.
 */
Menu_item
xvmobs_menu_config_item1_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_config_item1_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		xv_set(Xvmobs_config_sim_window->config_sim_window, XV_SHOW, TRUE, NULL);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `menu_config (Enviroment ...)'.
 */
Menu_item
xvmobs_menu_config_item2_callback(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_config_item2_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		xv_set(Xvmobs_config_env_window->config_env_window, XV_SHOW, TRUE, NULL);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Notify callback function for `mouse_snap'.
 */
void
xvmobs_config_editor_window_mouse_snap_notify_callback(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
/*	xvmobs_config_editor_window_objects *ip = (xvmobs_config_editor_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: xvmobs_config_editor_window_mouse_snap_notify_callback: value: %u\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	if (value == 1)
	{
		xv_set(Xvmobs_config_editor_window->snap_coord, PANEL_INACTIVE, TRUE, NULL);
	}
	
	if (value == 0)
	{
		xv_set(Xvmobs_config_editor_window->snap_coord, PANEL_INACTIVE, FALSE, NULL);
	}
	
	if (value == 1)
	{
		xv_set(Xvmobs_config_editor_window->snap_angle, PANEL_INACTIVE, TRUE, NULL);
	}
	
	if (value == 0)
	{
		xv_set(Xvmobs_config_editor_window->snap_angle, PANEL_INACTIVE, FALSE, NULL);
	}
	
	/* gxv_end_connections */

}

/*
 * User-defined action for `mouse_snap'.
 */
void
mouse_snap_on(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: mouse_snap_on\n", stderr);
#endif

	snap_grid = TRUE;
}

/*
 * User-defined action for `mouse_snap'.
 */
void
mouse_snap_off(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: mouse_snap_off\n", stderr);
#endif

	snap_grid = FALSE;
}

/*
 * Notify callback function for `snap_coord'.
 */
Panel_setting
set_snap_coord(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_config_editor_window_objects *ip = (xvmobs_config_editor_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	int	value = (int) xv_get(item, PANEL_VALUE);
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: set_snap_coord: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	snap_position = ((double)value) / 1000.0;

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `snap_angle'.
 */
Panel_setting
xvmobs_config_editor_window_snap_angle_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_config_editor_window_objects *ip = (xvmobs_config_editor_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	int	value = (int) xv_get(item, PANEL_VALUE);
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: xvmobs_config_editor_window_snap_angle_notify_callback: value: %d\n", value);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	snap_angle = ((double)value) * PI / 180.0;

	return panel_text_notify(item, event);
}

/*
 * User-defined action for `snap_angle'.
 */
void
set_snap_angle(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: set_snap_angle\n", stderr);
#endif
}

/*
 * Notify callback function for `pick_height'.
 */
Panel_setting
xvmobs_config_editor_window_pick_height_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_config_editor_window_objects *ip = (xvmobs_config_editor_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	int	value = (int) xv_get(item, PANEL_VALUE);
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: xvmobs_config_editor_window_pick_height_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	pick_height = value;

	return panel_text_notify(item, event);
}

/*
 * User-defined action for `pick_height'.
 */
void
set_pick_height(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: set_pick_height\n", stderr);
#endif
}

/*
 * Notify callback function for `air_speed'.
 */
Panel_setting
xvmobs_config_env_window_air_speed_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_config_env_window_objects *ip = (xvmobs_config_env_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	int	value = (int) xv_get(item, PANEL_VALUE);
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: xvmobs_config_env_window_air_speed_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	mob_world_set(world, MWS_AIR_SPEED, (double)value);

	return panel_text_notify(item, event);
}

/*
 * User-defined action for `air_speed'.
 */
void
set_air_speed(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: set_air_speed\n", stderr);
#endif
}

/*
 * Notify callback function for `air_damping'.
 */
Panel_setting
xvmobs_config_env_window_air_damping_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_config_env_window_objects *ip = (xvmobs_config_env_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	int	value = (int) xv_get(item, PANEL_VALUE);
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: xvmobs_config_env_window_air_damping_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	if (value>0)
	    mob_world_set(world, MWS_US_AIR_DAMPING, ((double)value)/1000.0);
	else 
	    mob_world_set(world, MWS_US_AIR_DAMPING, 0.0);

	return panel_text_notify(item, event);
}

/*
 * User-defined action for `air_damping'.
 */
void
set_air_damping(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: set_air_damping\n", stderr);
#endif
}

/*
 * Notify callback function for `reflexion_count'.
 */
Panel_setting
xvmobs_config_env_window_reflexion_count_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_config_env_window_objects *ip = (xvmobs_config_env_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	int	value = (int) xv_get(item, PANEL_VALUE);
	
#ifdef GUI_DEBUG
	fprintf(stderr, "xvmobs: xvmobs_config_env_window_reflexion_count_notify_callback: value: %d\n", value);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	mob_world_set(world, MWS_MAX_REFLEXION_COUNT, (double)value);

	return panel_text_notify(item, event);
}

/*
 * User-defined action for `reflexion_count'.
 */
void
set_reflexion_count(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: set_reflexion_count\n", stderr);
#endif
}




/****************************************************************************
 * OBJECT EDIT WINDOW
 */


enum o_changes {
    CO_POSITION = 1,
    CO_COLOR = 2,
    CO_NAME = 4,
    CO_VISUAL = 8,
    CO_SIZE = 16,
    CO_PAR = 32
};



typedef struct {
    long x, y, z;
} xyz;

typedef struct {
    long red, green, blue;
} rgb;



typedef struct {
    int 	type;
    int		function;
    char	*name;

    xyz         pos;
    xyz         ax;
    long        ag;
    xyz         sz;
    rgb         color;
    long        damp;

    vector3	position;
    quaternion	orientation;
    vector3	axis;
    double	angle;

    vector3	size;

    char	*visual_file;
    
    float	red;
    float	green;
    float	blue;

    double	us_reflexion_damping;
} edited_obj;

static edited_obj o;
static long o_changed = 0;
static int object_window_displayed = 0;


void object_changed()
{
    o_changed = 1;
}

void object_window_repaint()
{
    static mw_object old_obj = 0;
    double d;

    if (object_window_displayed==0 || (o_changed==0 && act_obj == old_obj))
	return;
    
    if (act_obj==0) {
	xv_set(Xvmobs_object_window->object_type, 
	       PANEL_VALUE, "no object selected", NULL);


	if (old_obj==0) return;

	xv_set(Xvmobs_object_window->object_function, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_name, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_3d_app, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_x_pos, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_y_pos, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_z_pos, PANEL_INACTIVE, TRUE, NULL);    
	xv_set(Xvmobs_object_window->object_x_axis, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_y_axis, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_z_axis, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_angle, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->color_red, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->color_green, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->color_blue, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_dim1, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_dim2, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_dim3, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_dim4, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_dim5, PANEL_INACTIVE, TRUE, NULL);

	old_obj = act_obj;
	return;
    }
    
    old_obj = act_obj;
	
    o_changed = 0;

    o.type = mw_object_type(act_obj);
    o.function = mw_object_function(act_obj);
    o.name = mw_object_get_name(act_obj);
    mw_object_position(&o.position, &o.orientation, act_obj);
    q_get_av(&o.angle, &o.axis, o.orientation);
    d = v3_sum(o.axis);
    if (!is_zero(d)) {
	d = 1 / d;
	v3_scalar(o.axis, o.axis, d);
    }
    o.visual_file = mw_object_get_3d(act_obj);
    
    xv_set(Xvmobs_object_window->object_function,  PANEL_INACTIVE, FALSE, NULL);

    switch(o.type) {
    case MWO_TRIANGLE:
	xv_set(Xvmobs_object_window->object_type, 
	       PANEL_VALUE, "triangle", NULL);
	break;
    case MWO_RECTANGLE:
	xv_set(Xvmobs_object_window->object_type, 
	       PANEL_VALUE, "rectangle", NULL);
	break;
    case MWO_CUBOID:
	xv_set(Xvmobs_object_window->object_type, 
	       PANEL_VALUE, "cuboid", NULL);
	break;
    case MWO_GROUP:
	xv_set(Xvmobs_object_window->object_type, 
	       PANEL_VALUE, "group", NULL);
	break;
    default:
	xv_set(Xvmobs_object_window->object_type, 
	       PANEL_VALUE, "unknown object type!", NULL);
	    break;
    }
    

    switch(o.function) {
    case MWF_MOB:
	xv_set(Xvmobs_object_window->object_function, 
	       PANEL_VALUE, "robot controller", NULL);
	break;
    case MWF_SENSOR:
	    xv_set(Xvmobs_object_window->object_function, 
		   PANEL_VALUE, "us sensor", NULL);
	break;
    case MWF_CAMERA:
	xv_set(Xvmobs_object_window->object_function, 
	       PANEL_VALUE, "camera", NULL);
	break;
    case MWF_OBSTACLE:
	xv_set(Xvmobs_object_window->object_function, 
	       PANEL_VALUE, "obstacle", NULL);
	break;
    default:
	    xv_set(Xvmobs_object_window->object_function, 
		   PANEL_VALUE, "unkown object function!", NULL);
	break;
    }

    if (o.name==0) o.name = "";
    xv_set(Xvmobs_object_window->object_name, PANEL_VALUE, o.name,
	   PANEL_INACTIVE, FALSE, NULL);
    
    
    xv_set(Xvmobs_object_window->object_3d_app, PANEL_VALUE, 
	   o.visual_file ? o.visual_file : "", PANEL_INACTIVE, FALSE, NULL);

    xv_set(Xvmobs_object_window->object_x_pos, PANEL_VALUE, 
	   (long)dround(o.position[0]*1000.0), PANEL_INACTIVE, FALSE, NULL);
    xv_set(Xvmobs_object_window->object_y_pos, PANEL_VALUE, 
	   (long)dround(o.position[1]*1000.0), PANEL_INACTIVE, FALSE, NULL);
    xv_set(Xvmobs_object_window->object_z_pos, PANEL_VALUE, 
	   (long)dround(o.position[2]*1000.0), PANEL_INACTIVE, FALSE, NULL);
    
    xv_set(Xvmobs_object_window->object_x_axis, PANEL_VALUE, 
	       (long)dround(o.axis[0]*1000), PANEL_INACTIVE, FALSE, NULL);
    xv_set(Xvmobs_object_window->object_y_axis, PANEL_VALUE, 
	   (long)dround(o.axis[1]*1000), PANEL_INACTIVE, FALSE, NULL);
    xv_set(Xvmobs_object_window->object_z_axis, PANEL_VALUE, 
	   (long)dround(o.axis[2]*1000), PANEL_INACTIVE, FALSE, NULL);
    xv_set(Xvmobs_object_window->object_angle, PANEL_VALUE, 
	   (long)dround(o.angle * 180.0 / PI), PANEL_INACTIVE, FALSE, NULL);
    
    
    if (o.visual_file || o.type == MWO_GROUP) {
	xv_set(Xvmobs_object_window->color_red, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->color_green, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->color_blue, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_dim1, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_dim2, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_dim3, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_dim4, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_object_window->object_dim5, PANEL_INACTIVE, TRUE, NULL);
    }
    else {
	mw_object_get_scale(&o.size, act_obj);
	mw_object_get_color(&o.red, &o.green, &o.blue, act_obj);
	o.us_reflexion_damping = mw_object_get_us_reflexion_damping(act_obj);
	if (o.us_reflexion_damping <= 0.0) o.us_reflexion_damping = 1.0;

	xv_set(Xvmobs_object_window->color_red, PANEL_VALUE, 
	       o.color.red = dround(o.red * 255.0), 
	       PANEL_INACTIVE, FALSE, NULL);
	xv_set(Xvmobs_object_window->color_green, PANEL_VALUE, 
	       o.color.green = dround(o.green * 255.0),
	       PANEL_INACTIVE, FALSE, NULL);
	xv_set(Xvmobs_object_window->color_blue, PANEL_VALUE, 
	       o.color.blue = dround(o.blue * 255.0),
	       PANEL_INACTIVE, FALSE, NULL);
	
	xv_set(Xvmobs_object_window->object_dim1, PANEL_VALUE, 
	       o.sz.x = dround(o.size[0]*1000.0),
	       PANEL_INACTIVE, FALSE, NULL);
	xv_set(Xvmobs_object_window->object_dim2, PANEL_VALUE, 
	       o.sz.y = dround(o.size[1]*1000.0),
	       PANEL_INACTIVE, FALSE, NULL);
	switch(o.type) {
	case MWO_TRIANGLE:
	    xv_set(Xvmobs_object_window->object_dim3, PANEL_VALUE, 
		   o.sz.z = dround(o.size[2]*1000.0),
		   PANEL_LABEL_STRING, "Angle (mdeg)", 
		   PANEL_INACTIVE, FALSE, NULL);
	    break;
	default:
	    xv_set(Xvmobs_object_window->object_dim3, PANEL_VALUE, 
		   o.sz.z = dround(o.size[2]*1000.0),
		   PANEL_LABEL_STRING, "Size Z (mm)", 
		   PANEL_INACTIVE, o.type==MWO_RECTANGLE ? TRUE : FALSE, NULL);
	    break;
	}
	
	xv_set(Xvmobs_object_window->object_dim5, PANEL_VALUE, 
	       o.damp = dround(log10(o.us_reflexion_damping)),
	       PANEL_INACTIVE, FALSE, NULL);

	xv_set(Xvmobs_object_window->object_dim4, PANEL_INACTIVE, FALSE, NULL);
    }
}


/*
 * User-defined action for `menu_control'.
 */
void
object_edit_open(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	

	object_window_displayed = 1;
	object_window_repaint();
}

/*
 * Done callback function for `object_window'.
 */
void
object_edit_done(frame)
	Frame		frame;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: object_edit_done\n", stderr);
#endif
	xv_set(frame, XV_SHOW, FALSE, NULL);
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */
	/* gxv_end_connections */

	object_window_displayed = 0;
}


/*
 * Notify callback function for `object_apply'.
 */
void
object_apply(item, event)
	Panel_item	item;
	Event		*event;
{
    /* xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
    
#ifdef GUI_DEBUG
    fputs("xvmobs: object_apply\n", stderr);
#endif    
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	/* gxv_end_connections */
    
    if (act_obj) {
	edited_obj old;

	old = o;
	
	/* get new object-values
	 */
	
	o.name = (char *)xv_get(Xvmobs_object_window->object_name, PANEL_VALUE);
	o.visual_file = (char *)xv_get(Xvmobs_object_window->object_3d_app, PANEL_VALUE);

	o.pos.x = xv_get(Xvmobs_object_window->object_x_pos, PANEL_VALUE);
	o.pos.y = xv_get(Xvmobs_object_window->object_y_pos, PANEL_VALUE);
	o.pos.z = xv_get(Xvmobs_object_window->object_z_pos, PANEL_VALUE);

	o.ax.x = xv_get(Xvmobs_object_window->object_x_axis, PANEL_VALUE);
	o.ax.y = xv_get(Xvmobs_object_window->object_y_axis, PANEL_VALUE);
	o.ax.z = xv_get(Xvmobs_object_window->object_z_axis, PANEL_VALUE);
	o.ag = xv_get(Xvmobs_object_window->object_angle, PANEL_VALUE);

	o.sz.x = xv_get(Xvmobs_object_window->object_dim1, PANEL_VALUE);
	o.sz.y = xv_get(Xvmobs_object_window->object_dim2, PANEL_VALUE);
	o.sz.z = xv_get(Xvmobs_object_window->object_dim3, PANEL_VALUE);
	o.damp = xv_get(Xvmobs_object_window->object_dim5, PANEL_VALUE);
    

	if (o.type != MWO_GROUP) {
	    o.color.red = xv_get(Xvmobs_object_window->color_red, PANEL_VALUE);
	    o.color.green = xv_get(Xvmobs_object_window->color_green, PANEL_VALUE);
	    o.color.blue = xv_get(Xvmobs_object_window->color_blue, PANEL_VALUE);
	}


	/* update changed values
	 */


	if (memcmp(&o.pos, &old.pos, sizeof(xyz))!=0 ||
	    memcmp(&o.ax, &old.ax, sizeof(xyz))!=0 ||
	    o.ag != old.ag) {
	    o.angle = o.ag * PI / 180.0;
	    o.axis[0] = o.ax.x / 1000.0;
	    o.axis[1] = o.ax.y / 1000.0;
	    o.axis[2] = o.ax.z / 1000.0;
	    o.position[0] =  1e-3 * o.pos.x;
	    o.position[1] =  1e-3 * o.pos.y;
	    o.position[2] =  1e-3 * o.pos.z;
	    q_set_av(&o.orientation, o.angle, o.axis);
	    mw_object_set_position(act_obj, o.position, o.orientation);
	}

	if (o.type != MWO_GROUP) {
	    if (memcmp(&o.color, &old.color, sizeof(rgb))!=0) {
		o.red = o.color.red / 255.0;
		o.green = o.color.green / 255.0;       
		o.blue = o.color.blue / 255.0;
		mw_object_set_color(act_obj, o.red, o.green, o.blue);
	    }

	    if (memcmp(&o.sz, &old.sz, sizeof(xyz))!=0) {
		o.size[0] = 1e-3 * o.sz.x;
		o.size[1] = 1e-3 * o.sz.y;
		o.size[2] = 1e-3 * o.sz.z;
		mw_object_scale(act_obj, o.size);
	    }
	    if (o.damp != old.damp) {
		o.us_reflexion_damping = pow(10.0, min(0, o.damp));
		mw_object_set_us_reflexion_damping(act_obj, o.us_reflexion_damping);
	    }
	}

	if ((o.name==0 && old.name) || (o.name && old.name==0) ||
	    strcmp(o.name, old.name)!=0) {
	    mw_object_set_name(act_obj, o.name);
	}

	if ((o.visual_file==0 && old.visual_file) ||
	    (o.visual_file && old.visual_file==0) ||
	    strcmp(o.visual_file, old.visual_file)!=0) {
	    mw_object_set_3d(act_obj, o.visual_file);
	}

	repaint_display();
    }
}

/*
 * Notify callback function for `object_edit_done'.
 */
void
xvmobs_object_window_object_edit_done_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*	xvmobs_object_window_objects *ip = (xvmobs_object_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */
	
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_object_window_object_edit_done_notify_callback\n", stderr);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(Xvmobs_object_window->object_window, XV_SHOW, FALSE, NULL);
	
	object_edit_done(item, event);
	
	/* gxv_end_connections */

}



/*
 * User-defined action for `display_modes'.
 */
void
no_edge_reflection_notify(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: no_edge_reflection_notify\n", stderr);
#endif

	display_modes |= DF_NO_EDGE_REFL;
}

/*
 * User-defined action for `menu_2D'.
 */
void
object_deselect_all(item, op)
	Menu_item	item;
	Menu_generate	op;
{
/*	xvmobs_window1_objects * ip = (xvmobs_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE); */


    unhighlight_objects();
    repaint_display();
}



/*
 * Repaint callback function for `display_2D'.
 */
void
display_repaint(canvas, paint_window, display, xid, rects)
	Canvas		canvas;
	Xv_window	paint_window;
	Display		*display;
	Window		xid;
	Xv_xrectlist	*rects;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: display_repaint\n", stderr);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(Xvmobs_window1->display_2D, XV_SHOW, TRUE, NULL);
	
	/* gxv_end_connections */

	mydisplay = display;
	mywindow = xid;
	mygc = DefaultGC(display, DefaultScreen(display));
	mywidth = (int)xv_get(paint_window, XV_WIDTH);
	myheight = (int)xv_get(paint_window, XV_HEIGHT);
	
	repaint_display();
}


/*
 * Menu handler for `menu_edit'.
 */
Menu
xvmobs_menu_edit_menu_callback(menu, op)
	Menu		menu;
	Menu_generate	op;
{
/*	Xv_opaque ip = (Xv_opaque) xv_get(menu, XV_KEY_DATA, INSTANCE); */
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
#ifdef GUI_DEBUG
		fputs("xvmobs: xvmobs_menu_edit_menu_callback: MENU_NOTIFY\n", stderr);
#endif
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		insert_sign(menu, op);
		
		/* gxv_end_connections */

		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return menu;
}


/*
 * Notify callback function for `remote_spawn'.
 */
void
xvmobs_robot_control_remote_spawn_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*
	xvmobs_robot_control_objects *ip = (xvmobs_robot_control_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
*/
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_robot_control_remote_spawn_notify_callback\n", stderr);
#endif	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	remote_spawn(item, event);
	
	/* gxv_end_connections */

}

/*
 * User-defined action for `remote_spawn'.
 */
void
remote_spawn(item, event)
	Panel_item	item;
	Event		*event;
{
    char *task = (char *)xv_get(Xvmobs_robot_control->remote_task, PANEL_VALUE);

#ifdef GUI_DEBUG
    fputs("xvmobs: remote_spawn\n", stderr);
#endif

    if (act_mob && mw_object_get_remote_tid(act_mob) > 0) {
	xv_set(Xvmobs_robot_control->robocmd, 
	       PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_robot_control->remote_task, 
	       PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_robot_control->remote_spawn, 
	       PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_robot_control->remote_kill, 
	       PANEL_INACTIVE, FALSE, NULL);
	xv_set(Xvmobs_robot_control->control_mode, 
	       PANEL_VALUE, "allready connected", NULL);
	return;
    }
    
    if (spawn_remote_task(task)) {
	xv_set(Xvmobs_robot_control->robocmd, 
	       PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_robot_control->remote_task, 
	       PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_robot_control->remote_spawn, 
	       PANEL_INACTIVE, TRUE, NULL);
	xv_set(Xvmobs_robot_control->remote_kill, 
	       PANEL_INACTIVE, FALSE, NULL);
	xv_set(Xvmobs_robot_control->control_mode, 
	       PANEL_VALUE, "spawning", NULL);
    }
    else {
	xv_set(Xvmobs_robot_control->control_mode, 
	       PANEL_VALUE, "spaw error", NULL);
	xv_set(Xvmobs_robot_control->remote_task, 
	       PANEL_INACTIVE, FALSE, NULL);
    }
}

/*
 * Notify callback function for `remote_kill'.
 */
void
xvmobs_robot_control_remote_kill_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
/*
	xvmobs_robot_control_objects *ip = (xvmobs_robot_control_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
*/
#ifdef GUI_DEBUG
	fputs("xvmobs: xvmobs_robot_control_remote_kill_notify_callback\n", stderr);
#endif

	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(Xvmobs_robot_control->remote_task, PANEL_INACTIVE, FALSE, NULL);
	
	xv_set(Xvmobs_robot_control->control_mode, PANEL_VALUE, "manual", NULL);
	
	remote_kill(item, event);
	
	xv_set(Xvmobs_robot_control->remote_spawn, PANEL_INACTIVE, FALSE, NULL);
	
	xv_set(Xvmobs_robot_control->remote_kill, PANEL_INACTIVE, TRUE, NULL);
	
	xv_set(Xvmobs_robot_control->robocmd, PANEL_INACTIVE, FALSE, NULL);
	
	/* gxv_end_connections */

}

/*
 * User-defined action for `remote_kill'.
 */
void
remote_kill(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: remote_kill\n", stderr);
#endif
	if (act_mob) {
	mw_object_kill_remote(act_mob);
    }
}

/*
 * Notify callback function for `editor_config_apply'.
 */
void
xvmobs_config_editor_window_editor_config_apply_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG
    fputs("xvmobs: xvmobs_config_editor_window_editor_config_apply_notify_callback\n", stderr);
#endif

	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	editor_config_apply(item, event);
	
	/* gxv_end_connections */

}

/*
 * User-defined action for `editor_config_apply'.
 */
void
editor_config_apply(item, event)
	Panel_item	item;
	Event		*event;
{
    long value;
#ifdef GUI_DEBUG
    fputs("xvmobs: editor_config_apply\n", stderr);
#endif

    value = xv_get(Xvmobs_config_editor_window->snap_coord, PANEL_VALUE);
    snap_position = ((double)value) / 1000.0;

    value = xv_get(Xvmobs_config_editor_window->snap_angle, PANEL_VALUE);
    snap_angle = ((double)value) * PI / 180.0;

    value = xv_get(Xvmobs_config_editor_window->center_radius, PANEL_VALUE);
    center_radius = value;

    value = xv_get(Xvmobs_config_editor_window->pick_height, PANEL_VALUE);
    pick_height = value;

    value = xv_get(Xvmobs_config_editor_window->mouse_snap, PANEL_VALUE);
    snap_grid = value ? FALSE : TRUE;
}

/*
 * Notify callback function for `env_config_apply'.
 */
void
xvmobs_config_env_window_env_config_apply_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG
    fputs("xvmobs: xvmobs_config_env_window_env_config_apply_notify_callback\n", stderr);
#endif
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	env_config_apply(item, event);
	
	/* gxv_end_connections */

}

/*
 * User-defined action for `env_config_apply'.
 */
void
env_config_apply(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG
    fputs("xvmobs: env_config_apply\n", stderr);
#endif
    long air_speed = xv_get(Xvmobs_config_env_window->air_speed, 
			    PANEL_VALUE);
    long air_damping = xv_get(Xvmobs_config_env_window->air_damping,
			      PANEL_VALUE);
    long reflexion_count = xv_get(Xvmobs_config_env_window->reflexion_count,
				  PANEL_VALUE);
    
    mob_world_set(world, MWS_AIR_SPEED, (double)air_speed);
    if (air_damping>0)
	mob_world_set(world, MWS_US_AIR_DAMPING, ((double)air_damping)/1000.0);
    else 
	mob_world_set(world, MWS_US_AIR_DAMPING, 0.0);
    mob_world_set(world, MWS_MAX_REFLEXION_COUNT, (double)reflexion_count);
}


/*
 * Notify callback function for `sim_config_apply'.
 */
void
xvmobs_config_sim_window_sim_config_apply_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG	
	fputs("xvmobs: xvmobs_config_sim_window_sim_config_apply_notify_callback\n", stderr);
#endif
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	sim_config_apply(item, event);
	
	/* gxv_end_connections */

}

/*
 * User-defined action for `sim_config_apply'.
 */
void
sim_config_apply(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG
    fputs("xvmobs: sim_config_apply\n", stderr);
#endif
    long collision_stepsize = xv_get(Xvmobs_config_sim_window->collision_rate, 
				     PANEL_VALUE);

    mob_world_set(world, MWS_COLLISION_STEP, ((double)collision_stepsize)/1000.0);
}


/*
 * User-defined action for `attention_continue'.
 */
void
new_world(item, event)
	Panel_item	item;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: new_world\n", stderr);
#endif
	mob_world old = world;

	world = mob_world_new();
	mob_world_free(old);

	act_mob = 0;
	act_obj = 0;
	list_remove_all(selected_objs);

	v2_set(window_center, 0.0, 0.0);
	frames_per_second = 10.0;
	pixels_per_meter = 100.0;
	
	repaint_display();
}


/*
 * User-defined action for `display_modes'.
 */
void
sync_mode_notify(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
#ifdef GUI_DEBUG
	fputs("xvmobs: sync_mode_notify\n", stderr);
#endif

	display_modes |= DF_SYNCHRON;
}

